package ec.edu.espe.movilidad.MovilidadWS.Controller;

import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavClasPre;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoClaspreClassPresupNombre;

import ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavClasPre.IServiceUzyTavClasPre;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

import static ec.edu.espe.movilidad.MovilidadWS.Constant.GlobalConstants.V1_API_VERSION;

@RestController
@CrossOrigin(origins = "http://localhost:4200")
@RequestMapping(V1_API_VERSION+"/claspre")
public class UzyTavClasPreController {


    private final IServiceUzyTavClasPre serviceUzyTavClasPre;

    public UzyTavClasPreController(IServiceUzyTavClasPre serviceUzyTavClasPre) {
        this.serviceUzyTavClasPre = serviceUzyTavClasPre;
    }


    @GetMapping("/exampleFindId/{id}")
    public ResponseEntity<DtoUzyTavClasPre> ListarPorID(@PathVariable Long id) {
        return new ResponseEntity<>(serviceUzyTavClasPre.ListarPorID(id), HttpStatus.OK);
    }

    @GetMapping("/obtenerClasificadorPresupNombre/{claspreID}")
    public ResponseEntity<List<DtoClaspreClassPresupNombre>> obtenerClasificadorPresupNombre(@PathVariable Long claspreID) {
        return new ResponseEntity<>(serviceUzyTavClasPre.obtenerClasificadorPresupNombre(claspreID), HttpStatus.OK);
    }


    @GetMapping("/getAll")
    public ResponseEntity<List<DtoUzyTavClasPre>> ListarRegistros() {
        return new ResponseEntity<>(serviceUzyTavClasPre.ListarRegistros(), HttpStatus.OK);

    }




    @GetMapping("/obtenerClasificadorPresup/{presupID}")
    public ResponseEntity<List<DtoUzyTavClasPre>> obtenerClasificadorPresup(@PathVariable Long presupID) {
        return new ResponseEntity<>(serviceUzyTavClasPre.obtenerClasificadorPresup(presupID), HttpStatus.OK);

    }

    @GetMapping("/obtenerTodosClasificadorPresupNombre")
    public ResponseEntity<List<DtoClaspreClassPresupNombre>> obtenerTodosClasificadorPresupNombre() {
        return new ResponseEntity<>(serviceUzyTavClasPre.obtenerTodosClasificadorPresupNombre(), HttpStatus.OK);
    }



    @PostMapping("/guardar")
    public ResponseEntity<DtoUzyTavClasPre> guardar(@Valid @RequestBody DtoUzyTavClasPre dtoUzyTavClasPre) {
        DtoUzyTavClasPre savedDto = serviceUzyTavClasPre.guardar(dtoUzyTavClasPre);
        return ResponseEntity.ok(savedDto);
    }




    @PutMapping("/editar/{id}")
    public ResponseEntity<DtoUzyTavClasPre> editar(@PathVariable Long id, @RequestBody DtoUzyTavClasPre dtoUzyTavClasPre) {
        return new  ResponseEntity<>(serviceUzyTavClasPre.editar(id, dtoUzyTavClasPre), HttpStatus.OK);
    }



    @DeleteMapping("/eliminar/{id}")
    public ResponseEntity<String> eliminar(@PathVariable Long id){
        boolean eliminado = serviceUzyTavClasPre.eliminar(id);
        if (eliminado) {
            return ResponseEntity.ok("El registro se eliminó exitosamente.");
        } else {
            return ResponseEntity.notFound().build();
        }
    }

}
