package ec.edu.espe.movilidad.MovilidadWS.Controller;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavParaEva;
import ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavParaEva.IServiceUzyTavParaEva;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import javax.validation.Valid;
import java.util.List;
import static ec.edu.espe.movilidad.MovilidadWS.Constant.GlobalConstants.V1_API_VERSION;

@RestController
@CrossOrigin(origins = "http://localhost:4200")
@RequestMapping(V1_API_VERSION + "/paraeva")
public class UzyTavParaEvaController {

    private final IServiceUzyTavParaEva serviceUzyTavParaEva;

    @Autowired
    public UzyTavParaEvaController(IServiceUzyTavParaEva serviceUzyTavParaEva) {
        this.serviceUzyTavParaEva = serviceUzyTavParaEva;
    }

    @GetMapping("/exampleFindId/{id}")
    public ResponseEntity<DtoUzyTavParaEva> obtenerPorID(@PathVariable Long id) {
        DtoUzyTavParaEva dto = serviceUzyTavParaEva.ListarPorID(id);
        return ResponseEntity.ok(dto);
    }

    @GetMapping("/datosRelacionadosConParaEva1/{id}")
    public ResponseEntity<List<DtoUzyTavParaEva>> obtenerDatosRelacionadosConParaEvaTipo1(@PathVariable("id") Long id) {
        List<DtoUzyTavParaEva> dtos = serviceUzyTavParaEva.findByIdDatosConParaEvaTipo1(id);
        return ResponseEntity.ok(dtos);
    }

    @GetMapping("/datosRelacionadosConParaEva2/{id}")
    public ResponseEntity<List<DtoUzyTavParaEva>> obtenerDatosRelacionadosConParaEvaTipo2(@PathVariable("id") Long id) {
        List<DtoUzyTavParaEva> dtos = serviceUzyTavParaEva.findByIdDatosConParaEvaTipo2(id);
        return ResponseEntity.ok(dtos);
    }

    @GetMapping("/datosRelacionadosCabParaEvaf/{id}")
    public ResponseEntity<List<DtoUzyTavParaEva>> obtenerPorDatosUzytavcabparaevafId(@PathVariable("id") Long id) {
        List<DtoUzyTavParaEva> dtos = serviceUzyTavParaEva.findByDatosUzytavcabparaevafId(id);
        return ResponseEntity.ok(dtos);
    }

    @GetMapping("/datosRelacionadosConParaEva")
    public ResponseEntity<List<DtoUzyTavParaEva>> obtenerTodosLosDatosConParaEva() {
        List<DtoUzyTavParaEva> datos = serviceUzyTavParaEva.getAllDatosConParaEva();
        return ResponseEntity.ok(datos);
    }

    @GetMapping("/getAll")
    public ResponseEntity<List<DtoUzyTavParaEva>> obtenerTodosLosRegistros() {
        List<DtoUzyTavParaEva> dtos = serviceUzyTavParaEva.ListarRegistros();
        return ResponseEntity.ok(dtos);
    }

    @PostMapping("/guardar")
    public ResponseEntity<DtoUzyTavParaEva> guardar(@Valid @RequestBody DtoUzyTavParaEva dtoUzyTavParaEva) {
        DtoUzyTavParaEva savedDto = serviceUzyTavParaEva.guardar(dtoUzyTavParaEva);
        return ResponseEntity.ok(savedDto);
    }

    @PutMapping("/editar/{id}")
    public ResponseEntity<DtoUzyTavParaEva> editar(@PathVariable Long id, @Valid @RequestBody DtoUzyTavParaEva dtoUzyTavParaEva) {
        DtoUzyTavParaEva editedDto = serviceUzyTavParaEva.editar(id, dtoUzyTavParaEva);
        return ResponseEntity.ok(editedDto);
    }

    @DeleteMapping("/eliminar/{id}")
    public ResponseEntity<Void> eliminar(@PathVariable Long id) {
        serviceUzyTavParaEva.eliminar(id);
        return ResponseEntity.ok().build();
    }
}