package ec.edu.espe.movilidad.MovilidadWS.Controller;

import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzytavObjetivoPrograma;
import ec.edu.espe.movilidad.MovilidadWS.Service.UzytavObjetivoPrograma.IServiceUzytavObjetivoPrograma;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

import static ec.edu.espe.movilidad.MovilidadWS.Constant.GlobalConstants.V1_API_VERSION;

@RestController
@CrossOrigin(origins = "http://localhost:4200")
@RequestMapping(V1_API_VERSION+"/objetivoprograma")
public class UzytavObjetivoProgramaController {

    private final IServiceUzytavObjetivoPrograma serviceUzytavObjetivoPrograma;

    public UzytavObjetivoProgramaController(IServiceUzytavObjetivoPrograma serviceUzytavObjetivoPrograma) {
        this.serviceUzytavObjetivoPrograma = serviceUzytavObjetivoPrograma;
    }

    @GetMapping("/exampleFindId/{id}")
    public ResponseEntity<DtoUzytavObjetivoPrograma> ListarPorID(@PathVariable Long id) {
        return new ResponseEntity<>(serviceUzytavObjetivoPrograma.ListarPorID(id), HttpStatus.OK);
    }

    @GetMapping("/getAll")
    public ResponseEntity<List<DtoUzytavObjetivoPrograma>> ListarRegistros() {
        return new ResponseEntity<>(serviceUzytavObjetivoPrograma.ListarRegistros(), HttpStatus.OK);
    }

    @GetMapping("/registrosRelacionadosConPrograma/{programaId}")
    public ResponseEntity<List<DtoUzytavObjetivoPrograma>> ListarRegistrosRelacionadosConPrograma(@PathVariable Long programaId) {
        return new ResponseEntity<>(serviceUzytavObjetivoPrograma.obtenerObjetivoprogramaPorPrograma(programaId), HttpStatus.OK);
    }

    @GetMapping("/registrosRelacionadosConProyecto/{proyectID}")
    public ResponseEntity<List<DtoUzytavObjetivoPrograma>> obtenerObjetivoprogramaPorProyecto(@PathVariable Long proyectID) {
        return new ResponseEntity<>(serviceUzytavObjetivoPrograma.obtenerObjetivoprogramaPorProyecto(proyectID), HttpStatus.OK);
    }

    @PostMapping("/guardar")
    public ResponseEntity<DtoUzytavObjetivoPrograma> guardar(@Valid @RequestBody DtoUzytavObjetivoPrograma dtoUzytavObjetivoPrograma) {
        DtoUzytavObjetivoPrograma savedDto = serviceUzytavObjetivoPrograma.guardar(dtoUzytavObjetivoPrograma);
        return ResponseEntity.ok(savedDto);
    }



    @PutMapping("/editar/{id}")
    public ResponseEntity<DtoUzytavObjetivoPrograma> editar(@PathVariable Long id, @RequestBody DtoUzytavObjetivoPrograma dtoUzytavObjetivoPrograma) {
        return new  ResponseEntity<>(serviceUzytavObjetivoPrograma.editar(id, dtoUzytavObjetivoPrograma), HttpStatus.OK);
    }


    @PutMapping("/eliminarFechasPorId/{id}")
    public ResponseEntity<DtoUzytavObjetivoPrograma> eliminarFechasPorId(@PathVariable Long id) {
        return new  ResponseEntity<>(serviceUzytavObjetivoPrograma.eliminarFechasPorId(id), HttpStatus.OK);
    }


    @DeleteMapping("/eliminar/{id}")
    public ResponseEntity<String> eliminar(@PathVariable Long id){
        boolean eliminado = serviceUzytavObjetivoPrograma.eliminar(id);
        if (eliminado) {
            return ResponseEntity.ok("El registro se eliminó exitosamente.");
        } else {
            return ResponseEntity.notFound().build();
        }
    }

    @DeleteMapping("/eliminarHijo/{id}")
    public ResponseEntity<String> eliminarHijo(@PathVariable Long id){
        boolean eliminado = serviceUzytavObjetivoPrograma.eliminarHijo(id);
        if (eliminado) {
            return ResponseEntity.ok("El registro se eliminó exitosamente.");
        } else {
            return ResponseEntity.notFound().build();
        }
    }
}

