package ec.edu.espe.movilidad.MovilidadWS.Exceptions.Configuration;

import ec.edu.espe.movilidad.MovilidadWS.Exceptions.DuplicateStateException;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ErrorResponse;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.InvalidArgumentException;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.http.HttpStatus;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.validation.FieldError;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.MissingPathVariableException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.http.ResponseEntity;

import org.springframework.web.bind.annotation.RestControllerAdvice;

import java.util.ArrayList;
import java.util.List;
import java.util.NoSuchElementException;

import org.springframework.dao.EmptyResultDataAccessException;

import org.springframework.http.converter.HttpMessageNotReadableException;


import java.io.IOException;
import java.sql.SQLException;

import java.util.concurrent.TimeoutException;

//Clase para capturar y manejar globalmente las excepciones producidas
@RestControllerAdvice
public class GlobalExceptionHandler {

    @ExceptionHandler(ResourceNotFoundException.class)
    public ResponseEntity<String> handleResourceNotFoundException(ResourceNotFoundException ex) {
        return ResponseEntity.status(HttpStatus.NOT_FOUND).body(ex.getMessage());
    }



    @ExceptionHandler(NoSuchElementException.class)
    public ResponseEntity<String> handleNoSuchElementException(NoSuchElementException ex) {
        return ResponseEntity.status(HttpStatus.NOT_FOUND).body(ex.getMessage());
    }

    @ExceptionHandler(InvalidArgumentException.class)
    public ResponseEntity<String> handleInvalidArgumentException(InvalidArgumentException ex) {
        return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(ex.getMessage());
    }

    @ExceptionHandler(RuntimeException.class)
    public ResponseEntity<String> handleRuntimeException(RuntimeException ex) {
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("Ha ocurrido un error inesperado en el servidor: ->  " + ex.getMessage());
    }

    //Excepción cuando falte el valor de una path variable
    @ExceptionHandler(MissingPathVariableException.class)
    public ResponseEntity<String> handleMissingPathVariableException(MissingPathVariableException ex) {
        String mensajeError = "Falta el valor del path variable: " + ex.getVariableName();
        return new ResponseEntity<>(mensajeError, HttpStatus.BAD_REQUEST);
    }

    @ExceptionHandler(Exception.class)
    public ResponseEntity<String> handleGenericException(Exception ex) {
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("Ha ocurrido un error INTERNO en el servidor: -> " + ex.getMessage());
    }

    //Se encarga de capturar y manejar las excepciones de validación
    @ExceptionHandler(MethodArgumentNotValidException.class)
    public ResponseEntity<ErrorResponse> handleValidationException(MethodArgumentNotValidException ex) {
        List<String> errors = new ArrayList<>(); //Se almacenan los mensajes producidos
        for (ObjectError error : ex.getBindingResult().getAllErrors()) { // Se itera todos los errores de la excepcion
            if (error instanceof FieldError) {
                FieldError fieldError = (FieldError) error;
                String errorMessage = fieldError.getField() + ": " + error.getDefaultMessage();
                errors.add(errorMessage);
            } else {
                errors.add(error.getDefaultMessage());
            }
        }
        ErrorResponse errorResponse = new ErrorResponse("Error de validación", errors);
        return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(errorResponse);
    }

    ////Restricción única o una clave foránea que no se cumple al realizar operaciones de persistencia en la base de datos como :@Column(unique = true).
    @ExceptionHandler(DataIntegrityViolationException.class)
    public ResponseEntity<String> handleDataIntegrityViolationException(DataIntegrityViolationException ex) {
        return ResponseEntity.status(HttpStatus.BAD_REQUEST)
                .body("Error de integridad de datos: " + ex.getMessage());
    }
    //Restricción única o una clave foránea que no se cumple al realizar operaciones de persistencia en la base de datos Como por ejemplo un ID que no se encuentre en la BDD.
    @ExceptionHandler(EmptyResultDataAccessException.class)
    public ResponseEntity<String> handleEmptyResultDataAccessException(EmptyResultDataAccessException ex) {
        return ResponseEntity.status(HttpStatus.NOT_FOUND)
                .body(ex.getMessage());
    }
    //Manejar excepcion cuando hay problemas en las consultas o en la conexión con la base de datos.

    @ExceptionHandler(SQLException.class)
    public ResponseEntity<String> handleSQLException(SQLException ex) {
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR)
                .body("Error en la base de datos: " + ex.getMessage());
    }

    //Excepcion cuando el body no se ajusta al formato esperado
    @ExceptionHandler(HttpMessageNotReadableException.class)
    public ResponseEntity<String> handleHttpMessageNotReadableException(HttpMessageNotReadableException ex) {
        return ResponseEntity.status(HttpStatus.BAD_REQUEST)
                .body("Error al leer el cuerpo de la solicitud: " + ex.getMessage());
    }

    //Maneja errores relacionados con operaciones de entrada/salida (I/O), como cuando se hace una carga de algún archivo
    @ExceptionHandler(IOException.class)
    public ResponseEntity<String> handleIOException(IOException ex) {
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR)
                .body("Error de entrada/salida: " + ex.getMessage());
    }
    @ExceptionHandler(UsernameNotFoundException.class)
    public ResponseEntity<String> handleUsernameNotFoundException(UsernameNotFoundException ex) {
        return ResponseEntity.status(HttpStatus.NOT_FOUND).body("Usuario no encontrado");
    }
    @ExceptionHandler(DuplicateStateException.class)
    public ResponseEntity<String> handleDuplicateStateException(DuplicateStateException ex) {
        return ResponseEntity.status(HttpStatus.CONFLICT).body(ex.getMessage());
    }

    //Se maneja la excepcion para se intenta acceder a un objeto nullo
    @ExceptionHandler(NullPointerException.class)
    public ResponseEntity<String> handleNullPointerException(NullPointerException ex) {
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR)
                .body("Error de puntero nulo: " + ex.getMessage());
    }
    //Excepcion cuando no se obtiene el formato numérico

    @ExceptionHandler(NumberFormatException.class)
    public ResponseEntity<String> handleNumberFormatException(NumberFormatException ex) {
        return ResponseEntity.status(HttpStatus.BAD_REQUEST)
                .body("Error al convertir una cadena a número: " + ex.getMessage());
    }

    //Excepción cuando se sobrepasa el límite de espera de alguna solicitud.
    @ExceptionHandler(TimeoutException.class)
    public ResponseEntity<String> handleTimeoutException(TimeoutException ex) {
        return ResponseEntity.status(HttpStatus.REQUEST_TIMEOUT)
                .body("Tiempo de espera agotado: " + ex.getMessage());
    }
}


