package ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class;

import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavObjetivoBVivir;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavProgObj;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavObjetivoBVivir;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavProgObj;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavPrograma_V;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavProyec;
import org.modelmapper.ModelMapper;
import org.modelmapper.PropertyMap;
import org.springframework.stereotype.Component;

import javax.persistence.EntityManager;
import java.util.List;
import java.util.stream.Collectors;

@Component
public class UzyTavProgObjMapper {
    private final ModelMapper modelMapper;
    private final EntityManager entityManager;

    public UzyTavProgObjMapper(ModelMapper modelMapper, EntityManager entityManager) {
        this.modelMapper = modelMapper;
        this.entityManager = entityManager;
        configureMappings();
    }

    public DtoUzyTavProgObj entityToDto(ModelUzyTavProgObj entity) {
        DtoUzyTavProgObj dto = modelMapper.map(entity, DtoUzyTavProgObj.class);
        if (entity.getUzytavobjetivo_b_vivir() != null) {
            DtoUzyTavObjetivoBVivir objetivoBVivirDto = objetivoBVivirToDto(entity.getUzytavobjetivo_b_vivir());
            dto.setObjetivoBVivir(objetivoBVivirDto);
        }
        return dto;
    }

    private DtoUzyTavObjetivoBVivir objetivoBVivirToDto(ModelUzyTavObjetivoBVivir objetivoBVivirEntity) {
        return modelMapper.map(objetivoBVivirEntity, DtoUzyTavObjetivoBVivir.class);
    }

    public ModelUzyTavProgObj dtoToEntity(DtoUzyTavProgObj dto) {
        ModelUzyTavProgObj entity = modelMapper.map(dto, ModelUzyTavProgObj.class);
        Long buenvivirID = dto.getUzytavobjetivo_b_vivir_id();
        Long programaID = dto.getUzytavprograma_v_id();
        Long proyectID = dto.getUzytavproyec_id();
        if(buenvivirID !=null){
            ModelUzyTavObjetivoBVivir buenvivir = entityManager.find(ModelUzyTavObjetivoBVivir.class, buenvivirID);
            entity.setUzytavobjetivo_b_vivir(buenvivir);
        }
        if(programaID !=null){
            ModelUzyTavPrograma_V programaV = entityManager.find(ModelUzyTavPrograma_V.class, programaID);
            entity.setUzytavprograma_v(programaV);
        }
        if(proyectID !=null){
            ModelUzyTavProyec proyec = entityManager.find(ModelUzyTavProyec.class, proyectID);
            entity.setUzytavproyec(proyec);
        }
        return entity;
    }

    public List<DtoUzyTavProgObj> entitiesToDtos(List<ModelUzyTavProgObj> entities) {
        return entities.stream()
                .map(this::entityToDto)
                .collect(Collectors.toList());
    }

    private void configureMappings() {
        modelMapper.addMappings(new PropertyMap<ModelUzyTavProgObj, DtoUzyTavProgObj>() {
            @Override
            protected void configure() {
                map().setUzytavprog_obj_code(source.getUzytavprog_obj_code());
                map().setUzytavprograma_v_id(source.getUzytavprograma_v().getUzytavprograma_v_id());
                map().setUzytavobjetivo_b_vivir_id(source.getUzytavobjetivo_b_vivir().getUzytavobjetivo_b_vivir_id());
                map().setUzytavproyec_id(source.getUzytavproyec().getUzytavproyec_id());

            }
        });
    }
}
