package ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class;

import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavProyec;
import ec.edu.espe.movilidad.MovilidadWS.Model.*;
import org.modelmapper.ModelMapper;
import org.modelmapper.PropertyMap;
import org.springframework.stereotype.Component;

import javax.persistence.EntityManager;
import java.util.List;
import java.util.stream.Collectors;

@Component
public class UzyTavProyecMapper {
    private final ModelMapper modelMapper;
    private final EntityManager entityManager;

    public UzyTavProyecMapper(ModelMapper modelMapper, EntityManager entityManager) {
        this.modelMapper = modelMapper;
        this.entityManager = entityManager;
        configureMappings();
    }

    public DtoUzyTavProyec entityToDto(ModelUzyTavProyec entity) {
        return modelMapper.map(entity, DtoUzyTavProyec.class);
    }

    public ModelUzyTavProyec dtoToEntity(DtoUzyTavProyec dto) {
        ModelUzyTavProyec entity = modelMapper.map(dto, ModelUzyTavProyec.class);

        Long programaVID = dto.getUzytavprograma_v_id();
        Long convocaID = dto.getUzytavconvoca_id();
        Long cabcomcaID = dto.getUzytavcabcomca_id();
        Long conparaevaID = dto.getUzytavconparaeva_id();
        Long detID = dto.getUzytparametros_det_id();
        Long detidCobertuda = dto.getUzytparametros_det_id_cobertura();
        Long actaApruebaID = dto.getUzytavactaprueba_id();
        Long catGenID = dto.getUzytcata_gen_id();
        Long userID = dto.getUzytusuario_id();
        entity.setUzytavprograma_v(programaVID != null ? entityManager.getReference(ModelUzyTavPrograma_V.class, programaVID) : null);
        entity.setUzytavconvoca(convocaID != null ? entityManager.getReference(ModelUzyTavConvoca.class, convocaID) : null);
        entity.setUzytavcabcomca(cabcomcaID != null ? entityManager.getReference(ModelUzyTavCabComca.class, cabcomcaID) : null);
        entity.setUzytavconparaeva(conparaevaID != null ? entityManager.getReference(ModelUzyTavConparaEva.class, conparaevaID) : null);
        entity.setUzytparametros_detID(detID != null ? entityManager.getReference(ModelUzyTavParametros_Det.class, detID) : null);
        entity.setUzytparametros_detCobertura(detidCobertuda != null ? entityManager.getReference(ModelUzyTavParametros_Det.class, detidCobertuda) : null);
        entity.setUzyTavacAprueba(actaApruebaID != null ? entityManager.getReference(ModelUzyTavActaPrueba.class, actaApruebaID) : null);
        entity.setUzytcataGen(catGenID != null ? entityManager.getReference(ModelUzyTCatalogosGenerales.class, catGenID) : null);
        entity.setUzytusuario(userID != null ? entityManager.getReference(ModelUzyTUsuario.class, userID) : null);

        return entity;
    }

    public List<DtoUzyTavProyec> entitiesToDtos(List<ModelUzyTavProyec> entities) {
        return entities.stream()
                .map(this::entityToDto)
                .collect(Collectors.toList());
    }

    private void configureMappings() {
        modelMapper.addMappings(new PropertyMap<ModelUzyTavProyec, DtoUzyTavProyec>() {
            @Override
            protected void configure() {
                map().setUzytavproyec_id(source.getUzytavproyec_id());
                map().setUzytavprograma_v_id(source.getUzytavprograma_v().getUzytavprograma_v_id());
                map().setUzytavconvoca_id(source.getUzytavconvoca().getUzytavconvoca_id());
                map().setUzytavcabcomca_id(source.getUzytavcabcomca().getUzytavcabcomca_id());
                map().setUzytavconparaeva_id(source.getUzytavconparaeva().getUzytavconparaeva_id());
                map().setUzytparametros_det_id(source.getUzytparametros_detID().getUzytparametros_det_id());
                map().setUzytparametros_det_id_cobertura(source.getUzytparametros_detCobertura().getUzytparametros_det_id());
                map().setUzytcata_gen_id(source.getUzytcataGen().getUzytcata_gen_id());
                map().setUzytusuario_id(source.getUzytusuario().getUzytusuario_id());

            }
        });
    }
}
