package ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class;

import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzytavObjetivoPrograma;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavPrograma_V;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavProyec;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzytavObjetivoPrograma;
import org.modelmapper.ModelMapper;
import org.modelmapper.PropertyMap;
import org.springframework.stereotype.Component;

import javax.persistence.EntityManager;
import java.util.List;
import java.util.stream.Collectors;


@Component
public class UzytavObjetivoProgramaMapper {

    private final ModelMapper modelMapper;
    private final EntityManager entityManager;

    public UzytavObjetivoProgramaMapper(ModelMapper modelMapper, EntityManager entityManager) {
        this.modelMapper = modelMapper;
        this.entityManager = entityManager;
        configureMappings();
    }

    public DtoUzytavObjetivoPrograma entityToDto(ModelUzytavObjetivoPrograma entity) {
        DtoUzytavObjetivoPrograma dto = modelMapper.map(entity, DtoUzytavObjetivoPrograma.class);
        // Mapear el padre para que traiga todos los regitros
        if (entity.getUzytavobjetivo_programaPadre() != null) {
            DtoUzytavObjetivoPrograma dtoPadre = entityToDto(entity.getUzytavobjetivo_programaPadre());
            dto.setUzytavobjetivo_programaPadre(dtoPadre);
        }
        return dto;
    }


    public ModelUzytavObjetivoPrograma dtoToEntity(DtoUzytavObjetivoPrograma dto) {
        ModelUzytavObjetivoPrograma entity = modelMapper.map(dto, ModelUzytavObjetivoPrograma.class);
        Long programaID = dto.getUzytavprograma_v_id();
        Long proyectoID = dto.getUzytavproyec_id();
        Long padreID = dto.getUzytavobjetivo_programa_id_padre();
        //Enviar el objeto completo
        if (dto.getUzytavobjetivo_programaPadre() != null) {
            ModelUzytavObjetivoPrograma padreEntity = dtoToEntity(dto.getUzytavobjetivo_programaPadre());
            entity.setUzytavobjetivo_programaPadre(padreEntity);
        }
        if (programaID != null) {
            ModelUzyTavPrograma_V programa_v = entityManager.find(ModelUzyTavPrograma_V.class, programaID);
            entity.setUzytavprograma_v(programa_v);
        }
        if (proyectoID != null) {
            ModelUzyTavProyec proyec = entityManager.find(ModelUzyTavProyec.class, proyectoID);
            entity.setUzytavproyec(proyec);
        }
        //enviar solo el ID
        if (padreID != null) {
            ModelUzytavObjetivoPrograma objetivoPrograma = entityManager.find(ModelUzytavObjetivoPrograma.class, padreID);
            entity.setUzytavobjetivo_programaPadre(objetivoPrograma);
        }
        return entity;
    }


    public List<DtoUzytavObjetivoPrograma> entitiesToDtos(List<ModelUzytavObjetivoPrograma> entities) {
        return entities.stream()
                .map(this::entityToDto)
                .collect(Collectors.toList());
    }

    private void configureMappings() {
        modelMapper.addMappings(new PropertyMap<ModelUzytavObjetivoPrograma, DtoUzytavObjetivoPrograma>() {
            @Override
            protected void configure() {
                map().setUzytavobjetivo_programa_id(source.getUzytavobjetivo_programa_id());
                map().setUzytavobjetivo_programa_id_padre(source.getUzytavobjetivo_programaPadre().getUzytavobjetivo_programa_id());
            }
        });
    }


}
