package ec.edu.espe.movilidad.MovilidadWS.Security;
import com.fasterxml.jackson.databind.ObjectMapper;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUserLogeado;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.web.authentication.WebAuthenticationDetailsSource;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;




public class JwtAuthenticationFilter extends OncePerRequestFilter {

    @Autowired
    JwtUserDetailsService jwtUserDetailsService;

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain) throws ServletException, IOException {
        // Obtener el correo electrónico del encabezado
        String email = request.getHeader("email");
        //String Token = request.getHeader("Authorization"); loggerAuth.info("Token válido obtenido en el backend:  {}", Token);

        if (email != null && SecurityContextHolder.getContext().getAuthentication() == null) {
            try {
                // Autenticar al usuario por correo electrónico
                UserDetails userDetails = jwtUserDetailsService.loadUserByUsername(email);
                if (userDetails != null) {
                    UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(
                            userDetails, null, userDetails.getAuthorities());
                    authenticationToken.setDetails(new WebAuthenticationDetailsSource().buildDetails(request));
                    SecurityContextHolder.getContext().setAuthentication(authenticationToken);
                }
            } catch (UsernameNotFoundException ex) {
                // Manejar la excepción y enviar una respuesta adecuada
                response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
                response.getWriter().write("El usuario no es correcto");
                response.getWriter().flush();
                return;
            }
        }

        if (request.getRequestURI().equals("/api/v1.0/usuarioLogeado")) {

            String username = SecurityContextHolder.getContext().getAuthentication().getName(); //Se obtiene el nombre del usuario autenticado
            DtoUserLogeado userInfo = jwtUserDetailsService.getUserInfoByUsername(username);
            //Se devuelve los datos en formato JSON
            ObjectMapper objectMapper = new ObjectMapper();
            String jsonResponse = objectMapper.writeValueAsString(userInfo);
            response.setContentType("application/json");
            response.getWriter().write(jsonResponse);
            return;
        }

        filterChain.doFilter(request, response);
    }
}
