package ec.edu.espe.movilidad.MovilidadWS.Security;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTUsuario;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUserLogeado;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTPerfil;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTPerfilMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTPerfil;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTUsuario;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Service
public class JwtUserDetailsService implements UserDetailsService {

    private final DaoUzyTUsuario userRepository;

    private final  UzyTPerfilMapper perfilMapper;

    @Autowired
    public JwtUserDetailsService(DaoUzyTUsuario userRepository, UzyTPerfilMapper perfilMapper) {
        this.userRepository = userRepository;
        this.perfilMapper = perfilMapper;
    }

    @Override
    public UserDetails loadUserByUsername(String email) throws UsernameNotFoundException {
        ModelUzyTUsuario user = userRepository.findByEmail(email);
        if (user == null) {
            throw new UsernameNotFoundException("Usuario no encontrado");
        }
        List<GrantedAuthority> authorities = new ArrayList<>();
        for (ModelUzyTPerfil perfil : user.getUzytperfils()) {
            // Aquí puedes mapear tus roles o permisos a objetos GrantedAuthority
            String roleName = "ROLE_" + perfil.getUzytperfil_nombre();
            authorities.add(new SimpleGrantedAuthority(roleName));

            // Imprimir los roles en la consola
            System.out.println("Rol asignado: " + roleName);
            // Puedes agregar más lógica para manejar permisos específicos si es necesario
        }
        return new User(
                user.getUzytusuario_email_institucional(),
                "{noop}",
                authorities
        );
    }

    public DtoUserLogeado getUserInfoByUsername(String username) {
        ModelUzyTUsuario user = userRepository.findByEmail(username);
        if (user == null) {
            throw new UsernameNotFoundException("Usuario no encontrado");
        }

        DtoUserLogeado userInfo = new DtoUserLogeado();
        userInfo.setUzytusuario_id(user.getUzytusuario_id());
        userInfo.setUzytusuario_clave(user.getUzytusuario_clave());
        userInfo.setUzytusuario_nombres(user.getUzytusuario_nombres());
        userInfo.setUzytusuario_email(user.getUzytusuario_email());
        userInfo.setUzytusuario_email_institucional(user.getUzytusuario_email_institucional());
        userInfo.setUzytusuario_apellidos(user.getUzytusuario_apellidos());
        userInfo.setUzytusuario_telefono_celular(user.getUzytusuario_telefono_celular());
        userInfo.setUzytusuario_telefono_fijo(user.getUzytusuario_telefono_fijo());
        userInfo.setSpriden_id(user.getSpriden_id());
        userInfo.setUzytusuario_CAMPUS_CODE(user.getUzytusuario_CAMPUS_CODE());
        userInfo.setUzytusuario_DEPARTAMENTO_CODE(user.getUzytusuario_DEPARTAMENTO_CODE());
        userInfo.setUzytusuario_CAMPUS_ABREV_CODE(user.getUzytusuario_CAMPUS_ABREV_CODE());
        userInfo.setUzytusuario_DPTO_ABREV_CODE(user.getUzytusuario_DPTO_ABREV_CODE());
        // Obtener el perfil asociado al usuario
        Set<ModelUzyTPerfil> perfiles = user.getUzytperfils();
        Set<DtoUzyTPerfil> dtoPerfiles = new HashSet<>();
        for (ModelUzyTPerfil perfil : perfiles) {
            DtoUzyTPerfil dtoPerfil = perfilMapper.entityToDto(perfil);
            dtoPerfiles.add(dtoPerfil);
        }
        userInfo.setPerfiles(dtoPerfiles);

        return userInfo;
    }


}

