package ec.edu.espe.movilidad.MovilidadWS.Service.FileStorage;

import ec.edu.espe.movilidad.MovilidadWS.Exceptions.FileStorageException;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.nio.file.*;

@Service
public class FileStorageService implements IFileStorageService {

    private final Path rootFolder = Paths.get("uploadsPDF");

    private final Path rootFolderIMG = Paths.get("uploadsIMG");

    private static final String FILE_ALREADY_EXISTS_MESSAGE = "El archivo ya existe: ";


    @Override
    public String saveFile(MultipartFile file, String fileName, boolean overwrite) {
        try {
            Path filePath = rootFolder.resolve(fileName);

            // Verificar si el archivo ya existe y si se permite la sobrescritura
            if (Files.exists(filePath) && !overwrite) {
                throw new FileAlreadyExistsException(FILE_ALREADY_EXISTS_MESSAGE + fileName);
            }

            // Eliminar el archivo anterior si existe
            if (Files.exists(filePath)) {
                Files.delete(filePath);
            }

            // Guardar el archivo, reemplazando si es necesario
            try (InputStream inputStream = file.getInputStream()) {
                Files.copy(inputStream, filePath, StandardCopyOption.REPLACE_EXISTING);
            }

            return filePath.toString();
        } catch (FileAlreadyExistsException e) {
            throw new ResourceNotFoundException(FILE_ALREADY_EXISTS_MESSAGE + fileName + e);
        } catch (IOException e) {
            throw new FileStorageException("Failed to save file: " + fileName, e);
        }
    }

    @Override
    public String moveFile(String sourceFilePath, String targetFileName, boolean overwrite) throws IOException {
        Path sourcePath = Paths.get(sourceFilePath);
        Path targetPath = rootFolder.resolve(targetFileName);

        if (Files.exists(targetPath) && !overwrite) {
            throw new FileAlreadyExistsException(FILE_ALREADY_EXISTS_MESSAGE + targetFileName);
        }

        // Eliminar el archivo objetivo si existe y se permite la sobrescritura
        if (Files.exists(targetPath)) {
            Files.delete(targetPath);
        }

        // Mover el archivo de la ubicación de origen a la ubicación de destino
        Files.move(sourcePath, targetPath, StandardCopyOption.REPLACE_EXISTING);

        return targetPath.toString();
    }

    @Override
    public String saveFileIMG(MultipartFile file, String fileName, boolean overwrite) {
        try {
            Path filePath = rootFolderIMG.resolve(fileName);

            // Verificar si el archivo ya existe y si se permite la sobrescritura
            if (Files.exists(filePath) && !overwrite) {
                throw new FileAlreadyExistsException("FILE_ALREADY_EXISTS_MESSAGE " + fileName);
            }

            // Eliminar el archivo anterior si existe
            if (Files.exists(filePath)) {
                Files.delete(filePath);
            }

            // Guardar el archivo, reemplazando si es necesario
            try (InputStream inputStream = file.getInputStream()) {
                Files.copy(inputStream, filePath, StandardCopyOption.REPLACE_EXISTING);
            }

            return filePath.toString();
        } catch (FileAlreadyExistsException e) {
            throw new ResourceNotFoundException(FILE_ALREADY_EXISTS_MESSAGE + fileName + e);
        } catch (IOException e) {
            throw new FileStorageException("Failed to save file" + fileName, e);
        }
    }

    @Override
    public String moveFileIMG(String sourceFilePath, String targetFileName, boolean overwrite) throws IOException {
        Path sourcePath = Paths.get(sourceFilePath);
        Path targetPath = rootFolderIMG.resolve(targetFileName);

        if (Files.exists(targetPath) && !overwrite) {
            throw new FileAlreadyExistsException(FILE_ALREADY_EXISTS_MESSAGE + targetFileName);
        }

        // Eliminar el archivo objetivo si existe y se permite la sobrescritura
        if (Files.exists(targetPath)) {
            Files.delete(targetPath);
        }

        // Mover el archivo de la ubicación de origen a la ubicación de destino
        Files.move(sourcePath, targetPath, StandardCopyOption.REPLACE_EXISTING);

        return targetPath.toString();
    }


}