package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTCanton;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTCanton;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTCanton;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTProvincia;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTCantonMapper;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTProvinciaMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTCanton;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTProvincia;
import org.springframework.stereotype.Service;
import java.util.List;


@Service
public class ServiceUzyTCanton implements IServiceUzyTCanton {

    private final DaoUzyTCanton daoUzyTCanton;
    private final UzyTCantonMapper mapper;
    private final UzyTProvinciaMapper mapperProvincia;

    public ServiceUzyTCanton(DaoUzyTCanton daoUzyTCanton, UzyTCantonMapper mapper, UzyTProvinciaMapper mapperProvincia) {
        this.daoUzyTCanton = daoUzyTCanton;
        this.mapper = mapper;
        this.mapperProvincia = mapperProvincia;
    }

    @Override
    public DtoUzyTCanton ListarPorID(String id) {
        try {
            ModelUzyTCanton entity = daoUzyTCanton.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
            return mapper.entityToDto(entity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) { // Manejo de cualquier error en el servidor
            throw new RuntimeException("Error al buscar el canton con ID: " + id);
        }
    }

    @Override
    public List<DtoUzyTCanton> ListarRegistros() {
        try {
            List<ModelUzyTCanton> entities = daoUzyTCanton.findAll();
            return mapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar registros de canton: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTCanton guardar(DtoUzyTCanton dtoUzyTCanton) {
        try {
            ModelUzyTCanton entity = mapper.dtoToEntity(dtoUzyTCanton);
            ModelUzyTCanton nuevoEntity = daoUzyTCanton.save(entity);
            return mapper.entityToDto(nuevoEntity);
        } catch (Exception ex) {
            throw new RuntimeException("Error al guardar la provincia: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTCanton editar(String id, DtoUzyTCanton dtoUzyTCanton) {
        try {
            ModelUzyTCanton entity = daoUzyTCanton.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("Canton no encontrado con ID: " + id));
            entity.setUzytcantonnombre(dtoUzyTCanton.getUzytcantonnombre());
            entity.setUzytcantonbanner(dtoUzyTCanton.getUzytcantonbanner());
            entity.setUzytcantonGrupo(dtoUzyTCanton.getUzytcantongrupo());
            entity.setUzytcantonLatitud(dtoUzyTCanton.getUzytcantonlatitud());
            entity.setUzytcantonLongitud(dtoUzyTCanton.getUzytcantonlongitud());
            ModelUzyTCanton updatedEntity = daoUzyTCanton.save(entity);
            return mapper.entityToDto(updatedEntity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        }
        catch (Exception e) {
            throw new ResourceNotFoundException("Error al editar el registro: " + e.getMessage());
        }
    }

    @Override
    public List<DtoUzyTCanton> findByIdDatosConProvincia(String id) {
        try {
            List<ModelUzyTCanton> entities = daoUzyTCanton.findByIdDatosConProvincia(id) ;
            return mapper.entitiesToDtos(entities);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) { // Manejo de cualquier error en el servidor
            throw new RuntimeException("Error al buscar los registros con el ID: " + id);
        }
    }


    @Override
    public DtoUzyTProvincia findCantonByCantonId(String uzytcanton_id) {
        try {
            ModelUzyTProvincia entity =  daoUzyTCanton.findCantonByCantonId(uzytcanton_id);
            return mapperProvincia.entityToDto(entity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) { // Manejo de cualquier error en el servidor
            throw new RuntimeException("Error al buscar el registro con ID: " + uzytcanton_id);
        }
    }


    @Override
    public boolean eliminar(String id) {
        try {
            ModelUzyTCanton entity = daoUzyTCanton.findById(id).orElseThrow(() -> new ResourceNotFoundException("Canton no encontrado con ID: " + id));
            if (entity != null) {
                daoUzyTCanton.delete(entity);
                return true;
            }
            return false;
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al eliminar el canton: " + ex.getMessage());
        }
    }
}
