package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTCatalogosGenerales;
import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTCatalogosGenerales;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTCatalogosGenerales;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTCatalogosGeneralesMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTCatalogosGenerales;
import org.springframework.stereotype.Service;

import java.util.List;
@Service
public class ServiceUzyTCatalogosGenerales implements IServiceUzyTCatalogosGenerales{

    private final DaoUzyTCatalogosGenerales daoUzyTCatalogosGenerales;
    private final UzyTCatalogosGeneralesMapper mapper;

    public ServiceUzyTCatalogosGenerales(DaoUzyTCatalogosGenerales daoUzyTCatalogosGenerales, UzyTCatalogosGeneralesMapper mapper) {
        this.daoUzyTCatalogosGenerales = daoUzyTCatalogosGenerales;
        this.mapper = mapper;
    }

    @Override
    public DtoUzyTCatalogosGenerales ListarPorID(Long id) {
        if (id <= 0) {
            throw new IllegalArgumentException("El parámetro 'id' debe ser un valor positivo.");
        }
        try {
            ModelUzyTCatalogosGenerales entity = daoUzyTCatalogosGenerales.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
            return mapper.entityToDto(entity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) { // Manejo de cualquier error en el servidor
            throw new RuntimeException("Error al buscar el registro con ID: " + id);
        }
    }
    @Override
    public List<DtoUzyTCatalogosGenerales> ListarRegistros() {
        try {
            List<ModelUzyTCatalogosGenerales> entities = daoUzyTCatalogosGenerales.findAll();
            return mapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar los registros: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTCatalogosGenerales guardar(DtoUzyTCatalogosGenerales dtoUzyTCatalogosGenerales) {
        try {
            ModelUzyTCatalogosGenerales entity = mapper.dtoToEntity(dtoUzyTCatalogosGenerales);
            ModelUzyTCatalogosGenerales nuevoEntity = daoUzyTCatalogosGenerales.save(entity);
            return mapper.entityToDto(nuevoEntity);
        }
        catch (Exception ex) {
            throw new RuntimeException("Error al guardar el registro: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTCatalogosGenerales editar(Long id, DtoUzyTCatalogosGenerales dtoUzyTCatalogosGenerales) {
        if (id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
        ModelUzyTCatalogosGenerales entity = daoUzyTCatalogosGenerales.findById(id).get();
            if (entity != null) {
                if(dtoUzyTCatalogosGenerales.getUzytcata_gen_tipo()!=null){
                    entity.setUzytcata_gen_tipo(dtoUzyTCatalogosGenerales.getUzytcata_gen_tipo());
                }
                if(dtoUzyTCatalogosGenerales.getUzytcata_gen_nombre()!=null){
                    entity.setUzytcata_gen_nombre(dtoUzyTCatalogosGenerales.getUzytcata_gen_nombre());
                }
                if(dtoUzyTCatalogosGenerales.getUzytcata_gen_estado()!=null){
                    entity.setUzytcata_gen_estado(dtoUzyTCatalogosGenerales.getUzytcata_gen_estado());
                }

                ModelUzyTCatalogosGenerales updatedEntity = daoUzyTCatalogosGenerales.save(entity);
                return mapper.entityToDto(updatedEntity);
            }else {
                throw new ResourceNotFoundException("No se encontró el registro con ID: " + id);
            }
        } catch (Exception e) {
            throw new ResourceNotFoundException("Error al editar el registro: " + e.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        if (id == null || id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {

            ModelUzyTCatalogosGenerales entity = daoUzyTCatalogosGenerales.findById(id).orElseThrow(() -> new ResourceNotFoundException("Registro no encontrado con ID: " + id));
            if (entity != null) {
                daoUzyTCatalogosGenerales.delete(entity);
                return true;
            }
            return false;
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al eliminar el registro: " + ex.getMessage());
        }
    }
}
