package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTMenu;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTMenu;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTMenu;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.InvalidArgumentException;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTMenuMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTMenu;
import org.springframework.stereotype.Service;
import java.util.List;

@Service
public class ServiceUzyTMenu implements IServiceUzyTMenu {
    private final DaoUzyTMenu daoUzyTMenu;
    private final UzyTMenuMapper uzyTMenuMapper;

    public ServiceUzyTMenu(DaoUzyTMenu daoUzyTMenu, UzyTMenuMapper uzyTMenuMapper) {
        this.daoUzyTMenu = daoUzyTMenu;
        this.uzyTMenuMapper = uzyTMenuMapper;
    }

    @Override
    public DtoUzyTMenu ListarPorID(Long id) {
        if (id <= 0) {
            throw new InvalidArgumentException("El parámetro 'id' debe ser un valor positivo.");
        }
        try {
            ModelUzyTMenu entity = daoUzyTMenu.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
            return uzyTMenuMapper.entityToDto(entity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) { // Manejo de cualquier error en el servidor
            throw new RuntimeException("Error al buscar el registro con ID: " + id);
        }
    }

    @Override
    public List<DtoUzyTMenu> ListarRegistros() {
        try {
            List<ModelUzyTMenu> entities = daoUzyTMenu.findAll();
            return uzyTMenuMapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar registros de menú: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTMenu guardar(DtoUzyTMenu dtoUzyTMenu) {
        try {
            ModelUzyTMenu entity = uzyTMenuMapper.dtoToEntity(dtoUzyTMenu);
            ModelUzyTMenu savedEntity = daoUzyTMenu.save(entity);
            return uzyTMenuMapper.entityToDto(savedEntity);
        } catch (Exception ex) {
            throw new RuntimeException("Error al guardar el registro: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTMenu editar(Long id, DtoUzyTMenu dtoUzyTMenu) {
        if (id <= 0) {
            throw new InvalidArgumentException("El ID del menú debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTMenu entity = daoUzyTMenu.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("Registro no encontrado con ID: " + id));
            if (dtoUzyTMenu.getUzytmenuNombre() != null) {
                // Actualizar los campos de la entidad con los valores del DTO
                entity.setUzytmenuNombre(dtoUzyTMenu.getUzytmenuNombre());
            }
            ModelUzyTMenu updatedEntity = daoUzyTMenu.save(entity);
            return uzyTMenuMapper.entityToDto(updatedEntity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al editar el registro -> " + ex.getMessage());
        }
    }


    @Override
    public boolean eliminar(Long id) {
        if (id == null || id <= 0) {
            throw new InvalidArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {

            ModelUzyTMenu entity = daoUzyTMenu.findById(id).orElseThrow(() -> new ResourceNotFoundException("Registro no encontrado con ID: " + id));
            if (entity != null) {
                daoUzyTMenu.delete(entity);
                return true;
            }
            return false;
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al eliminar el registro: " + ex.getMessage());
        }
    }
}
