package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTParroquia;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTParroquia;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoParroquiaCantonProvincia;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTCanton;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTParroquia;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTCantonMapper;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTParroquiaMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTCanton;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTParroquia;

import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PathVariable;

import javax.transaction.Transactional;
import java.util.List;

@Service
@Transactional
public class ServiceUzyTParroquia implements IServiceUzyTParroquia {

    private final DaoUzyTParroquia daoUzyTParroquia;

    private final UzyTCantonMapper mapperCanton;
    private final UzyTParroquiaMapper mapper;

    public ServiceUzyTParroquia(DaoUzyTParroquia daoUzyTParroquia, UzyTCantonMapper mapperCanton, UzyTParroquiaMapper mapper) {
        this.daoUzyTParroquia = daoUzyTParroquia;
        this.mapperCanton = mapperCanton;
        this.mapper = mapper;
    }

    @Override
    public DtoUzyTParroquia ListarPorID(@PathVariable String id) {
        try {
            ModelUzyTParroquia entity = daoUzyTParroquia.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
            return mapper.entityToDto(entity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) { // Manejo de cualquier error en el servidor
            throw new RuntimeException("Error al buscar la parroquia con ID: " + id);
        }
    }


    @Override
    public List<DtoUzyTParroquia> ListarRegistros() {
        try {
            List<ModelUzyTParroquia> entities = daoUzyTParroquia.findAll();
            return mapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar registros de parroquia: " + ex.getMessage());
        }
    }

    @Override
    public List<DtoUzyTParroquia> findByIdDatosConCanton(String id) {
        try {
            List<ModelUzyTParroquia> entities = daoUzyTParroquia.findByIdDatosConCanton(id);
            return mapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar los registros: " + ex.getMessage());
        }
    }


    @Override
    public DtoUzyTCanton findCantonByParroquiaId(String parroquiaId) {
        try {
            ModelUzyTCanton entity = daoUzyTParroquia.findCantonByParroquiaId(parroquiaId);
            return mapperCanton.entityToDto(entity);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar los registros:  " + ex.getMessage());
        }
    }

    @Override
    public  List<DtoParroquiaCantonProvincia> obtenerIdsCantonYProvinciaPorParroquia(String parroquiaId) {
        try {
            return daoUzyTParroquia.findCantonAndProvinciaIdsByParroquiaId(parroquiaId);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar los registros: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTParroquia guardar(DtoUzyTParroquia dtoUzyTParroquia) {
        try {
            ModelUzyTParroquia entity = mapper.dtoToEntity(dtoUzyTParroquia);
            ModelUzyTParroquia nuevoEntity = daoUzyTParroquia.save(entity);
            return mapper.entityToDto(nuevoEntity);
        } catch (Exception ex) {
            throw new RuntimeException("Error al guardar la parroquia: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTParroquia editar(String id, DtoUzyTParroquia dtoUzyTParroquia) {
        try {
            ModelUzyTParroquia entity = daoUzyTParroquia.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));

            if(dtoUzyTParroquia.getUzytparroquia_id()!=null){
                entity.setUzytparroquia_id(dtoUzyTParroquia.getUzytparroquia_id());
            }
            if(dtoUzyTParroquia.getUzytparroquia_nombre() !=null){
                entity.setUzytparroquia_nombre(dtoUzyTParroquia.getUzytparroquia_nombre());
            }
            if(dtoUzyTParroquia.getUzytparroquia_tipo()!=null){
                entity.setUzytparroquia_tipo(dtoUzyTParroquia.getUzytparroquia_tipo());
            }
            if(dtoUzyTParroquia.getUzytparroquia_grupo_recinto() !=null){
                entity.setUzytparroquia_grupo_recinto(dtoUzyTParroquia.getUzytparroquia_grupo_recinto());
            }
            ModelUzyTParroquia updatedEntity = daoUzyTParroquia.save(entity);
            return mapper.entityToDto(updatedEntity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        }catch (Exception ex) {
            throw new RuntimeException("Error al editar la provincia -> " + ex.getMessage());
        }
    }


    @Override
    public boolean eliminar(String id) {
        try {
            ModelUzyTParroquia entity = daoUzyTParroquia.findById(id).orElseThrow(() -> new ResourceNotFoundException("Provincia no encontrada con ID: " + id));
            if (entity != null) {
                daoUzyTParroquia.delete(entity);
                return true;
            }
            return false;
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al eliminar la parroquia: " + ex.getMessage());
        }
    }
}
