package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTPerfil;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTPerfil;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTPerfil;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.InvalidArgumentException;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTPerfilMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTPerfil;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PathVariable;

import java.util.List;

@Service
public class ServiceUzyTPerfil implements IServiceUzyTPerfil {

    private final DaoUzyTPerfil daoUzyTPerfil;
    private final UzyTPerfilMapper mapper;

    public ServiceUzyTPerfil(DaoUzyTPerfil daoUzyTPerfil, UzyTPerfilMapper mapper) {
        this.daoUzyTPerfil = daoUzyTPerfil;
        this.mapper = mapper;
    }

    @Override
    public DtoUzyTPerfil ListarPorID(@PathVariable Long id) {
        if (id <= 0) {
            throw new InvalidArgumentException("El parámetro 'id' debe ser un valor positivo.");
        }
        try {
            ModelUzyTPerfil entity = daoUzyTPerfil.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
            return mapper.entityToDto(entity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) { // Manejo de cualquier error en el servidor
            throw new RuntimeException("Error al buscar el registro con ID: " + id);
        }
    }

    @Override
    public List<DtoUzyTPerfil> ListarRegistros() {
        try {
            List<ModelUzyTPerfil> entities = daoUzyTPerfil.findAll();
            return mapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar los registros: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTPerfil guardar(DtoUzyTPerfil dtoUzyTPerfil) {
        try {
            ModelUzyTPerfil entity = mapper.dtoToEntity(dtoUzyTPerfil);
            ModelUzyTPerfil nuevoEntity = daoUzyTPerfil.save(entity);
            return mapper.entityToDto(nuevoEntity);
        } catch (Exception ex) {
            throw new RuntimeException("Error al guardar el registro: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTPerfil editar(Long id, DtoUzyTPerfil dtoUzyTPerfil) {
        if (id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTPerfil entity = daoUzyTPerfil.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
            if (dtoUzyTPerfil.getUzytperfil_nombre() != null) {
                entity.setUzytperfil_nombre(dtoUzyTPerfil.getUzytperfil_nombre());
            }
            ModelUzyTPerfil updatedEntity = daoUzyTPerfil.save(entity);
            return mapper.entityToDto(updatedEntity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al editar el registro -> " + ex.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        if (id == null || id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTPerfil entity = daoUzyTPerfil.findById(id).orElseThrow(() -> new ResourceNotFoundException("Registro no encontrado con ID: " + id));
            if (entity != null) {
                daoUzyTPerfil.delete(entity);
                return true;
            }
            return false;
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al eliminar el registro: " + ex.getMessage());
        }
    }
}
