package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTPerfilMenu;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTPerfilMenu;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTPerfilMenu;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTPerfilMenuMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTPerfilMenu;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PathVariable;

import java.util.List;

@Service
public class ServiceUzyTPerfilMenu implements IServiceUzyTPerfilMenu {

    private final DaoUzyTPerfilMenu daoUzyTPerfilMenu;
    private final UzyTPerfilMenuMapper mapper;

    public ServiceUzyTPerfilMenu(DaoUzyTPerfilMenu daoUzyTPerfilMenu, UzyTPerfilMenuMapper mapper) {
        this.daoUzyTPerfilMenu = daoUzyTPerfilMenu;
        this.mapper = mapper;
    }

    @Override
    public DtoUzyTPerfilMenu ListarPorID(@PathVariable Long id) {
        if (id <= 0) {
            throw new IllegalArgumentException("El parámetro 'id' debe ser un valor positivo.");
        }
        try {
            ModelUzyTPerfilMenu entity = daoUzyTPerfilMenu.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
            return mapper.entityToDto(entity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) { // Manejo de cualquier error en el servidor
            throw new RuntimeException("Error al buscar el registro con ID: " + id);
        }
    }

    @Override
    public List<DtoUzyTPerfilMenu> ListarRegistros() {
        try {
            List<ModelUzyTPerfilMenu> entities = daoUzyTPerfilMenu.findAll();
            return mapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar registros" + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTPerfilMenu guardar(DtoUzyTPerfilMenu dtoUzyTPerfilMenu) {
        try {
            ModelUzyTPerfilMenu entity = mapper.dtoToEntity(dtoUzyTPerfilMenu);
            ModelUzyTPerfilMenu nuevoEntity = daoUzyTPerfilMenu.save(entity);
            return mapper.entityToDto(nuevoEntity);
        } catch (Exception ex) {
            throw new RuntimeException("Error al guardar el registro: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTPerfilMenu editar(Long id, DtoUzyTPerfilMenu dtoUzyTPerfilMenu) {
        if (id <= 0) {
            throw new IllegalArgumentException("El ID  debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTPerfilMenu entity = daoUzyTPerfilMenu.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
            ModelUzyTPerfilMenu updatedEntity = daoUzyTPerfilMenu.save(entity);
            return mapper.entityToDto(updatedEntity);
        }catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al editar el registro -> " + ex.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        if (id == null || id <= 0) {
            throw new IllegalArgumentException("El ID debe ser válido y mayor que cero.");
        }
        try {

            ModelUzyTPerfilMenu entity = daoUzyTPerfilMenu.findById(id).orElseThrow(() -> new ResourceNotFoundException("Registro no encontrado con ID: " + id));
            if (entity != null) {
                daoUzyTPerfilMenu.delete(entity);
                return true;
            }
            return false;
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al eliminar el registro: " + ex.getMessage());
        }
    }
}
