package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTPlanificacion;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTPlanificacion;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTPlanificacion;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTPlanificacionMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTPlanificacion;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PathVariable;

import java.util.List;

@Service
public class ServiceUzyTPlanificacion implements IServiceUzyTPlanificacion {

    private final DaoUzyTPlanificacion daoUzyTPlanificacion;
    private final UzyTPlanificacionMapper mapper;

    public ServiceUzyTPlanificacion(DaoUzyTPlanificacion daoUzyTPlanificacion, UzyTPlanificacionMapper mapper) {
        this.daoUzyTPlanificacion = daoUzyTPlanificacion;
        this.mapper = mapper;
    }

    @Override
    public DtoUzyTPlanificacion ListarPorID(@PathVariable Long id) {
        if (id <= 0) {
            throw new IllegalArgumentException("El parámetro 'id' debe ser un valor positivo.");
        }
        try {
            ModelUzyTPlanificacion entity = daoUzyTPlanificacion.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
            return mapper.entityToDto(entity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) { // Manejo de cualquier error en el servidor
            throw new RuntimeException("Error al buscar el registro con ID: " + id);
        }
    }

    @Override
    public List<DtoUzyTPlanificacion> ListarRegistros() {
        try {
            List<ModelUzyTPlanificacion> entities = daoUzyTPlanificacion.findAll();
            return mapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar los registros: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTPlanificacion guardar(DtoUzyTPlanificacion dtoUzyTPlanificacion) {
        try {
            ModelUzyTPlanificacion entity = mapper.dtoToEntity(dtoUzyTPlanificacion);
            ModelUzyTPlanificacion nuevoEntity = daoUzyTPlanificacion.save(entity);
            return mapper.entityToDto(nuevoEntity);
        }
        catch (Exception ex) {
            throw new RuntimeException("Error al guardar el registro: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTPlanificacion editar(Long id, DtoUzyTPlanificacion dtoUzyTPlanificacion) {
        if (id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTPlanificacion entity = daoUzyTPlanificacion.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
            if(dtoUzyTPlanificacion.getUzytplanificacionnombre() !=null){
                entity.setUzytplanificacionNombre(dtoUzyTPlanificacion.getUzytplanificacionnombre());
            }
            if(dtoUzyTPlanificacion.getUzytplanificacionnombre_largo() !=null){
                entity.setUzytplanificacionNombreLargo(dtoUzyTPlanificacion.getUzytplanificacionnombre_largo());
            }
            ModelUzyTPlanificacion updatedEntity = daoUzyTPlanificacion.save(entity);
            return mapper.entityToDto(updatedEntity);
        }catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al editar el registro -> " + ex.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        if (id == null || id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {

            ModelUzyTPlanificacion entity = daoUzyTPlanificacion.findById(id).orElseThrow(() -> new ResourceNotFoundException("Registro no encontrado con ID: " + id));
            if (entity != null) {
                daoUzyTPlanificacion.delete(entity);
                return true;
            }
            return false;
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al eliminar el registro: " + ex.getMessage());
        }
    }
}
