package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTProvincia;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTProvincia;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTProvincia;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTProvinciaMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTProvincia;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PathVariable;

import java.util.List;

@Service
public class ServiceUzyTProvincia implements IServiceUzyTProvincia {

    private final DaoUzyTProvincia daoUzyTProvincia;
    private final UzyTProvinciaMapper mapper;

    public ServiceUzyTProvincia(DaoUzyTProvincia daoUzyTProvincia, UzyTProvinciaMapper mapper) {
        this.daoUzyTProvincia = daoUzyTProvincia;
        this.mapper = mapper;
    }

    @Override
    public DtoUzyTProvincia ListarPorID(@PathVariable String id) {
        try {
            ModelUzyTProvincia entity = daoUzyTProvincia.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
            return mapper.entityToDto(entity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) { // Manejo de cualquier error en el servidor
            throw new RuntimeException("Error al buscar la privincia con ID: " + id);
        }
    }


    @Override
    public DtoUzyTProvincia guardar(DtoUzyTProvincia dtoUzyTProvincia) {
        try {
            ModelUzyTProvincia entity = mapper.dtoToEntity(dtoUzyTProvincia);
            ModelUzyTProvincia nuevoEntity = daoUzyTProvincia.save(entity);
            return mapper.entityToDto(nuevoEntity);
        } catch (Exception ex) {
            throw new RuntimeException("Error al guardar la provincia: " + ex.getMessage());
        }
    }

    @Override
    public List<DtoUzyTProvincia> ListarRegistros() {
        try {
            List<ModelUzyTProvincia> entities = daoUzyTProvincia.findAll();
            return mapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar registros de provincia: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTProvincia editar(String id, DtoUzyTProvincia dtoUzyTProvincia) {
        try {
            ModelUzyTProvincia entity = daoUzyTProvincia.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("Provincia no encontrada con ID: " + id));
            if (dtoUzyTProvincia.getUzytprovincia_nombre() != null) {
                entity.setUzytprovincia_nombre(dtoUzyTProvincia.getUzytprovincia_nombre());
            }
            if (dtoUzyTProvincia.getUzytprovincia_cod_inen() != null) {
                entity.setUzytprovincia_cod_inen(dtoUzyTProvincia.getUzytprovincia_cod_inen());
            }
            if (dtoUzyTProvincia.getUzytprovincia_region() != null) {
                entity.setUzytprovincia_region(dtoUzyTProvincia.getUzytprovincia_region());
            }
            ModelUzyTProvincia updatedEntity = daoUzyTProvincia.save(entity);
            return mapper.entityToDto(updatedEntity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        }catch (Exception ex) {
            throw new RuntimeException("Error al editar la provincia -> " + ex.getMessage());
        }
    }

    @Override
    public boolean eliminar(String id) {
        try {
            ModelUzyTProvincia entity = daoUzyTProvincia.findById(id).orElseThrow(() -> new ResourceNotFoundException("Provincia no encontrada con ID: " + id));
            if (entity != null) {
                daoUzyTProvincia.delete(entity);
                return true;
            }
            return false;
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al eliminar la provincia: " + ex.getMessage());
        }
    }

}
