package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTUsuario;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTPerfil;
import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTUsuario;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUsuarioConPerfiles;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTPerfil;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTUsuario;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.InvalidArgumentException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTPerfilMapper;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTUsuarioMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTPerfil;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTUsuario;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Set;

import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;


@Service
public class ServiceUzyTUsuario implements IServiceUzyTUsuario {

    //Mejores prácticas de diseño y desarrollo, más fácil probar el servicio a través de pruebas unitarias.
    private final DaoUzyTUsuario daoUzyTUsuario;
    private final DaoUzyTPerfil daoUzyTPerfil;
    private final UzyTUsuarioMapper mapper;
    private final UzyTPerfilMapper mapperPerfil;

    private static final String MESSAGE = "Usuario no encontrado con ID: ";

    public ServiceUzyTUsuario(DaoUzyTUsuario daoUzyTUsuario, DaoUzyTPerfil daoUzyTPerfil, UzyTUsuarioMapper mapper, UzyTPerfilMapper mapperPerfil) {
        this.daoUzyTUsuario = daoUzyTUsuario;
        this.daoUzyTPerfil = daoUzyTPerfil;
        this.mapper = mapper;
        this.mapperPerfil = mapperPerfil;
    }


    @Override
    public DtoUzyTUsuario ListarPorID(Long id) {
        if (id <= 0) {
            throw new InvalidArgumentException("El parámetro 'id' debe ser un valor positivo.");
        }
        ModelUzyTUsuario entity = daoUzyTUsuario.findById(id)
                .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
        return mapper.entityToDto(entity);
    }

    @Override
    public List<DtoUzyTUsuario> ListarRegistros() {
        try {
            List<ModelUzyTUsuario> entities = daoUzyTUsuario.findAll();
            return mapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar registros de usuarios: " + ex.getMessage());
        }
    }


    @Override
    public List<DtoUzyTUsuario> findByUsuario(String usuario) {
        if (usuario != null && !usuario.isEmpty()) {
            try {
                // Buscar por cualquier letra en nombres o apellidos
                List<ModelUzyTUsuario> entities = daoUzyTUsuario.findByUsuario(usuario);
                if (entities.isEmpty()) {
                    throw new ResourceNotFoundException("No se encontraron usuarios con el nombre proporcionado.");
                }
                return mapper.entitiesToDtos(entities);
            } catch (ResourceNotFoundException ex) {
                throw ex;
            } catch (Exception ex) {
                throw new RuntimeException("Error al buscar usuarios por nombre: " + ex.getMessage());
            }
        } else {
            throw new InvalidArgumentException("Parámetro de búsqueda inválido: el usuario no puede ser nulo o vacío.");
        }
    }

    @Override
    public DtoUzyTUsuario findByEmailPerfilAdmin(String email) {
        ModelUzyTUsuario entities = daoUzyTUsuario.findByEmailPerfilAdmin(email);
        return mapper.entityToDto(entities);
    }

    @Override
    public DtoUzyTUsuario guardar(DtoUzyTUsuario dtoUzyTUsuario) {
        try {
            ModelUzyTUsuario entity = mapper.dtoToEntity(dtoUzyTUsuario);
            ModelUzyTUsuario nuevoEntity = daoUzyTUsuario.save(entity);
            return mapper.entityToDto(nuevoEntity);
        } catch (Exception ex) {
            throw new RuntimeException("Error al guardar el usuario: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTUsuario editar(Long id, DtoUzyTUsuario dtoUzyTUsuario) {
        if (id <= 0) {
            throw new InvalidArgumentException("El ID del usuario debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTUsuario entity = daoUzyTUsuario.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
            if (dtoUzyTUsuario.getUzytusuario_nombres() != null) {
                entity.setUzytusuario_nombres(dtoUzyTUsuario.getUzytusuario_nombres());
            }
            if (dtoUzyTUsuario.getUzytusuario_apellidos() != null) {
                entity.setUzytusuario_apellidos(dtoUzyTUsuario.getUzytusuario_apellidos());
            }
            if (dtoUzyTUsuario.getUzytusuario_email() != null) {
                entity.setUzytusuario_email(dtoUzyTUsuario.getUzytusuario_email());
            }
            if (dtoUzyTUsuario.getUzytusuario_fecharegistro() != null) {
                entity.setUzytusuario_fecharegistro(dtoUzyTUsuario.getUzytusuario_fecharegistro());
            }
            if (dtoUzyTUsuario.getUzytusuario_estado() != null) {
                entity.setUzytusuario_estado(dtoUzyTUsuario.getUzytusuario_estado());
            }
            if (dtoUzyTUsuario.getUzytusuario_mail_incompletos() != null) {
                entity.setUzytusuario_mail_incompletos(dtoUzyTUsuario.getUzytusuario_mail_incompletos());
            }
            if (dtoUzyTUsuario.getUzytusuario_tipo_doc() != null) {
                entity.setUzytusuario_tipo_doc(dtoUzyTUsuario.getUzytusuario_tipo_doc());
            }
            if (dtoUzyTUsuario.getUzytusuario_cod_senescyt() != null) {
                entity.setUzytusuario_cod_senescyt(dtoUzyTUsuario.getUzytusuario_cod_senescyt());
            }
            if (dtoUzyTUsuario.getUzytusuario_fecha_act_micuenta() != null) {
                entity.setUzytusuario_fecha_act_micuenta(dtoUzyTUsuario.getUzytusuario_fecha_act_micuenta());
            }
            if (dtoUzyTUsuario.getUzytusuario_hexadecimalid() != null) {
                entity.setUzytusuario_hexadecimalid(dtoUzyTUsuario.getUzytusuario_hexadecimalid());
            }
            if (dtoUzyTUsuario.getUzytusuario_tratamiento() != null) {
                entity.setUzytusuario_tratamiento(dtoUzyTUsuario.getUzytusuario_tratamiento());
            }
            if (dtoUzyTUsuario.getUzytusuario_modificacion() != null) {
                entity.setUzytusuario_modificacion(dtoUzyTUsuario.getUzytusuario_modificacion());
            }
            if (dtoUzyTUsuario.getUzytusuario_tipo_user() != null) {
                entity.setUzytusuario_tipo_user(dtoUzyTUsuario.getUzytusuario_tipo_user());
            }
            if (dtoUzyTUsuario.getUzytusuario_email_institucional() != null) {
                entity.setUzytusuario_email_institucional(dtoUzyTUsuario.getUzytusuario_email_institucional());
            }
            if (dtoUzyTUsuario.getUzytusuario_telefono_fijo() != null) {
                entity.setUzytusuario_telefono_fijo(dtoUzyTUsuario.getUzytusuario_telefono_fijo());
            }
            if (dtoUzyTUsuario.getUzytusuario_telefono_celular() != null) {
                entity.setUzytusuario_telefono_celular(dtoUzyTUsuario.getUzytusuario_telefono_celular());
            }
            if (dtoUzyTUsuario.getSpriden_id() != null) {
                entity.setSpriden_id(dtoUzyTUsuario.getSpriden_id());
            }
            if (dtoUzyTUsuario.getSpriden_pidm() != null) {
                entity.setSpriden_pidm(dtoUzyTUsuario.getSpriden_pidm());
            }
            if (dtoUzyTUsuario.getUzytusuario_clave_tmp() != null) {
                entity.setUzytusuario_clave_tmp(dtoUzyTUsuario.getUzytusuario_clave_tmp());
            }
            if (dtoUzyTUsuario.getSpbpers_name_prefix() != null) {
                entity.setSpbpers_name_prefix(dtoUzyTUsuario.getSpbpers_name_prefix());
            }
            if (dtoUzyTUsuario.getSpbpers_sex() != null) {
                entity.setSpbpers_sex(dtoUzyTUsuario.getSpbpers_sex());
            }
            if (dtoUzyTUsuario.getUzytusuario_CAMPUS_ABREV_CODE() != null) {
                entity.setUzytusuario_CAMPUS_ABREV_CODE(dtoUzyTUsuario.getUzytusuario_CAMPUS_ABREV_CODE());
            }
            if (dtoUzyTUsuario.getUzytusuario_DEPARTAMENTO_CODE() != null) {
                entity.setUzytusuario_DPTO_ABREV_CODE(dtoUzyTUsuario.getUzytusuario_DEPARTAMENTO_CODE());
            }
            ModelUzyTUsuario usuarioActualizado = daoUzyTUsuario.save(entity);
            return mapper.entityToDto(usuarioActualizado);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al editar el usuario -> " + ex.getMessage());
        }
    }


    @Override
    public boolean eliminar(Long id) {
        if (id == null || id <= 0) {
            throw new InvalidArgumentException("El ID del usuario debe ser válido y mayor que cero.");
        }
        try {

            ModelUzyTUsuario entity = daoUzyTUsuario.findById(id).orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
            if (entity != null) {
                daoUzyTUsuario.delete(entity);
                return true;
            }
            return false;
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al eliminar el usuario: " + ex.getMessage());
        }
    }

    @Override
    public DtoUsuarioConPerfiles asignarPerfilAUsuario(Long uzytusuario_id, Long uzytperfil_id) {
        if (uzytusuario_id <= 0 || uzytperfil_id <= 0) {
            throw new InvalidArgumentException("Se debe ingresar un ID válido y mayor que cero.");
        }

        try {

            ModelUzyTUsuario usuario = daoUzyTUsuario.findById(uzytusuario_id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + uzytusuario_id));
            ModelUzyTPerfil perfil = daoUzyTPerfil
                    .findById(uzytperfil_id).orElseThrow(() -> new ResourceNotFoundException("Perfil no encontrado con ID: " + uzytperfil_id));
            if (usuario.getUzytperfils().contains(perfil)) {
                throw new InvalidArgumentException("El perfil ya está asignado al usuario.");
            }
            // Aquí asigno el perfil al usuario
            usuario.getUzytperfils().add(perfil);
            daoUzyTUsuario.save(usuario);
            Set<DtoUzyTPerfil> perfilesDto = mapperPerfil.entitiesToDtosSet(usuario.getUzytperfils());

            DtoUsuarioConPerfiles dtoUsuarioConPerfiles = new DtoUsuarioConPerfiles();
            dtoUsuarioConPerfiles.setUzytusuario_id(usuario.getUzytusuario_id());
            dtoUsuarioConPerfiles.setUzytusuario_nombres(usuario.getUzytusuario_nombres());
            dtoUsuarioConPerfiles.setPerfiles(perfilesDto);

            return dtoUsuarioConPerfiles;

        } catch (InvalidArgumentException | ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al asignar perfil al usuario: " + ex.getMessage());
        }
    }

    public DtoUsuarioConPerfiles eliminarPerfilDeUsuario(Long uzytusuario_id, Long uzytperfil_id) {
        if (uzytusuario_id <= 0 || uzytperfil_id <= 0) {
            throw new InvalidArgumentException("Se debe ingresar un ID válido y mayor que cero.");
        }

        try {
            ModelUzyTUsuario usuario = daoUzyTUsuario.findById(uzytusuario_id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + uzytusuario_id));

            ModelUzyTPerfil perfil = daoUzyTPerfil.findById(uzytperfil_id)
                    .orElseThrow(() -> new ResourceNotFoundException("Perfil no encontrado con ID: " + uzytperfil_id));

            if (!usuario.getUzytperfils().contains(perfil)) {
                throw new InvalidArgumentException("El perfil no está asignado al usuario.");
            }

            // Aquí eliminas el perfil del usuario
            usuario.getUzytperfils().remove(perfil);
            daoUzyTUsuario.save(usuario);
            Set<DtoUzyTPerfil> perfilesDto = mapperPerfil.entitiesToDtosSet(usuario.getUzytperfils());

            DtoUsuarioConPerfiles dtoUsuarioConPerfiles = new DtoUsuarioConPerfiles();
            dtoUsuarioConPerfiles.setUzytusuario_id(usuario.getUzytusuario_id());
            dtoUsuarioConPerfiles.setUzytusuario_nombres(usuario.getUzytusuario_nombres());
            dtoUsuarioConPerfiles.setPerfiles(perfilesDto);

            return dtoUsuarioConPerfiles;

        } catch (InvalidArgumentException | ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al eliminar perfil del usuario: " + ex.getMessage());
        }
    }

    @Override
    public Set<DtoUzyTPerfil> findPerfilesByUsuarioId(Long uzytusuario_id) {
        if (uzytusuario_id == null || uzytusuario_id <= 0) {
            throw new InvalidArgumentException("El ID  debe ser válido y mayor que cero.");
        }
        try {
            Set<ModelUzyTPerfil> perfiles = daoUzyTUsuario.findPerfilesByUsuarioId(uzytusuario_id);
            if (perfiles.isEmpty()) { // cuando la colección de perfiles es nula
                throw new ResourceNotFoundException("No se encontraron perfiles de usuario para el ID: " + uzytusuario_id);
            }
            return mapperPerfil.entitiesToDtosSet(perfiles);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al buscar perfiles de usuario: " + ex.getMessage());
        }
    }

    @Override
    public Set<DtoUzyTPerfil> findPerfilesByUsuarioEmail(String email) {
        Set<ModelUzyTPerfil> perfiles = daoUzyTUsuario.findPerfilesByUsuarioEmail(email);
        if (perfiles.isEmpty()) { // cuando la colección de perfiles es nula
            throw new ResourceNotFoundException("No se encontraron perfiles de usuario para el ID: " + email);
        }
        return mapperPerfil.entitiesToDtosSet(perfiles);
    }

}
