package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavActProy;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavActProy;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavActProy;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavActProyMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavActProy;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class ServiceUzyTavActProy implements IServiceUzyTavActProy {
    private final DaoUzyTavActProy daoUzyTavActProy;
    private final UzyTavActProyMapper mapper;
    private static final String MESSAGE = "No se encontró el registro con ID: ";


    public ServiceUzyTavActProy(DaoUzyTavActProy daoUzyTavActProy, UzyTavActProyMapper mapper) {
        this.daoUzyTavActProy = daoUzyTavActProy;
        this.mapper = mapper;
    }

    @Override
    public DtoUzyTavActProy ListarPorID(Long id) {
        if (id <= 0) {
            throw new IllegalArgumentException("El parámetro 'id' debe ser un valor positivo.");
        }
        try {
            ModelUzyTavActProy entity = daoUzyTavActProy.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
            return mapper.entityToDto(entity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) { // Manejo de cualquier error en el servidor
            throw new RuntimeException("Error al buscar el registro con ID: " + id);
        }
    }

    @Override
    public List<DtoUzyTavActProy> ListarRegistros() {
        try {
            List<ModelUzyTavActProy> entities = daoUzyTavActProy.findAll();
            return mapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar los registros: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTavActProy guardar(DtoUzyTavActProy dtoUzyTavActProy) {
        try {
            ModelUzyTavActProy entity = mapper.dtoToEntity(dtoUzyTavActProy);
            ModelUzyTavActProy nuevoEntity = daoUzyTavActProy.save(entity);
            return mapper.entityToDto(nuevoEntity);
        } catch (DataIntegrityViolationException ex) {
            throw ex;
        } catch (Exception ex) {
            // Manejo para otras excepciones
            throw new RuntimeException("Error al guardar el registro: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTavActProy editar(Long id, DtoUzyTavActProy dtoUzyTavActProy) {
        if (id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTavActProy entity = daoUzyTavActProy.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
            if (entity != null) {
                if (dtoUzyTavActProy.getUzytavactproy_fech_regis() != null) {
                    entity.setUzytavactproy_fech_regis(dtoUzyTavActProy.getUzytavactproy_fech_regis());
                }
                if (dtoUzyTavActProy.getUzytavactproy_observa() != null) {
                    entity.setUzytavactproy_observa(dtoUzyTavActProy.getUzytavactproy_observa());
                }

                ModelUzyTavActProy updatedEntity = daoUzyTavActProy.save(entity);
                return mapper.entityToDto(updatedEntity);
            } else {
                throw new ResourceNotFoundException(MESSAGE + id);
            }
        } catch (Exception e) {
            throw new RuntimeException("Error al editar el registro: " + e.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        if (id == null || id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTavActProy entity = daoUzyTavActProy.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("Registro no encontrado con ID: " + id));
            if (entity != null) {
                daoUzyTavActProy.delete(entity);
                return true;
            }
            return false;
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al eliminar el registro: " + ex.getMessage());
        }
    }
}
