package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavActaResponsa;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavActaResponsa;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavActaResponsa;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavActaResponsaMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavActaResponsa;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class ServiceUzyTavActaResponsa implements IServiceUzyTavActaResponsa {
    private final DaoUzyTavActaResponsa daoUzyTavActaResponsa;
    private final UzyTavActaResponsaMapper mapper;

    public ServiceUzyTavActaResponsa(DaoUzyTavActaResponsa daoUzyTavActaResponsa, UzyTavActaResponsaMapper mapper) {
        this.daoUzyTavActaResponsa = daoUzyTavActaResponsa;
        this.mapper = mapper;
    }

    @Override
    public DtoUzyTavActaResponsa ListarPorID(Long id) {
        if (id <= 0) {
            throw new IllegalArgumentException("El parámetro 'id' debe ser un valor positivo.");
        }
        try {
            ModelUzyTavActaResponsa entity = daoUzyTavActaResponsa.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
            return mapper.entityToDto(entity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) { // Manejo de cualquier error en el servidor
            throw new RuntimeException("Error al buscar el registro con ID: " + id);
        }
    }

    @Override
    public List<DtoUzyTavActaResponsa> ListarRegistros() {
        try {
            List<ModelUzyTavActaResponsa> entities = daoUzyTavActaResponsa.findAll();
            return mapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar los registros: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTavActaResponsa guardar(DtoUzyTavActaResponsa dtoUzyTavActaResponsa) {
        try {
            ModelUzyTavActaResponsa entity = mapper.dtoToEntity(dtoUzyTavActaResponsa);
            ModelUzyTavActaResponsa nuevoEntity = daoUzyTavActaResponsa.save(entity);
            return mapper.entityToDto(nuevoEntity);
        }
        catch (Exception ex) {
            throw new RuntimeException("Error al guardar el registro: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTavActaResponsa editar(Long id, DtoUzyTavActaResponsa dtoUzyTavActaResponsa) {
        if (id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTavActaResponsa entity = daoUzyTavActaResponsa.findById(id).get();
            if (entity != null) {
                if(dtoUzyTavActaResponsa.getUzytavactaresponsa_secuencial()!=null){
                    entity.setUzytavactaresponsa_secuencial(dtoUzyTavActaResponsa.getUzytavactaresponsa_secuencial());
                }
                if(dtoUzyTavActaResponsa.getUzytavactaresponsa_fecha_crea()!=null){
                    entity.setUzytavactaresponsa_fecha_crea(dtoUzyTavActaResponsa.getUzytavactaresponsa_fecha_crea());
                }
                if(dtoUzyTavActaResponsa.getUzytusuario_id_crea() !=null){
                    entity.setUzytusuario_id_crea(dtoUzyTavActaResponsa.getUzytusuario_id_crea());
                }
                if(dtoUzyTavActaResponsa.getUzytavactaresponsa_cerrado() !=null){
                    entity.setUzytavactaresponsa_cerrado(dtoUzyTavActaResponsa.getUzytavactaresponsa_cerrado());
                }
                if(dtoUzyTavActaResponsa.getUzytavactaresponsa_doc_nom() !=null){
                    entity.setUzytavactaresponsa_doc_nom(dtoUzyTavActaResponsa.getUzytavactaresponsa_doc_nom());
                }
                if(dtoUzyTavActaResponsa.getUzytavactaresponsa_doc_url() !=null){
                    entity.setUzytavactaresponsa_doc_url(dtoUzyTavActaResponsa.getUzytavactaresponsa_doc_url());
                }
                ModelUzyTavActaResponsa updatedEntity = daoUzyTavActaResponsa.save(entity);
                return mapper.entityToDto(updatedEntity);
            }else {
                throw new ResourceNotFoundException("No se encontró el registro con ID: " + id);
            }
        } catch (Exception e) {
            throw new ResourceNotFoundException("Error al editar el registro: " + e.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        if (id == null || id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {

            ModelUzyTavActaResponsa entity = daoUzyTavActaResponsa.findById(id).orElseThrow(() -> new ResourceNotFoundException("Registro no encontrado con ID: " + id));
            if (entity != null) {
                daoUzyTavActaResponsa.delete(entity);
                return true;
            }
            return false;
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al eliminar el registro: " + ex.getMessage());
        }
    }
}
