package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavAnexoSPR;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavAnexoSPR;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavAnexoSPR;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavAnexoSPRMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavAnexoSPR;
import ec.edu.espe.movilidad.MovilidadWS.Service.FileStorage.IFileStorageService;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;

@Service
public class ServiceUzyTavAnexoSPR implements IServiceUzyTavAnexoSPR{

    private final DaoUzyTavAnexoSPR daoUzyTavAnexoSPR;

    private final UzyTavAnexoSPRMapper mapper;

    private final Path rootFolder= Paths.get("uploadsIMG");

    private final IFileStorageService fileStorageService;

    public ServiceUzyTavAnexoSPR(DaoUzyTavAnexoSPR daoUzyTavAnexoSPR, UzyTavAnexoSPRMapper mapper, IFileStorageService fileStorageService) {
        this.daoUzyTavAnexoSPR = daoUzyTavAnexoSPR;
        this.mapper = mapper;
        this.fileStorageService = fileStorageService;
    }

    @Override
    public DtoUzyTavAnexoSPR ListarPorID(Long id) {
        Optional<ModelUzyTavAnexoSPR> optionalUzyTavConvoca = daoUzyTavAnexoSPR.findById(id);
        if (optionalUzyTavConvoca.isPresent()) {
            ModelUzyTavAnexoSPR model = optionalUzyTavConvoca.get();
            return mapper.entityToDto(model);
        }
        return null;
    }

    @Override
    public List<DtoUzyTavAnexoSPR> ListarRegistros() {
        List<ModelUzyTavAnexoSPR> entities = daoUzyTavAnexoSPR.findAll();
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public List<DtoUzyTavAnexoSPR> ListarRegistrosPorProyecID(Long proyecID) {
        List<ModelUzyTavAnexoSPR> entities = daoUzyTavAnexoSPR.findByProyecID(proyecID);
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public DtoUzyTavAnexoSPR save(DtoUzyTavAnexoSPR dtoUzyTavAnexoSPR) throws IOException {
        MultipartFile file = dtoUzyTavAnexoSPR.getUzytavanexospr_digital();
        if (file != null && !file.isEmpty()) {
            // Guardar el archivo en una ubicación temporal
            String tempFileName = "temp_" + file.getOriginalFilename();
            String tempFilePath = fileStorageService.saveFileIMG(file, tempFileName, true);

            // Guardar el registro en la base de datos para generar el ID
            ModelUzyTavAnexoSPR uzyTavAnexoSPR = mapper.dtoToEntity(dtoUzyTavAnexoSPR);
            uzyTavAnexoSPR = daoUzyTavAnexoSPR.save(uzyTavAnexoSPR);

            // Generar el nombre del archivo con el ID generado por la base de datos
            Long uzytavanexospr_id = uzyTavAnexoSPR.getUzytavanexospr_id();
            String fileName = uzytavanexospr_id + "_" + file.getOriginalFilename();

            // Mueve el archivo de la ubicación temporal a la ubicación final
            String fileUrl = fileStorageService.moveFileIMG(tempFilePath, fileName, true);
            uzyTavAnexoSPR.setUzytavanexospr_nombre(fileName);
            uzyTavAnexoSPR.setUzytavanexospr_url(fileUrl);

            //SE OBTIENE LA FECHA Y HORA ACTUAL
            LocalDateTime fechaHoraActual = LocalDateTime.now();
            uzyTavAnexoSPR.setUzytavanexospr_fech_subida(fechaHoraActual);
            // Actualiza el registro en la base de datos con el nombre y la URL del archivo
            uzyTavAnexoSPR = daoUzyTavAnexoSPR.save(uzyTavAnexoSPR);

            return mapper.entityToDto(uzyTavAnexoSPR);
        } else {
            ModelUzyTavAnexoSPR uzyTavConvoca = mapper.dtoToEntity(dtoUzyTavAnexoSPR);

            //SE OBTIENE LA FECHA Y HORA ACTUAL
            LocalDateTime fechaHoraActual = LocalDateTime.now();
            uzyTavConvoca.setUzytavanexospr_fech_subida(fechaHoraActual);
            // No se proporcionó ningún archivo, guardar solo el registro en la base de datos
            uzyTavConvoca = daoUzyTavAnexoSPR.save(uzyTavConvoca);
            return mapper.entityToDto(uzyTavConvoca);
        }
    }


    @Override
    public DtoUzyTavAnexoSPR edit(Long uzytavanexospr_id, DtoUzyTavAnexoSPR dtoUzyTavAnexoSPR) {
        return null;
    }

    @Override
    public void eliminar(Long id) {
        daoUzyTavAnexoSPR.deleteById(id);
    }

    @Override
    public byte[] obtenerContenidoPDF(Long uzytavanexospr_id) {
        Optional<ModelUzyTavAnexoSPR> optionalConvoca = daoUzyTavAnexoSPR.findById(uzytavanexospr_id);
        if (optionalConvoca.isPresent()) {
            ModelUzyTavAnexoSPR convoca = optionalConvoca.get();
            String fileName = convoca.getUzytavanexospr_nombre();
            if (fileName != null && !fileName.isEmpty()) {
                Path filePath = rootFolder.resolve(fileName);
                try {
                    return Files.readAllBytes(filePath);
                } catch (IOException e) {
                    throw new RuntimeException("Failed to read PDF file: " + fileName, e);
                }
            }
        }
        return null;
    }

    @Override
    public String obtenerUrlPDF(Long uzytavanexospr_id) {
        Optional<ModelUzyTavAnexoSPR> optionalUzyTavConvoca = daoUzyTavAnexoSPR.findById(uzytavanexospr_id);
        if (optionalUzyTavConvoca.isPresent()) {
            ModelUzyTavAnexoSPR uzyTavConvoca = optionalUzyTavConvoca.get();
            String fileName = uzyTavConvoca.getUzytavanexospr_nombre();
            if (fileName != null && !fileName.isEmpty()) {
                return rootFolder.resolve(fileName).toString();
            }
        }
        return null;
    }

    @Override
    public String obtenerNombrePDF(Long uzytavanexospr_id) {
        Optional<ModelUzyTavAnexoSPR> optionalUzyTavConvoca = daoUzyTavAnexoSPR.findById(uzytavanexospr_id);
        if (optionalUzyTavConvoca.isPresent()) {
            ModelUzyTavAnexoSPR uzyTavConvoca = optionalUzyTavConvoca.get();
            return uzyTavConvoca.getUzytavanexospr_nombre();
        }
        return null;
    }
    @Override
    public void eliminarConPDF(Long uzytavanexospr_id) {
        Optional<ModelUzyTavAnexoSPR> optionalUzyTavConvoca = daoUzyTavAnexoSPR.findById(uzytavanexospr_id);
        if (optionalUzyTavConvoca.isPresent()) {
            ModelUzyTavAnexoSPR uzyTavConvoca = optionalUzyTavConvoca.get();
            String fileName = uzyTavConvoca.getUzytavanexospr_nombre();
            if (fileName != null && !fileName.isEmpty()) {
                // Eliminar el archivo PDF
                Path filePath = rootFolder.resolve(fileName);
                try {
                    Files.deleteIfExists(filePath);
                } catch (IOException e) {
                    throw new RuntimeException("Failed to delete file: " + fileName, e);
                }
            }
            // Eliminar el registro de la base de datos
            daoUzyTavAnexoSPR.deleteById(uzytavanexospr_id);
        }
    }
}
