package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavConfObj;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavConfObj;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavConfObj;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavConfObjMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavConfObj;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class ServiceUzyTavConfObj implements IServiceUzyTavConfObj{

    private final DaoUzyTavConfObj daoUzyTavConfObj;
    private final UzyTavConfObjMapper mapper;

    public ServiceUzyTavConfObj(DaoUzyTavConfObj daoUzyTavConfObj, UzyTavConfObjMapper mapper) {
        this.daoUzyTavConfObj = daoUzyTavConfObj;
        this.mapper = mapper;
    }

    @Override
    public DtoUzyTavConfObj ListarPorID(Long id) {
        ModelUzyTavConfObj entity = daoUzyTavConfObj.findById(id)
                .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
        return mapper.entityToDto(entity);
    }

    @Override
    public List<DtoUzyTavConfObj> ListarRegistros() {
        List<ModelUzyTavConfObj> entities = daoUzyTavConfObj.findAll();
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public DtoUzyTavConfObj guardar(DtoUzyTavConfObj dtoUzyTavConfObj) {
        ModelUzyTavConfObj entity = mapper.dtoToEntity(dtoUzyTavConfObj);
        ModelUzyTavConfObj nuevoEntity = daoUzyTavConfObj.save(entity);
        return mapper.entityToDto(nuevoEntity);
    }

    @Override
    public DtoUzyTavConfObj editar(Long id, DtoUzyTavConfObj dtoUzyTavConfObj) {
        try {
            ModelUzyTavConfObj entity = daoUzyTavConfObj.findById(id).get();
            // Seteamos los nuevos datos del registro
            if (entity != null) {
                // Aquí realizas la asignación de los nuevos datos a la entidad
                entity.setUzytavconfobj_descripcion(dtoUzyTavConfObj.getUzytavconfobj_descripcion());
                entity.setUzytavconfobj_fecha(dtoUzyTavConfObj.getUzytavconfobj_fecha());
                entity.setUzytavconfobj_estado(dtoUzyTavConfObj.getUzytavconfobj_estado());

                ModelUzyTavConfObj updatedEntity = daoUzyTavConfObj.save(entity);

                return mapper.entityToDto(updatedEntity);
            } else {
                throw new ResourceNotFoundException("No se encontró el registro con ID: " + id);
            }
        } catch (Exception e) {
            throw new ResourceNotFoundException("Error al editar el registro: " + e.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        ModelUzyTavConfObj entity = daoUzyTavConfObj.findById(id).orElse(null);
        if (entity != null) {
            daoUzyTavConfObj.delete(entity);
            return true;
        }
        return false;
    }

   
}
