package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavConfiguracion;
import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavConfiguracion;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavConfiguracion;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavConfiguracionMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavConfiguracion;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PathVariable;

import java.util.List;
import java.util.stream.Collectors;

@Service
public class ServiceUzyTavConfiguracion implements IServiceUzyTavConfiguracion {

    private final DaoUzyTavConfiguracion daoUzyTavConfiguracion;
    private final UzyTavConfiguracionMapper mapper;

    private static final String MESSAGE = "No se encontró el registro con ID: ";

    private static final String MESSAGE2 = "Error al editar el registro: ";

    public ServiceUzyTavConfiguracion(DaoUzyTavConfiguracion daoUzyTavConfiguracion, UzyTavConfiguracionMapper mapper) {
        this.daoUzyTavConfiguracion = daoUzyTavConfiguracion;
        this.mapper = mapper;
    }

    @Override
    public DtoUzyTavConfiguracion ListarPorID(@PathVariable Long id) {
        ModelUzyTavConfiguracion entity = daoUzyTavConfiguracion.findById(id)
                .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
        return mapper.entityToDto(entity);
    }

    @Override
    public DtoUzyTavConfiguracion buscarPorCedulaVicerrector(String cedula) {
        ModelUzyTavConfiguracion entity = daoUzyTavConfiguracion.findByCedulaVicerrector(cedula);
        if (entity == null) {
            throw new ResourceNotFoundException("No se encontró el registro con cédula: " + cedula);
        }
        return mapper.entityToDto(entity);
    }

    @Override
    public DtoUzyTavConfiguracion buscarPorCedulaDirector(String cedula) {
        ModelUzyTavConfiguracion entity = daoUzyTavConfiguracion.findByCedulaDirector(cedula);
        if (entity == null) {
            throw new ResourceNotFoundException("No se encontró el registro con cédula: " + cedula);
        }
        return mapper.entityToDto(entity);
    }

    @Override
    public List<DtoUzyTavConfiguracion> ListarRegistros() {
        List<ModelUzyTavConfiguracion> entities = daoUzyTavConfiguracion.findAll();
        return entities.stream()
                .map(mapper::entityToDto) // Utilizar mapper::entityToDto en lugar de mapper::dtoToEntity
                .collect(Collectors.toList());
    }

    @Override
    public DtoUzyTavConfiguracion guardar(DtoUzyTavConfiguracion dtoUzyTavConfiguracion) {
        ModelUzyTavConfiguracion entity = mapper.dtoToEntity(dtoUzyTavConfiguracion);
        ModelUzyTavConfiguracion nuevoEntity = daoUzyTavConfiguracion.save(entity);
        return mapper.entityToDto(nuevoEntity);
    }

    @Override
    public DtoUzyTavConfiguracion editar(Long id, DtoUzyTavConfiguracion dtoUzyTavConfiguracion) {
        try {
            ModelUzyTavConfiguracion entity = daoUzyTavConfiguracion.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
            entity.setUzytavconfiguracion_id(dtoUzyTavConfiguracion.getUzytavconfiguracion_id());
            entity.setUzytavconfiguracion_vicerrector_ced(
                    dtoUzyTavConfiguracion.getUzytavconfiguracion_vicerrector_ced());
            entity.setUzytavconfiguracion_vicerrector_nom(
                    dtoUzyTavConfiguracion.getUzytavconfiguracion_vicerrector_nom());
            entity.setUzytavconfiguracion_vicerrector_ape(
                    dtoUzyTavConfiguracion.getUzytavconfiguracion_vicerrector_ape());
            entity.setUzytavconfiguracion_director_unid_vincu_ced(
                    dtoUzyTavConfiguracion.getUzytavconfiguracion_director_unid_vincu_ced());
            entity.setUzytavconfiguracion_director_unid_vincu_nom(
                    dtoUzyTavConfiguracion.getUzytavconfiguracion_director_unid_vincu_nom());
            entity.setUzytavconfiguracion_director_unid_vincu_ape(
                    dtoUzyTavConfiguracion.getUzytavconfiguracion_director_unid_vincu_ape());
            entity.setUzytavconfiguracion_director_unid_vincu_abrev(
                    dtoUzyTavConfiguracion.getUzytavconfiguracion_director_unid_vincu_abrev());
            entity.setUzytavconfiguracion_vicerrector_abrev(
                    dtoUzyTavConfiguracion.getUzytavconfiguracion_vicerrector_abrev());
            entity.setUzytavconfiguracion_vicerrector_grado_mil(
                    dtoUzyTavConfiguracion.getUzytavconfiguracion_vicerrector_grado_mil());
            entity.setUzytavconfiguracion_director_unid_vincu_grado_mil(
                    dtoUzyTavConfiguracion.getUzytavconfiguracion_director_unid_vincu_grado_mil());
            entity.setUzytavconfiguracion_nro_horas_minimo_certificado_e(
                    dtoUzyTavConfiguracion.getUzytavconfiguracion_nro_horas_minimo_certificado_e());
            entity.setUzytavconfiguracion_nro_horas_minimo_certificado_d(
                    dtoUzyTavConfiguracion.getUzytavconfiguracion_nro_horas_minimo_certificado_d());
            ModelUzyTavConfiguracion updatedEntity = daoUzyTavConfiguracion.save(entity);
            return mapper.entityToDto(updatedEntity);
        } catch (Exception e) {
            throw new ResourceNotFoundException(MESSAGE2 + e.getMessage());
        }
    }

    @Override
    public DtoUzyTavConfiguracion editarHorasDocente(Long id, DtoUzyTavConfiguracion dtoUzyTavConfiguracion) {
        ModelUzyTavConfiguracion entity = daoUzyTavConfiguracion.findById(id)
                .orElseThrow(() -> new ResourceNotFoundException("No existe el registro con el ID: " + id));

        // Actualizamos las propiedades de la entidad con los valores del DTO
        entity.setUzytavconfiguracion_nro_horas_minimo_certificado_d(
                dtoUzyTavConfiguracion.getUzytavconfiguracion_nro_horas_minimo_certificado_d());

        ModelUzyTavConfiguracion entityActualizado = daoUzyTavConfiguracion.save(entity);
        return mapper.entityToDto(entityActualizado);
    }

    @Override
    public DtoUzyTavConfiguracion editarHorasEstudiante(Long id, DtoUzyTavConfiguracion dtoUzyTavConfiguracion) {
        ModelUzyTavConfiguracion entity = daoUzyTavConfiguracion.findById(id)
                .orElseThrow(() -> new ResourceNotFoundException("No existe el registro con el ID: " + id));

        // Actualizamos las propiedades de la entidad con los valores del DTO
        entity.setUzytavconfiguracion_nro_horas_minimo_certificado_e(
                dtoUzyTavConfiguracion.getUzytavconfiguracion_nro_horas_minimo_certificado_e());

        ModelUzyTavConfiguracion entityActualizado = daoUzyTavConfiguracion.save(entity);
        return mapper.entityToDto(entityActualizado);
    }

    @Override
    public DtoUzyTavConfiguracion editarDatosVicerrector(Long id, DtoUzyTavConfiguracion dtoUzyTavConfiguracion) {
        try {
            ModelUzyTavConfiguracion entity = daoUzyTavConfiguracion.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
            entity.setUzytavconfiguracion_vicerrector_ced(
                    dtoUzyTavConfiguracion.getUzytavconfiguracion_vicerrector_ced());
            entity.setUzytavconfiguracion_vicerrector_nom(
                    dtoUzyTavConfiguracion.getUzytavconfiguracion_vicerrector_nom());
            entity.setUzytavconfiguracion_vicerrector_ape(
                    dtoUzyTavConfiguracion.getUzytavconfiguracion_vicerrector_ape());
            entity.setUzytavconfiguracion_vicerrector_abrev(
                    dtoUzyTavConfiguracion.getUzytavconfiguracion_vicerrector_abrev());
            entity.setUzytavconfiguracion_vicerrector_grado_mil(
                    dtoUzyTavConfiguracion.getUzytavconfiguracion_vicerrector_grado_mil());
            ModelUzyTavConfiguracion updatedEntity = daoUzyTavConfiguracion.save(entity);
            return mapper.entityToDto(updatedEntity);
        } catch (Exception e) {
            throw new ResourceNotFoundException(MESSAGE2 + e.getMessage());
        }
    }

    @Override
    public DtoUzyTavConfiguracion editarDatosDirector(Long id, DtoUzyTavConfiguracion dtoUzyTavConfiguracion) {

        try {
            ModelUzyTavConfiguracion entity = daoUzyTavConfiguracion.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
            entity.setUzytavconfiguracion_director_unid_vincu_ced(
                    dtoUzyTavConfiguracion.getUzytavconfiguracion_director_unid_vincu_ced());
            entity.setUzytavconfiguracion_director_unid_vincu_nom(
                    dtoUzyTavConfiguracion.getUzytavconfiguracion_director_unid_vincu_nom());
            entity.setUzytavconfiguracion_director_unid_vincu_ape(
                    dtoUzyTavConfiguracion.getUzytavconfiguracion_director_unid_vincu_ape());
            entity.setUzytavconfiguracion_director_unid_vincu_abrev(
                    dtoUzyTavConfiguracion.getUzytavconfiguracion_director_unid_vincu_abrev());
            entity.setUzytavconfiguracion_director_unid_vincu_grado_mil(
                    dtoUzyTavConfiguracion.getUzytavconfiguracion_director_unid_vincu_grado_mil());
            ModelUzyTavConfiguracion updatedEntity = daoUzyTavConfiguracion.save(entity);
            return mapper.entityToDto(updatedEntity);
        } catch (Exception e) {
            throw new ResourceNotFoundException(MESSAGE2 + e.getMessage());
        }
    }

    @Override
    public void eliminar(Long id) {

        ModelUzyTavConfiguracion entity = daoUzyTavConfiguracion.findById(id).get();
        daoUzyTavConfiguracion.delete(entity);
    }
}
