package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavConparaEva;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavConparaEva;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoConParaevaParaEvaValorPar;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavConparaEva;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavConparaEvaMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavConparaEva;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PathVariable;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class ServiceUzyTavConparaEva implements IServiceUzyTavConparaEva {

    private final DaoUzyTavConparaEva daoUzyTavConparaEva;


    private final UzyTavConparaEvaMapper mapper;

    public ServiceUzyTavConparaEva(DaoUzyTavConparaEva daoUzyTavConparaEva, UzyTavConparaEvaMapper mapper) {
        this.daoUzyTavConparaEva = daoUzyTavConparaEva;
        this.mapper = mapper;
    }

    @Override
    public DtoUzyTavConparaEva ListarPorID(@PathVariable Long id) {

        ModelUzyTavConparaEva entity = daoUzyTavConparaEva.findById(id)
                .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
        return mapper.entityToDto(entity);
    }


    @Override
    public List<DtoUzyTavConparaEva> getAllDatosRelacionadosTavParaeva() {
        List<ModelUzyTavConparaEva> entities =  daoUzyTavConparaEva.getAllDatosRelacionadosTavParaeva();
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public List<DtoUzyTavConparaEva> findByIdDatosRelacionadosTavParaeva(Long id) {
        List<ModelUzyTavConparaEva> entities =  daoUzyTavConparaEva.findByIdDatosRelacionadosTavParaeva(id);
        return mapper.entitiesToDtos(entities);
    }



    @Override
    public List<DtoUzyTavConparaEva> ListarRegistros() {
        List<ModelUzyTavConparaEva> entities = daoUzyTavConparaEva.findAll();
        return entities.stream()
                .map(mapper::entityToDto)
                .collect(Collectors.toList());
    }

    @Override
    public List<DtoUzyTavConparaEva> ListarRegistrosParametrosIniciales() {
        List<ModelUzyTavConparaEva> entities = daoUzyTavConparaEva.findAllEnable().orElse(new ArrayList<>());
        return entities.stream()
                .map(mapper::entityToDto)
                .collect(Collectors.toList());
    }

    @Override
    public List<DtoUzyTavConparaEva> ListarRegistrosParametrosFinales() {

        List<ModelUzyTavConparaEva> entities = daoUzyTavConparaEva.noActivo().orElse(new ArrayList<>());
        return mapper.entitiesToDtos(entities);
    }


    @Override
    public DtoUzyTavConparaEva guardar(DtoUzyTavConparaEva dtoUzyTavConparaEva) {
        ModelUzyTavConparaEva entity = mapper.dtoToEntity(dtoUzyTavConparaEva);
        //SE OBTIENE LA FECHA Y HORA ACTUAL
        LocalDateTime fechaHoraActual = LocalDateTime.now();

        // Se mantiene la fecha ingresada por el usuario y solo se actualiza la hora
        LocalDateTime fechaIngresada = entity.getUzytavconparaeva_fecha_c();
        LocalDateTime fechaActualizada = fechaIngresada.withHour(fechaHoraActual.getHour())
                .withMinute(fechaHoraActual.getMinute())
                .withSecond(fechaHoraActual.getSecond());
        entity.setUzytavconparaeva_fecha_c(fechaActualizada);
        ModelUzyTavConparaEva nuevoEntity = daoUzyTavConparaEva.save(entity);
        return mapper.entityToDto(nuevoEntity);
    }

    @Override
    public DtoUzyTavConparaEva editar(@PathVariable Long id, DtoUzyTavConparaEva dtoUzyTavConparaEva) {
        try {
            ModelUzyTavConparaEva entity = daoUzyTavConparaEva.findById(id).get();

            //Seteamos los nuevos datos del registro
            if (entity != null) {

                entity.setUzytavconparaeva_fecha_c(dtoUzyTavConparaEva.getUzytavconparaeva_fecha_c());
                entity.setUzytavconparaeva_descrip(dtoUzyTavConparaEva.getUzytavconparaeva_descrip());
                entity.setUzytavconparaeva_puminire(dtoUzyTavConparaEva.getUzytavconparaeva_puminire());
                entity.setUzytavconparaeva_tipo(dtoUzyTavConparaEva.getUzytavconparaeva_tipo());
                entity.setUzytavconparaeva_estado(dtoUzyTavConparaEva.getUzytavconparaeva_estado());
                ModelUzyTavConparaEva updatedEntity = daoUzyTavConparaEva.save(entity);
                return mapper.entityToDto(updatedEntity);
            }else {
                throw new ResourceNotFoundException("No se encontró el registro con ID: " + id);
            }
        } catch (Exception e) {
            throw new ResourceNotFoundException("Error al editar el registro: " + e.getMessage());
        }
    }

    @Override
    public List<DtoConParaevaParaEvaValorPar> findByIdDatosRelacionadosConParaevaParaEvaValorPar(Long conparaevaID) {
        return  daoUzyTavConparaEva.findByIdDatosRelacionadosConParaevaParaEvaValorPar(conparaevaID);
    }

    @Override
    public boolean eliminar(Long id) {
        ModelUzyTavConparaEva entity = daoUzyTavConparaEva.findById(id).orElse(null);
        if (entity != null) {
            daoUzyTavConparaEva.delete(entity);
            return true;
        }
        return false;
    }
}
