package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavConvoca;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavConvoca;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavConvoca;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.DuplicateStateException;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavConvocaMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavConvoca;
import ec.edu.espe.movilidad.MovilidadWS.Service.FileStorage.IFileStorageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.nio.file.*;
import java.util.*;

@Service
public class ServiceUzyTavConvoca implements IServiceUzyTavConvoca{

    private final Path rootFolder= Paths.get("uploadsPDF");
    private final DaoUzyTavConvoca daoUzyTavConvoca;
    private final UzyTavConvocaMapper modelMapper;
    private final IFileStorageService fileStorageService;


    @Autowired
    public ServiceUzyTavConvoca(DaoUzyTavConvoca daoUzyTavConvoca, UzyTavConvocaMapper modelMapper, IFileStorageService fileStorageService) {
        this.daoUzyTavConvoca = daoUzyTavConvoca;
        this.modelMapper = modelMapper;
        this.fileStorageService= fileStorageService;
    }

    @Override
    public DtoUzyTavConvoca ListarPorID(Long id) {
        Optional<ModelUzyTavConvoca> optionalUzyTavConvoca = daoUzyTavConvoca.findById(id);
        if (optionalUzyTavConvoca.isPresent()) {
            ModelUzyTavConvoca model = optionalUzyTavConvoca.get();
            return modelMapper.entityToDto(model);
        }
        return null;
    }


    @Override
    public List<DtoUzyTavConvoca> ListarRegistros() {
        List<ModelUzyTavConvoca> entities = daoUzyTavConvoca.findAll();
        return modelMapper.entitiesToDtos(entities);
    }

    @Override
    public DtoUzyTavConvoca save(DtoUzyTavConvoca dtoUzyTavConvoca) throws IOException {
        MultipartFile file = dtoUzyTavConvoca.getUzytavconvoca_digital();
        if (file != null && !file.isEmpty()) {
            // Guardar el archivo en una ubicación temporal
            String tempFileName = "temp_" + file.getOriginalFilename();
            String tempFilePath = fileStorageService.saveFile(file, tempFileName, true);

            // Guardar el registro en la base de datos para generar el ID
            ModelUzyTavConvoca uzyTavConvoca = modelMapper.dtoToEntity(dtoUzyTavConvoca);
            uzyTavConvoca = daoUzyTavConvoca.save(uzyTavConvoca);

            // Generar el nombre del archivo con el ID generado por la base de datos
            Long uzytavconvocaId = uzyTavConvoca.getUzytavconvoca_id();
            String fileName = uzytavconvocaId + "_" + file.getOriginalFilename();

            // Mueve el archivo de la ubicación temporal a la ubicación final
            String fileUrl = fileStorageService.moveFile(tempFilePath, fileName, true);
            uzyTavConvoca.setUzytavconvoca_digital_nombre(fileName);
            uzyTavConvoca.setUzytavconvoca_digital_url(fileUrl);

            // Actualiza el registro en la base de datos con el nombre y la URL del archivo
            uzyTavConvoca = daoUzyTavConvoca.save(uzyTavConvoca);

            return modelMapper.entityToDto(uzyTavConvoca);
        } else {
            // No se proporcionó ningún archivo, guardar solo el registro en la base de datos
            ModelUzyTavConvoca uzyTavConvoca = modelMapper.dtoToEntity(dtoUzyTavConvoca);
            uzyTavConvoca = daoUzyTavConvoca.save(uzyTavConvoca);
            return modelMapper.entityToDto(uzyTavConvoca);
        }
    }

    @Override
    public DtoUzyTavConvoca edit(Long uzytavconvocaId, DtoUzyTavConvoca dtoUzyTavConvoca) {
        if (uzytavconvocaId == null || dtoUzyTavConvoca == null) {
            throw new IllegalArgumentException("Invalid parameters for editing UzyTavConvoca");
        }

        Optional<ModelUzyTavConvoca> optionalUzyTavConvoca = daoUzyTavConvoca.findById(uzytavconvocaId);
        if (optionalUzyTavConvoca.isPresent()) {
            ModelUzyTavConvoca uzyTavConvoca = optionalUzyTavConvoca.get();
            MultipartFile file = dtoUzyTavConvoca.getUzytavconvoca_digital();

            if (file != null && !file.isEmpty()) {
                // Eliminar el archivo PDF anterior, si existe
                String previousFileName = uzyTavConvoca.getUzytavconvoca_digital_nombre();
                if (previousFileName != null && !previousFileName.isEmpty()) {
                    try {
                        Path previousFilePath = rootFolder.resolve(previousFileName);
                        Files.deleteIfExists(previousFilePath);
                    } catch (IOException e) {
                        throw new RuntimeException("Falló la eliminacion del archivo" + previousFileName, e);
                    }
                }

                // Guardar el nuevo archivo PDF
                String fileName = uzytavconvocaId + "_" + file.getOriginalFilename();
                String fileUrl = fileStorageService.saveFile(file, fileName, true);
                uzyTavConvoca.setUzytavconvoca_digital_url(fileUrl);
                uzyTavConvoca.setUzytavconvoca_digital_nombre(fileName);
            }

            // Actualizar los campos necesarios del objeto uzyTavConvoca con los valores del DTO
            if(dtoUzyTavConvoca.getUzytavconvoca_nombre() !=null){
                uzyTavConvoca.setUzytavconvoca_nombre(dtoUzyTavConvoca.getUzytavconvoca_nombre());

            }
            if(dtoUzyTavConvoca.getUzytavconvoca_observacion() !=null){
                uzyTavConvoca.setUzytavconvoca_observacion(dtoUzyTavConvoca.getUzytavconvoca_observacion());
            }
            if(dtoUzyTavConvoca.getUzytavconvoca_estado() !=null){
                uzyTavConvoca.setUzytavconvoca_estado(dtoUzyTavConvoca.getUzytavconvoca_estado());
            }
            if(dtoUzyTavConvoca.getUzytavconvoca_fecha_inicio() !=null){
                uzyTavConvoca.setUzytavconvoca_fecha_inicio(dtoUzyTavConvoca.getUzytavconvoca_fecha_inicio());
            }
            if(dtoUzyTavConvoca.getUzytavconvoca_fecha_fin() !=null){
                uzyTavConvoca.setUzytavconvoca_fecha_fin(dtoUzyTavConvoca.getUzytavconvoca_fecha_fin());
            }
            // Actualizar otros campos según sea necesario

            uzyTavConvoca = daoUzyTavConvoca.save(uzyTavConvoca);
            return modelMapper.entityToDto(uzyTavConvoca);
        } else {
            throw new NoSuchElementException("UzyTavConvoca not found for editing");
        }
    }



    @Override
    public void eliminar(Long uzytavconvoca_id) {
        daoUzyTavConvoca.deleteById(uzytavconvoca_id);
    }


    @Override
    public void eliminarConPDF(Long uzytavconvoca_id) {
        Optional<ModelUzyTavConvoca> optionalUzyTavConvoca = daoUzyTavConvoca.findById(uzytavconvoca_id);
        if (optionalUzyTavConvoca.isPresent()) {
            ModelUzyTavConvoca uzyTavConvoca = optionalUzyTavConvoca.get();
            String fileName = uzyTavConvoca.getUzytavconvoca_digital_nombre();
            if (fileName != null && !fileName.isEmpty()) {
                // Eliminar el archivo PDF
                Path filePath = rootFolder.resolve(fileName);
                try {
                    Files.deleteIfExists(filePath);
                } catch (IOException e) {
                    throw new RuntimeException("Failed to delete file: " + fileName, e);
                }
            }
            // Eliminar el registro de la base de datos
            daoUzyTavConvoca.deleteById(uzytavconvoca_id);
        }
    }

    @Override
    public String obtenerUrlPDF(Long uzytavconvocaId) {
        Optional<ModelUzyTavConvoca> optionalUzyTavConvoca = daoUzyTavConvoca.findById(uzytavconvocaId);
        if (optionalUzyTavConvoca.isPresent()) {
            ModelUzyTavConvoca uzyTavConvoca = optionalUzyTavConvoca.get();
            String fileName = uzyTavConvoca.getUzytavconvoca_digital_nombre();
            if (fileName != null && !fileName.isEmpty()) {
                return rootFolder.resolve(fileName).toString();
            }
        }
        return null;
    }

    @Override
    public byte[] obtenerContenidoPDF(Long convocaId) {
        Optional<ModelUzyTavConvoca> optionalConvoca = daoUzyTavConvoca.findById(convocaId);
        if (optionalConvoca.isPresent()) {
            ModelUzyTavConvoca convoca = optionalConvoca.get();
            String fileName = convoca.getUzytavconvoca_digital_nombre();
            if (fileName != null && !fileName.isEmpty()) {
                Path filePath = rootFolder.resolve(fileName);
                try {
                    return Files.readAllBytes(filePath);
                } catch (IOException e) {
                    throw new RuntimeException("Failed to read PDF file: " + fileName, e);
                }
            }
        }
        return null;
    }
    @Override
    public DtoUzyTavConvoca guardar(DtoUzyTavConvoca dtoUzyTavConvoca) {
        boolean active = daoUzyTavConvoca.existsActive();

        if (dtoUzyTavConvoca.getUzytavconvoca_estado().equals("A") && active) {
            throw new DuplicateStateException("No se puede guardar el registro debido a que ya existe un registro con estado 'A'");
        }
        try {
            ModelUzyTavConvoca entityw = modelMapper.dtoToEntity(dtoUzyTavConvoca);
            ModelUzyTavConvoca nuevoEntity = daoUzyTavConvoca.save(entityw);
            return modelMapper.entityToDto(nuevoEntity);
        }
        catch (Exception ex) {
            throw new RuntimeException("Error al guardar el registro: " + ex.getMessage());
        }

    }

    @Override
    public DtoUzyTavConvoca editar(Long id, DtoUzyTavConvoca dtoUzyTavConvoca) {
        if (id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTavConvoca entity = daoUzyTavConvoca.findById(id).get();
            if (entity != null) {
                if(dtoUzyTavConvoca.getUzytavconvoca_nombre()!=null){
                    entity.setUzytavconvoca_nombre(dtoUzyTavConvoca.getUzytavconvoca_nombre());
                }
                if(dtoUzyTavConvoca.getUzytavconvoca_observacion()!=null){
                    entity.setUzytavconvoca_observacion(dtoUzyTavConvoca.getUzytavconvoca_observacion());
                }
                if(dtoUzyTavConvoca.getUzytavconvoca_estado()!=null){
                    entity.setUzytavconvoca_estado(dtoUzyTavConvoca.getUzytavconvoca_estado());
                }
                if(dtoUzyTavConvoca.getUzytavconvoca_fecha_inicio()!=null){
                    entity.setUzytavconvoca_fecha_inicio(dtoUzyTavConvoca.getUzytavconvoca_fecha_inicio());
                }
                if(dtoUzyTavConvoca.getUzytavconvoca_fecha_fin()!=null){
                    entity.setUzytavconvoca_fecha_fin(dtoUzyTavConvoca.getUzytavconvoca_fecha_fin());
                }
                if(dtoUzyTavConvoca.getUzytavconvoca_digital_nombre()!=null){
                    entity.setUzytavconvoca_digital_nombre(dtoUzyTavConvoca.getUzytavconvoca_digital_nombre());
                }
                if(dtoUzyTavConvoca.getUzytavconvoca_digital_url()!=null){
                    entity.setUzytavconvoca_digital_url(dtoUzyTavConvoca.getUzytavconvoca_digital_url());
                }

                ModelUzyTavConvoca updatedEntity = daoUzyTavConvoca.save(entity);
                return modelMapper.entityToDto(updatedEntity);
            }else {
                throw new ResourceNotFoundException("No se encontró el registro con ID: " + id);
            }
        }
        catch (Exception e) {
            throw new ResourceNotFoundException("Error al editar el registro: " + e.getMessage());
        }
    }


    public String obtenerNombrePDF(Long convocaId) {
        Optional<ModelUzyTavConvoca> optionalUzyTavConvoca = daoUzyTavConvoca.findById(convocaId);
        if (optionalUzyTavConvoca.isPresent()) {
            ModelUzyTavConvoca uzyTavConvoca = optionalUzyTavConvoca.get();
            return uzyTavConvoca.getUzytavconvoca_digital_nombre();
        }
        return null;
    }

}
