package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavDepCenProy;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavDepCenProy;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavDepCenProy;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavDepCenProyMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavDepCenProy;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class ServiceUzyTavDepCenProy implements IServiceUzyTavDepCenProy{
    private final DaoUzyTavDepCenProy daoUzyTavDepCenProy;
    private final UzyTavDepCenProyMapper mapper;

    public ServiceUzyTavDepCenProy(DaoUzyTavDepCenProy daoUzyTavDepCenProy, UzyTavDepCenProyMapper mapper) {
        this.daoUzyTavDepCenProy = daoUzyTavDepCenProy;
        this.mapper = mapper;
    }

    @Override
    public DtoUzyTavDepCenProy ListarPorID(Long id) {
        if (id <= 0) {
            throw new IllegalArgumentException("El parámetro 'id' debe ser un valor positivo.");
        }
        try {
            ModelUzyTavDepCenProy entity = daoUzyTavDepCenProy.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
            return mapper.entityToDto(entity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) { // Manejo de cualquier error en el servidor
            throw new RuntimeException("Error al buscar el registro con ID: " + id);
        }
    }

    @Override
    public List<DtoUzyTavDepCenProy> ListarRegistros() {
        try {
            List<ModelUzyTavDepCenProy> entities = daoUzyTavDepCenProy.findAll();
            return mapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar los registros: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTavDepCenProy guardar(DtoUzyTavDepCenProy dtoUzyTavDepCenProy) {
        try {
            ModelUzyTavDepCenProy entity = mapper.dtoToEntity(dtoUzyTavDepCenProy);
            ModelUzyTavDepCenProy nuevoEntity = daoUzyTavDepCenProy.save(entity);
            return mapper.entityToDto(nuevoEntity);
        } catch (Exception ex) {
            throw new RuntimeException("Error al guardar el registro: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTavDepCenProy editar(Long id, DtoUzyTavDepCenProy dtoUzyTavDepCenProy) {
        if (id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTavDepCenProy entity = daoUzyTavDepCenProy.findById(id).get();
            if (entity != null) {
                if (dtoUzyTavDepCenProy.getStvsubj_code_depto() != null) {
                    entity.setStvsubj_code_depto(dtoUzyTavDepCenProy.getStvsubj_code_depto());
                }
                if (dtoUzyTavDepCenProy.getStvmajr_code_car() != null) {
                    entity.setStvmajr_code_car(dtoUzyTavDepCenProy.getStvmajr_code_car());
                }
                if (dtoUzyTavDepCenProy.getStvcamp_code() != null) {
                    entity.setStvcamp_code(dtoUzyTavDepCenProy.getStvcamp_code());
                }
                ModelUzyTavDepCenProy updatedEntity = daoUzyTavDepCenProy.save(entity);
                return mapper.entityToDto(updatedEntity);
            } else {
                throw new ResourceNotFoundException("No se encontró el registro con ID: " + id);
            }
        } catch (Exception e) {
            throw new ResourceNotFoundException("Error al editar el registro: " + e.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        if (id == null || id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {

            ModelUzyTavDepCenProy entity = daoUzyTavDepCenProy.findById(id).orElseThrow(() -> new ResourceNotFoundException("Registro no encontrado con ID: " + id));
            if (entity != null) {
                daoUzyTavDepCenProy.delete(entity);
                return true;
            }
            return false;
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al eliminar el registro: " + ex.getMessage());
        }
    }
}
