package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavDesigDoc;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavDesigDoc;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavDesigDoc;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavDesigDocMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavDesigDoc;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class ServiceUzyTavDesigDoc implements IServiceUzyTavDesigDoc {
    private static final String MESSAGE = "No se encontró el registro con ID: ";
    private final DaoUzyTavDesigDoc daoUzyTavDesigDoc;
    private final UzyTavDesigDocMapper mapper;

    public ServiceUzyTavDesigDoc(DaoUzyTavDesigDoc daoUzyTavDesigDoc, UzyTavDesigDocMapper mapper) {
        this.daoUzyTavDesigDoc = daoUzyTavDesigDoc;
        this.mapper = mapper;
    }

    @Override
    public DtoUzyTavDesigDoc ListarPorID(Long id) {
        if (id <= 0) {
            throw new IllegalArgumentException("El parámetro 'id' debe ser un valor positivo.");
        }
        try {
            ModelUzyTavDesigDoc entity = daoUzyTavDesigDoc.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
            return mapper.entityToDto(entity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) { // Manejo de cualquier error en el servidor
            throw new RuntimeException("Error al buscar el registro con ID: " + id);
        }
    }

    @Override
    public List<DtoUzyTavDesigDoc> ListarRegistros() {
        try {
            List<ModelUzyTavDesigDoc> entities = daoUzyTavDesigDoc.findAll();
            return mapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar los registros: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTavDesigDoc guardar(DtoUzyTavDesigDoc dtoUzyTavDesigDoc) {
        try {
            ModelUzyTavDesigDoc entity = mapper.dtoToEntity(dtoUzyTavDesigDoc);
            ModelUzyTavDesigDoc nuevoEntity = daoUzyTavDesigDoc.save(entity);
            return mapper.entityToDto(nuevoEntity);
        } catch (DataIntegrityViolationException ex) {
            throw ex;
        } catch (Exception ex) {
            // Manejo para otras excepciones
            throw new RuntimeException("Error al guardar el registro: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTavDesigDoc editar(Long id, DtoUzyTavDesigDoc dtoUzyTavDesigDoc) {
        if (id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTavDesigDoc entity = daoUzyTavDesigDoc.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
            if (entity != null) {
                if (dtoUzyTavDesigDoc.getUzytavdesigdoc_FUNCION() != null) {
                    entity.setUzytavdesigdoc_FUNCION(dtoUzyTavDesigDoc.getUzytavdesigdoc_FUNCION());
                }
                if (dtoUzyTavDesigDoc.getSpriden_ID() != null) {
                    entity.setSpriden_ID(dtoUzyTavDesigDoc.getSpriden_ID());
                }
                if (dtoUzyTavDesigDoc.getUzytavdesigdoc_FECHA_INICIO() != null) {
                    entity.setUzytavdesigdoc_FECHA_INICIO(dtoUzyTavDesigDoc.getUzytavdesigdoc_FECHA_INICIO());
                }
                if (dtoUzyTavDesigDoc.getUzytavdesigdoc_FECHA_FIN() != null) {
                    entity.setUzytavdesigdoc_FECHA_FIN(dtoUzyTavDesigDoc.getUzytavdesigdoc_FECHA_FIN());
                }
                if (dtoUzyTavDesigDoc.getUzytavdesigdoc_HORAS_PLANIFICADAS() != null) {
                    entity.setUzytavdesigdoc_HORAS_PLANIFICADAS(dtoUzyTavDesigDoc.getUzytavdesigdoc_HORAS_PLANIFICADAS());
                }
                if (dtoUzyTavDesigDoc.getUzytavdesigdoc_HORAS_DEDICADAS() != null) {
                    entity.setUzytavdesigdoc_HORAS_DEDICADAS(dtoUzyTavDesigDoc.getUzytavdesigdoc_HORAS_DEDICADAS());
                }

                if (dtoUzyTavDesigDoc.getUzytavdesigdoc_NRO_ACTA() != null) {
                    entity.setUzytavdesigdoc_NRO_ACTA(dtoUzyTavDesigDoc.getUzytavdesigdoc_NRO_ACTA());
                }
                if (dtoUzyTavDesigDoc.getUzytavdesigdoc_ACTA_CREADA() != null) {
                    entity.setUzytavdesigdoc_ACTA_CREADA(dtoUzyTavDesigDoc.getUzytavdesigdoc_ACTA_CREADA());
                }
                if (dtoUzyTavDesigDoc.getUzytavdesigdoc_ACTA_SUBIDA() != null) {
                    entity.setUzytavdesigdoc_ACTA_SUBIDA(dtoUzyTavDesigDoc.getUzytavdesigdoc_ACTA_SUBIDA());
                }
                if (dtoUzyTavDesigDoc.getUzytavdesigdoc_ESTADO_ACTA() != null) {
                    entity.setUzytavdesigdoc_ESTADO_ACTA(dtoUzyTavDesigDoc.getUzytavdesigdoc_ESTADO_ACTA());
                }
                if (dtoUzyTavDesigDoc.getDesigdoc_UZYTUSUARIO_CAMPUS_ABREV_CODE() != null) {
                    entity.setDesigdoc_UZYTUSUARIO_CAMPUS_ABREV_CODE(dtoUzyTavDesigDoc.getDesigdoc_UZYTUSUARIO_CAMPUS_ABREV_CODE());
                }
                if (dtoUzyTavDesigDoc.getDesigdoc_UZYTUSUARIO_DPTO_ABREV_CODE() != null) {
                    entity.setDesigdoc_UZYTUSUARIO_DPTO_ABREV_CODE(dtoUzyTavDesigDoc.getDesigdoc_UZYTUSUARIO_DPTO_ABREV_CODE());
                }
                if (dtoUzyTavDesigDoc.getDesigdoc_UZYTUSUARIO_DEPARTAMENTO_CODE() != null) {
                    entity.setDesigdoc_UZYTUSUARIO_DEPARTAMENTO_CODE(dtoUzyTavDesigDoc.getDesigdoc_UZYTUSUARIO_DEPARTAMENTO_CODE());
                }
                if (dtoUzyTavDesigDoc.getDesigdoc_UZYTUSUARIO_CAMPUS_CODE() != null) {
                    entity.setDesigdoc_UZYTUSUARIO_CAMPUS_CODE(dtoUzyTavDesigDoc.getDesigdoc_UZYTUSUARIO_CAMPUS_CODE());
                }
                if (dtoUzyTavDesigDoc.getUzytavdesigdoc_FECHA_REGISTRO() != null) {
                    entity.setUzytavdesigdoc_FECHA_REGISTRO(dtoUzyTavDesigDoc.getUzytavdesigdoc_FECHA_REGISTRO());
                }

                ModelUzyTavDesigDoc updatedEntity = daoUzyTavDesigDoc.save(entity);
                return mapper.entityToDto(updatedEntity);
            } else {
                throw new ResourceNotFoundException(MESSAGE + id);
            }
        } catch (Exception e) {
            throw new RuntimeException("Error al editar el registro: " + e.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        if (id == null || id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTavDesigDoc entity = daoUzyTavDesigDoc.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("Registro no encontrado con ID: " + id));
            if (entity != null) {
                daoUzyTavDesigDoc.delete(entity);
                return true;
            }
            return false;
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al eliminar el registro: " + ex.getMessage());
        }
    }
}
