package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavDesigEstu;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavDesigEstu;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavDesigEstu;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavDesigEstuMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavDesigEstu;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class ServiceUzyTavDesigEstu implements IServiceUzyTavDesigEstu {
    private static final String MESSAGE = "No se encontró el registro con ID: ";
    private final DaoUzyTavDesigEstu daoUzyTavDesigEstu;
    private final UzyTavDesigEstuMapper mapper;

    public ServiceUzyTavDesigEstu(DaoUzyTavDesigEstu daoUzyTavDesigEstu, UzyTavDesigEstuMapper mapper) {
        this.daoUzyTavDesigEstu = daoUzyTavDesigEstu;
        this.mapper = mapper;
    }

    @Override
    public DtoUzyTavDesigEstu ListarPorID(Long id) {
        if (id <= 0) {
            throw new IllegalArgumentException("El parámetro 'id' debe ser un valor positivo.");
        }
        try {
            ModelUzyTavDesigEstu entity = daoUzyTavDesigEstu.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
            return mapper.entityToDto(entity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) { // Manejo de cualquier error en el servidor
            throw new RuntimeException("Error al buscar el registro con ID: " + id);
        }
    }

    @Override
    public List<DtoUzyTavDesigEstu> ListarRegistros() {
        try {
            List<ModelUzyTavDesigEstu> entities = daoUzyTavDesigEstu.findAll();
            return mapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar los registros: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTavDesigEstu guardar(DtoUzyTavDesigEstu dtoUzyTavDesigEstu) {
        try {
            ModelUzyTavDesigEstu entity = mapper.dtoToEntity(dtoUzyTavDesigEstu);
            ModelUzyTavDesigEstu nuevoEntity = daoUzyTavDesigEstu.save(entity);
            return mapper.entityToDto(nuevoEntity);
        } catch (DataIntegrityViolationException ex) {
            throw ex;
        } catch (Exception ex) {
            // Manejo para otras excepciones
            throw new RuntimeException("Error al guardar el registro: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTavDesigEstu editar(Long id, DtoUzyTavDesigEstu dtoUzyTavDesigEstu) {
        if (id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTavDesigEstu entity = daoUzyTavDesigEstu.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
            if (entity != null) {
                if (dtoUzyTavDesigEstu.getUzytavdesigestu_FUNCION() != null) {
                    entity.setUzytavdesigestu_FUNCION(dtoUzyTavDesigEstu.getUzytavdesigestu_FUNCION());
                }
                if (dtoUzyTavDesigEstu.getSpriden_ID() != null) {
                    entity.setSpriden_ID(dtoUzyTavDesigEstu.getSpriden_ID());
                }
                if (dtoUzyTavDesigEstu.getUzytavdesigestu_FECHA_INICIO() != null) {
                    entity.setUzytavdesigestu_FECHA_INICIO(dtoUzyTavDesigEstu.getUzytavdesigestu_FECHA_INICIO());
                }
                if (dtoUzyTavDesigEstu.getUzytavdesigestu_FECHA_FIN() != null) {
                    entity.setUzytavdesigestu_FECHA_FIN(dtoUzyTavDesigEstu.getUzytavdesigestu_FECHA_FIN());
                }
                if (dtoUzyTavDesigEstu.getUzytavdesigestu_HORAS_PLANIFICADAS() != null) {
                    entity.setUzytavdesigestu_HORAS_PLANIFICADAS(dtoUzyTavDesigEstu.getUzytavdesigestu_HORAS_PLANIFICADAS());
                }
                if (dtoUzyTavDesigEstu.getUzytavdesigestu_HORAS_DEDICADAS() != null) {
                    entity.setUzytavdesigestu_HORAS_DEDICADAS(dtoUzyTavDesigEstu.getUzytavdesigestu_HORAS_DEDICADAS());
                }

                if (dtoUzyTavDesigEstu.getUzytavdesigestu_NRO_ACTA() != null) {
                    entity.setUzytavdesigestu_NRO_ACTA(dtoUzyTavDesigEstu.getUzytavdesigestu_NRO_ACTA());
                }
                if (dtoUzyTavDesigEstu.getUzytavdesigestu_FECHA_REGISTRO() != null) {
                    entity.setUzytavdesigestu_FECHA_REGISTRO(dtoUzyTavDesigEstu.getUzytavdesigestu_FECHA_REGISTRO());
                }
                if (dtoUzyTavDesigEstu.getUzytavdesigestu_ACTA_CREADA() != null) {
                    entity.setUzytavdesigestu_ACTA_CREADA(dtoUzyTavDesigEstu.getUzytavdesigestu_ACTA_CREADA());
                }
                if (dtoUzyTavDesigEstu.getUzytavdesigestu_ACTA_SUBIDA() != null) {
                    entity.setUzytavdesigestu_ACTA_SUBIDA(dtoUzyTavDesigEstu.getUzytavdesigestu_ACTA_SUBIDA());
                }
                if (dtoUzyTavDesigEstu.getDesigestu_UZYTUSUARIO_CAMPUS_ABREV_CODE() != null) {
                    entity.setDesigestu_UZYTUSUARIO_CAMPUS_ABREV_CODE(dtoUzyTavDesigEstu.getDesigestu_UZYTUSUARIO_CAMPUS_ABREV_CODE());
                }
                if (dtoUzyTavDesigEstu.getDesigestu_UZYTUSUARIO_DPTO_ABREV_CODE() != null) {
                    entity.setDesigestu_UZYTUSUARIO_DPTO_ABREV_CODE(dtoUzyTavDesigEstu.getDesigestu_UZYTUSUARIO_DPTO_ABREV_CODE());
                }
                if (dtoUzyTavDesigEstu.getDesigestu_UZYTUSUARIO_DEPARTAMENTO_CODE() != null) {
                    entity.setDesigestu_UZYTUSUARIO_DEPARTAMENTO_CODE(dtoUzyTavDesigEstu.getDesigestu_UZYTUSUARIO_DEPARTAMENTO_CODE());
                }
                if (dtoUzyTavDesigEstu.getDesigestu_UZYTUSUARIO_CAMPUS_CODE() != null) {
                    entity.setDesigestu_UZYTUSUARIO_CAMPUS_CODE(dtoUzyTavDesigEstu.getDesigestu_UZYTUSUARIO_CAMPUS_CODE());
                }
                ModelUzyTavDesigEstu updatedEntity = daoUzyTavDesigEstu.save(entity);
                return mapper.entityToDto(updatedEntity);
            } else {
                throw new ResourceNotFoundException(MESSAGE + id);
            }
        } catch (Exception e) {
            throw new RuntimeException("Error al editar el registro: " + e.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        if (id == null || id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTavDesigEstu entity = daoUzyTavDesigEstu.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("Registro no encontrado con ID: " + id));
            if (entity != null) {
                daoUzyTavDesigEstu.delete(entity);
                return true;
            }
            return false;
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al eliminar el registro: " + ex.getMessage());
        }
    }
}
