package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavDetActproy;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavDetActproy;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavDetActproy;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavDetActproyMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavDetActproy;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class ServiceUzyTavDetActproy implements IServiceUzyTavDetActproy {
    private static final String MESSAGE = "No se encontró el registro con ID: ";
    private final DaoUzyTavDetActproy daoUzyTavDetActproy;
    private final UzyTavDetActproyMapper mapper;

    public ServiceUzyTavDetActproy(DaoUzyTavDetActproy daoUzyTavDetActproy, UzyTavDetActproyMapper mapper) {
        this.daoUzyTavDetActproy = daoUzyTavDetActproy;
        this.mapper = mapper;
    }

    @Override
    public DtoUzyTavDetActproy ListarPorID(Long id) {
        if (id <= 0) {
            throw new IllegalArgumentException("El parámetro 'id' debe ser un valor positivo.");
        }
        try {
            ModelUzyTavDetActproy entity = daoUzyTavDetActproy.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
            return mapper.entityToDto(entity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) { // Manejo de cualquier error en el servidor
            throw new RuntimeException("Error al buscar el registro con ID: " + id);
        }
    }

    @Override
    public List<DtoUzyTavDetActproy> ListarRegistros() {
        try {
            List<ModelUzyTavDetActproy> entities = daoUzyTavDetActproy.findAll();
            return mapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar los registros: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTavDetActproy guardar(DtoUzyTavDetActproy dtoUzyTavDetActproy) {
        try {
            ModelUzyTavDetActproy entity = mapper.dtoToEntity(dtoUzyTavDetActproy);
            ModelUzyTavDetActproy nuevoEntity = daoUzyTavDetActproy.save(entity);
            return mapper.entityToDto(nuevoEntity);
        } catch (DataIntegrityViolationException ex) {
            throw ex;
        } catch (Exception ex) {
            // Manejo para otras excepciones
            throw new RuntimeException("Error al guardar el registro: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTavDetActproy editar(Long id, DtoUzyTavDetActproy dtoUzyTavDetActproy) {
        if (id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTavDetActproy entity = daoUzyTavDetActproy.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
            if (entity != null) {
                if (dtoUzyTavDetActproy.getUzytavdetactproyporcent() != null) {
                    entity.setUzytavdetactproyporcent(dtoUzyTavDetActproy.getUzytavdetactproyporcent());
                }
                if (dtoUzyTavDetActproy.getUzytavdetactproyobserva() != null) {
                    entity.setUzytavdetactproyobserva(dtoUzyTavDetActproy.getUzytavdetactproyobserva());
                }
                if (dtoUzyTavDetActproy.getUzytavdetactproyresultado() != null) {
                    entity.setUzytavdetactproyresultado(dtoUzyTavDetActproy.getUzytavdetactproyresultado());
                }

                ModelUzyTavDetActproy updatedEntity = daoUzyTavDetActproy.save(entity);
                return mapper.entityToDto(updatedEntity);
            } else {
                throw new ResourceNotFoundException(MESSAGE + id);
            }
        } catch (Exception e) {
            throw new RuntimeException("Error al editar el registro: " + e.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        if (id == null || id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTavDetActproy entity = daoUzyTavDetActproy.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("Registro no encontrado con ID: " + id));
            if (entity != null) {
                daoUzyTavDetActproy.delete(entity);
                return true;
            }
            return false;
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al eliminar el registro: " + ex.getMessage());
        }
    }
}
