package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavDoCompar;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavDoCompar;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavDoCompar;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavDoComparMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavDoCompar;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class ServiceUzyTavDoCompar implements IServiceUzyTavDoCompar {

    private final DaoUzyTavDoCompar daoUzyTavDoCompar;

    private final UzyTavDoComparMapper mapper;

    public ServiceUzyTavDoCompar(DaoUzyTavDoCompar daoUzyTavDoCompar, UzyTavDoComparMapper mapper) {
        this.daoUzyTavDoCompar = daoUzyTavDoCompar;
        this.mapper = mapper;
    }

    private static final String MESSAGE = "No se encontró el registro con ID: ";

    @Override
    public DtoUzyTavDoCompar ListarPorID(Long id) {
        if (id <= 0) {
            throw new IllegalArgumentException("El parámetro 'id' debe ser un valor positivo.");
        }
        try {
            ModelUzyTavDoCompar entity = daoUzyTavDoCompar.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
            return mapper.entityToDto(entity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) { // Manejo de cualquier error en el servidor
            throw new RuntimeException("Error al buscar el registro con ID: " + id);
        }
    }

    @Override
    public List<DtoUzyTavDoCompar> ListarRegistros() {
        try {
            List<ModelUzyTavDoCompar> entities = daoUzyTavDoCompar.findAll();
            return mapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar los registros: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTavDoCompar guardar(DtoUzyTavDoCompar dtoUzyTavDoCompar) {
        try {
            ModelUzyTavDoCompar entity = mapper.dtoToEntity(dtoUzyTavDoCompar);
            ModelUzyTavDoCompar nuevoEntity = daoUzyTavDoCompar.save(entity);
            return mapper.entityToDto(nuevoEntity);
        } catch (DataIntegrityViolationException ex) {
            throw ex;
        } catch (Exception ex) {
            // Manejo para otras excepciones
            throw new RuntimeException("Error al guardar el registro: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTavDoCompar editar(Long id, DtoUzyTavDoCompar dtoUzyTavDoCompar) {
        if (id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTavDoCompar entity = daoUzyTavDoCompar.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
            if (entity != null) {
                if (dtoUzyTavDoCompar.getUzytavdocompar_nombre_doc() != null) {
                    entity.setUzytavdocompar_nombre_doc(dtoUzyTavDoCompar.getUzytavdocompar_nombre_doc());
                }
                if (dtoUzyTavDoCompar.getUzytavdocompar_nombre_url() != null) {
                    entity.setUzytavdocompar_nombre_url(dtoUzyTavDoCompar.getUzytavdocompar_nombre_url());
                }
                if (dtoUzyTavDoCompar.getUzytavdocompar_fecha_crea() != null) {
                    entity.setUzytavdocompar_fecha_crea(dtoUzyTavDoCompar.getUzytavdocompar_fecha_crea());
                }

                ModelUzyTavDoCompar updatedEntity = daoUzyTavDoCompar.save(entity);
                return mapper.entityToDto(updatedEntity);
            } else {
                throw new ResourceNotFoundException(MESSAGE + id);
            }
        } catch (Exception e) {
            throw new RuntimeException("Error al editar el registro: " + e.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        if (id == null || id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTavDoCompar entity = daoUzyTavDoCompar.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
            if (entity != null) {
                daoUzyTavDoCompar.delete(entity);
                return true;
            }
            return false;
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al eliminar el registro: " + ex.getMessage());
        }
    }
}
