package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavDocParti;


import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavDocParti;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavDocParti;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavDocPartiMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavDocParti;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PathVariable;

import java.util.List;

@Service
public class ServiceUzyTavDocParti implements IServiceUzyTavDocParti {

    private final DaoUzyTavDocParti daoUzyTavDocParti;
    private final UzyTavDocPartiMapper mapper;

    public ServiceUzyTavDocParti(DaoUzyTavDocParti daoUzyTavDocParti, UzyTavDocPartiMapper mapper) {
        this.daoUzyTavDocParti = daoUzyTavDocParti;
        this.mapper = mapper;
    }

    @Override
    public DtoUzyTavDocParti ListarPorID(@PathVariable Long id) {
        ModelUzyTavDocParti entity = daoUzyTavDocParti.findById(id)
                .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
        return mapper.entityToDto(entity);
    }

    @Override
    public List<DtoUzyTavDocParti> ListarRegistros() {
        List<ModelUzyTavDocParti> entities = daoUzyTavDocParti.findAll();
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public List<DtoUzyTavDocParti> obtenerDocPartiPorProyecto(Long proyecID) {
        List<ModelUzyTavDocParti> entities = daoUzyTavDocParti.obtenerDocPartiPorProyecto(proyecID);
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public DtoUzyTavDocParti guardar(DtoUzyTavDocParti dtoUzyTavDocParti) {
        ModelUzyTavDocParti entity = mapper.dtoToEntity(dtoUzyTavDocParti);
        ModelUzyTavDocParti nuevoEntity = daoUzyTavDocParti.save(entity);
        return mapper.entityToDto(nuevoEntity);
    }

    @Override
    public DtoUzyTavDocParti editar(@PathVariable Long id, DtoUzyTavDocParti dtoUzyTavDocParti) {
        try {
            ModelUzyTavDocParti entity = daoUzyTavDocParti.findById(id).get();
            if (entity != null) {
                entity.setUzytavsubj_code(dtoUzyTavDocParti.getUzytavsubj_code());
                entity.setUzytavdocparti_numero(dtoUzyTavDocParti.getUzytavdocparti_numero());
                entity.setUzytavcamp_code(dtoUzyTavDocParti.getUzytavcamp_code());

                ModelUzyTavDocParti updatedEntity = daoUzyTavDocParti.save(entity);
                return mapper.entityToDto(updatedEntity);
            } else {
                throw new ResourceNotFoundException("No se encontró el registro con ID: " + id);
            }
        } catch (Exception e) {
            throw new ResourceNotFoundException("Error al editar el registro: " + e.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        ModelUzyTavDocParti entity = daoUzyTavDocParti.findById(id).orElse(null);
        if (entity != null) {
            daoUzyTavDocParti.delete(entity);
            return true;
        }
        return false;
    }
}
