package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavEstuParti;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavEstuParti;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavEstuParti;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavEstuPartiMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavEstuParti;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PathVariable;

import java.util.List;

@Service
public class ServiceUzyTavEstuParti implements IServiceUzyTavEstuParti {

    private final DaoUzyTavEstuParti daoUzyTavEstuParti;
    private final UzyTavEstuPartiMapper mapper;

    public ServiceUzyTavEstuParti(DaoUzyTavEstuParti daoUzyTavEstuParti, UzyTavEstuPartiMapper mapper) {
        this.daoUzyTavEstuParti = daoUzyTavEstuParti;
        this.mapper = mapper;
    }

    @Override
    public DtoUzyTavEstuParti ListarPorID(@PathVariable Long id) {
        ModelUzyTavEstuParti entity = daoUzyTavEstuParti.findById(id)
                .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
        return mapper.entityToDto(entity);
    }

    @Override
    public List<DtoUzyTavEstuParti> ListarRegistros() {
        List<ModelUzyTavEstuParti> entities = daoUzyTavEstuParti.findAll();
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public List<DtoUzyTavEstuParti> obtenerEstuPartiPorProyecto(Long proyecID) {
        List<ModelUzyTavEstuParti> entities = daoUzyTavEstuParti.obtenerEstuPartiPorProyecto(proyecID);
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public DtoUzyTavEstuParti guardar(DtoUzyTavEstuParti dtoUzyTavEstuParti) {
        ModelUzyTavEstuParti entity = mapper.dtoToEntity(dtoUzyTavEstuParti);
        ModelUzyTavEstuParti nuevoEntity = daoUzyTavEstuParti.save(entity);
        return mapper.entityToDto(nuevoEntity);
    }

    @Override
    public DtoUzyTavEstuParti editar(@PathVariable Long id, DtoUzyTavEstuParti dtoUzyTavEstuParti) {
        try {
            ModelUzyTavEstuParti entity = daoUzyTavEstuParti.findById(id).get();
            if (entity != null) {
                entity.setUzytavmajr_code(dtoUzyTavEstuParti.getUzytavmajr_code());
                entity.setUzytavestuparti_numero(dtoUzyTavEstuParti.getUzytavestuparti_numero());

                ModelUzyTavEstuParti updatedEntity = daoUzyTavEstuParti.save(entity);
                return mapper.entityToDto(updatedEntity);
            } else {
                throw new ResourceNotFoundException("No se encontró el registro con ID: " + id);
            }
        } catch (Exception e) {
            throw new ResourceNotFoundException("Error al editar el registro: " + e.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        ModelUzyTavEstuParti entity = daoUzyTavEstuParti.findById(id).orElse(null);
        if (entity != null) {
            daoUzyTavEstuParti.delete(entity);
            return true;
        }
        return false;
    }
}

