package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavHorEjec;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavHorEjec;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavHorEjec;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavHorEjecMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavHorEjec;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class ServiceUzyTavHorEjec implements IServiceUzyTavHorEjec{
    private final DaoUzyTavHorEjec daoUzyTavHorEjec;
    private final UzyTavHorEjecMapper mapper;

    public ServiceUzyTavHorEjec(DaoUzyTavHorEjec daoUzyTavHorEjec, UzyTavHorEjecMapper mapper) {
        this.daoUzyTavHorEjec = daoUzyTavHorEjec;
        this.mapper = mapper;
    }

    @Override
    public DtoUzyTavHorEjec ListarPorID(Long id) {
        if (id <= 0) {
            throw new IllegalArgumentException("El parámetro 'id' debe ser un valor positivo.");
        }
        try {
            ModelUzyTavHorEjec entity = daoUzyTavHorEjec.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
            return mapper.entityToDto(entity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) { // Manejo de cualquier error en el servidor
            throw new RuntimeException("Error al buscar el registro con ID: " + id);
        }
    }

    @Override
    public List<DtoUzyTavHorEjec> ListarRegistros() {
        try {
            List<ModelUzyTavHorEjec> entities = daoUzyTavHorEjec.findAll();
            return mapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar los registros: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTavHorEjec guardar(DtoUzyTavHorEjec dtoUzyTavHorEjec) {
        try {
            ModelUzyTavHorEjec entity = mapper.dtoToEntity(dtoUzyTavHorEjec);
            ModelUzyTavHorEjec nuevoEntity = daoUzyTavHorEjec.save(entity);
            return mapper.entityToDto(nuevoEntity);
        }
        catch (Exception ex) {
            throw new RuntimeException("Error al guardar el registro: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTavHorEjec editar(Long id, DtoUzyTavHorEjec dtoUzyTavHorEjec) {
        if (id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTavHorEjec entity = daoUzyTavHorEjec.findById(id).get();
            if (entity != null) {
                if(dtoUzyTavHorEjec.getUzytavhorejec_fec_fin()!=null){
                    entity.setUzytavhorejec_fec_fin(dtoUzyTavHorEjec.getUzytavhorejec_fec_fin());
                }
                if(dtoUzyTavHorEjec.getUzytavhorejec_fec_ini()!=null){
                    entity.setUzytavhorejec_fec_fin(dtoUzyTavHorEjec.getUzytavhorejec_fec_ini());
                }
                if(dtoUzyTavHorEjec.getUzytavhorejec_total()!=null){
                    entity.setUzytavhorejec_total(dtoUzyTavHorEjec.getUzytavhorejec_total());
                }
                if(dtoUzyTavHorEjec.getUzytavhorejec_calificacion()!=null){
                    entity.setUzytavhorejec_calificacion(dtoUzyTavHorEjec.getUzytavhorejec_calificacion());
                }

                ModelUzyTavHorEjec updatedEntity = daoUzyTavHorEjec.save(entity);
                return mapper.entityToDto(updatedEntity);
            }else {
                throw new ResourceNotFoundException("No se encontró el registro con ID: " + id);
            }
        } catch (Exception e) {
            throw new ResourceNotFoundException("Error al editar el registro: " + e.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        if (id == null || id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {

            ModelUzyTavHorEjec entity = daoUzyTavHorEjec.findById(id).orElseThrow(() -> new ResourceNotFoundException("Registro no encontrado con ID: " + id));
            if (entity != null) {
                daoUzyTavHorEjec.delete(entity);
                return true;
            }
            return false;
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al eliminar el registro: " + ex.getMessage());
        }
    }
}
