package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavInstProy;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavInstProy;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavInstProy;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavInstProyMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavInstProy;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class ServiceUzyTavInstProy implements IServiceUzyTavInstProy {

    private final UzyTavInstProyMapper mapper;
    private final DaoUzyTavInstProy daoUzyTavInstProy;
    private static final String MESSAGE = "No se encontró el registro con ID: ";

    public ServiceUzyTavInstProy(UzyTavInstProyMapper mapper, DaoUzyTavInstProy daoUzyTavInstProy) {
        this.mapper = mapper;
        this.daoUzyTavInstProy = daoUzyTavInstProy;
    }

    @Override
    public DtoUzyTavInstProy ListarPorID(Long id) {
        if (id <= 0) {
            throw new IllegalArgumentException("El parámetro 'id' debe ser un valor positivo.");
        }
        try {
            ModelUzyTavInstProy entity = daoUzyTavInstProy.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
            return mapper.entityToDto(entity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) { // Manejo de cualquier error en el servidor
            throw new RuntimeException("Error al buscar el registro con ID: " + id);
        }
    }

    @Override
    public List<DtoUzyTavInstProy> ListarRegistros() {
        try {
            List<ModelUzyTavInstProy> entities = daoUzyTavInstProy.findAll();
            return mapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar los registros: " + ex.getMessage());
        }
    }

    @Override
    public List<DtoUzyTavInstProy> obtenerRegistrosTavInsituc(Long uzytavinstituc_id) {
        try {
            List<ModelUzyTavInstProy> entities = daoUzyTavInstProy.obtenerRegistrosTavInsituc(uzytavinstituc_id);
            return mapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar los registros: " + ex.getMessage());
        }
    }

    @Override
    public List<DtoUzyTavInstProy> obtenerRegistrosProyec(Long uzytavproyec_id) {
        try {
            List<ModelUzyTavInstProy> entities = daoUzyTavInstProy.obtenerRegistrosProyec(uzytavproyec_id);
            return mapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar los registros: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTavInstProy guardar(DtoUzyTavInstProy dtoUzyTavInstProy) {
        try {
            ModelUzyTavInstProy entity = mapper.dtoToEntity(dtoUzyTavInstProy);
            ModelUzyTavInstProy nuevoEntity = daoUzyTavInstProy.save(entity);
            return mapper.entityToDto(nuevoEntity);
        } catch (DataIntegrityViolationException ex) {
            throw ex;
        } catch (Exception ex) {
            // Manejo para otras excepciones
            throw new RuntimeException("Error al guardar el registro: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTavInstProy editar(Long id, DtoUzyTavInstProy dtoUzyTavInstProy) {
        if (id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTavInstProy entity = daoUzyTavInstProy.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
            if (entity != null) {
                if (dtoUzyTavInstProy.getUzytavinstproy_detaotro() != null) {
                    entity.setUzytavinstproy_detaotro(dtoUzyTavInstProy.getUzytavinstproy_detaotro());
                }
                if (dtoUzyTavInstProy.getUzytavinstproy_declaracion() != null) {
                    entity.setUzytavinstproy_declaracion(dtoUzyTavInstProy.getUzytavinstproy_declaracion());
                }
                if (dtoUzyTavInstProy.getUzytavinstproy_fecha_crea() != null) {
                    entity.setUzytavinstproy_fecha_crea(dtoUzyTavInstProy.getUzytavinstproy_fecha_crea());
                }
                if (dtoUzyTavInstProy.getUzytavinstproy_dir_hombres() != null) {
                    entity.setUzytavinstproy_dir_hombres(dtoUzyTavInstProy.getUzytavinstproy_dir_hombres());
                }
                if (dtoUzyTavInstProy.getUzytavinstproy_dir_mujeres() != null) {
                    entity.setUzytavinstproy_dir_mujeres(dtoUzyTavInstProy.getUzytavinstproy_dir_mujeres());
                }
                if (dtoUzyTavInstProy.getUzytavinstproy_dir_ninios() != null) {
                    entity.setUzytavinstproy_dir_ninios(dtoUzyTavInstProy.getUzytavinstproy_dir_ninios());
                }
                if (dtoUzyTavInstProy.getUzytavinstproy_dir_discapacidad() != null) {
                    entity.setUzytavinstproy_dir_discapacidad(dtoUzyTavInstProy.getUzytavinstproy_dir_discapacidad());
                }
                if (dtoUzyTavInstProy.getUzytavinstproy_indirectos() != null) {
                    entity.setUzytavinstproy_indirectos(dtoUzyTavInstProy.getUzytavinstproy_indirectos());
                }
                if (dtoUzyTavInstProy.getUzytavinstproy_desc_benefic() != null) {
                    entity.setUzytavinstproy_desc_benefic(dtoUzyTavInstProy.getUzytavinstproy_desc_benefic());
                }
                if (dtoUzyTavInstProy.getUzytavinstproy_DIRECTOS() != null) {
                    entity.setUzytavinstproy_DIRECTOS(dtoUzyTavInstProy.getUzytavinstproy_DIRECTOS());
                }
                if (dtoUzyTavInstProy.getUzytavinstproy_TOTAL() != null) {
                    entity.setUzytavinstproy_TOTAL(dtoUzyTavInstProy.getUzytavinstproy_TOTAL());
                }
                ModelUzyTavInstProy updatedEntity = daoUzyTavInstProy.save(entity);
                return mapper.entityToDto(updatedEntity);
            } else {
                throw new ResourceNotFoundException(MESSAGE + id);
            }
        } catch (Exception e) {
            throw new RuntimeException("Error al editar el registro: " + e.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        if (id == null || id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTavInstProy entity = daoUzyTavInstProy.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("Registro no encontrado con ID: " + id));
            if (entity != null) {
                daoUzyTavInstProy.delete(entity);
                return true;
            }
            return false;
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al eliminar el registro: " + ex.getMessage());
        }
    }
}
