package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavInstituc;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavInstituc;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavInstituc;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavInstitucMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTParroquia;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavInstituc;
import org.springframework.stereotype.Service;

import javax.persistence.EntityManager;
import java.util.List;
import java.util.NoSuchElementException;

@Service
public class ServiceUzyTavInstituc implements IServiceUzyTavInstituc {

    private final DaoUzyTavInstituc daoUzyTavInstituc;
    private final UzyTavInstitucMapper mapper;
    private final EntityManager entityManager;

    public ServiceUzyTavInstituc(DaoUzyTavInstituc daoUzyTavInstituc, UzyTavInstitucMapper mapper, EntityManager entityManager) {
        this.daoUzyTavInstituc = daoUzyTavInstituc;
        this.mapper = mapper;
        this.entityManager = entityManager;
    }


    @Override
    public DtoUzyTavInstituc ListarPorID(Long id) {
        ModelUzyTavInstituc entity = daoUzyTavInstituc.findById(id)
                .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
        return mapper.entityToDto(entity);
    }

    @Override
    public List<DtoUzyTavInstituc> ListarRegistros() {
        List<ModelUzyTavInstituc> entities = daoUzyTavInstituc.findAll();
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public DtoUzyTavInstituc guardar(DtoUzyTavInstituc dtoUzyTavInstituc) {
        ModelUzyTavInstituc entity = mapper.dtoToEntity(dtoUzyTavInstituc);
        ModelUzyTavInstituc nuevoEntity = daoUzyTavInstituc.save(entity);
        return mapper.entityToDto(nuevoEntity);

    }

    @Override
    public DtoUzyTavInstituc editar(Long id,DtoUzyTavInstituc dtoUzyTavInstituc) {
        try {
            ModelUzyTavInstituc entity = daoUzyTavInstituc.findById(id)
                    .orElseThrow(() -> new NoSuchElementException("No existe el registro con el ID: " + id));

            if (entity != null) {
                // Actualizar los datos del registro
                entity.setUzytavinstituc_nombre(dtoUzyTavInstituc.getUzytavinstituc_nombre());
                entity.setUzytavinstituc_calle(dtoUzyTavInstituc.getUzytavinstituc_calle());
                entity.setUzytavinstituc_telefono(dtoUzyTavInstituc.getUzytavinstituc_telefono());
                entity.setUzytavinstituc_nomrepl(dtoUzyTavInstituc.getUzytavinstituc_nomrepl());
                entity.setUzytavinstituc_cargorl(dtoUzyTavInstituc.getUzytavinstituc_cargorl());
                entity.setUzytavinstituc_apellirl(dtoUzyTavInstituc.getUzytavinstituc_apellirl());
                entity.setUzytavinstituc_telrepl(dtoUzyTavInstituc.getUzytavinstituc_telrepl());
                entity.setUzytavinstituc_mailrl1(dtoUzyTavInstituc.getUzytavinstituc_mailrl1());
                entity.setUzytavinstituc_fecha_crea(dtoUzyTavInstituc.getUzytavinstituc_fecha_crea());
                if (dtoUzyTavInstituc.getUzytavinstituc_estado() !=null){
                    entity.setUzytavinstituc_estado(dtoUzyTavInstituc.getUzytavinstituc_estado());
                }
                if (dtoUzyTavInstituc.getUzytparroquia_id() != null) {
                    ModelUzyTParroquia parroquia = entityManager.find(ModelUzyTParroquia.class, dtoUzyTavInstituc.getUzytparroquia_id());
                    entity.setUzytparroquia(parroquia);
                }
                ModelUzyTavInstituc updatedEntity = daoUzyTavInstituc.save(entity);
                return mapper.entityToDto(updatedEntity);
            } else {
                throw new ResourceNotFoundException("No se encontró el registro con ID: " + id);
            }
        } catch (Exception e) {
            throw new ResourceNotFoundException("Error al editar el registro: " + e.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        ModelUzyTavInstituc entity = daoUzyTavInstituc.findById(id).orElse(null);
        if (entity != null) {
            entity.setUzytavinstituc_estado("I"); // Se cambia el estado a (I) inactivo
            daoUzyTavInstituc.save(entity);
            return true;
        }
        return false;
    }


}
