package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavOpcionInstitucion;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavOpcionInstitucion;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavOpcionInstitucion;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavOpcionInstitucionMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavOpcionInstitucion;

import org.springframework.stereotype.Service;


import java.util.List;


@Service
public class ServiceUzyTavOpcionInstitucion implements IServiceUzyTavOpcionInstitucion {

    private final DaoUzyTavOpcionInstitucion daoUzyTavOpcionInstitucion;

    private final UzyTavOpcionInstitucionMapper mapper;


    public ServiceUzyTavOpcionInstitucion(DaoUzyTavOpcionInstitucion daoUzyTavOpcionInstitucion,
                                          UzyTavOpcionInstitucionMapper mapper) {
        this.daoUzyTavOpcionInstitucion = daoUzyTavOpcionInstitucion;
        this.mapper = mapper;

    }

    @Override
    public DtoUzyTavOpcionInstitucion ListarPorID(Long id) {
        ModelUzyTavOpcionInstitucion entity = daoUzyTavOpcionInstitucion.findById(id)
                .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
        return mapper.entityToDto(entity);
    }

    @Override
    public List<DtoUzyTavOpcionInstitucion> ListarRegistros() {
        List<ModelUzyTavOpcionInstitucion> entities = daoUzyTavOpcionInstitucion.findAll();
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public List<DtoUzyTavOpcionInstitucion> ListarRegistrosRelacionadosConPrograma(Long programaId) {
        List<ModelUzyTavOpcionInstitucion> entities = daoUzyTavOpcionInstitucion.obtenerOpcionesInstitucionPorPrograma(programaId);
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public DtoUzyTavOpcionInstitucion guardar(DtoUzyTavOpcionInstitucion dtoUzyTavOpcionInstitucion) {
        ModelUzyTavOpcionInstitucion entity = mapper.dtoToEntity(dtoUzyTavOpcionInstitucion);
        ModelUzyTavOpcionInstitucion nuevoEntity = daoUzyTavOpcionInstitucion.save(entity);
        return mapper.entityToDto(nuevoEntity);
    }

    @Override
    public DtoUzyTavOpcionInstitucion editar(Long id, DtoUzyTavOpcionInstitucion dtoUzyTavOpcionInstitucion) {
        try {
            ModelUzyTavOpcionInstitucion dato = daoUzyTavOpcionInstitucion.findById(id).get();
            if (dato != null) {

                dato.setUzytavopcion_institucion_id(dtoUzyTavOpcionInstitucion.getUzytavopcion_institucion_id());
                dato.setUzytavopcion_institucionorden(dtoUzyTavOpcionInstitucion.getUzytavopcion_institucionorden());
                dato.setUzytavopcion_institucionnombre(dtoUzyTavOpcionInstitucion.getUzytavopcion_institucionnombre());
                dato.setUzytavopcion_institucionfecha_crea(
                        dtoUzyTavOpcionInstitucion.getUzytavopcion_institucionfecha_crea());
                dato.setUzytavopcion_instituciondireccion(
                        dtoUzyTavOpcionInstitucion.getUzytavopcion_instituciondireccion());
                dato.setUzytavopcion_institucionciudad(dtoUzyTavOpcionInstitucion.getUzytavopcion_institucionciudad());
                dato.setUzytavopcion_instituciontelefono(
                        dtoUzyTavOpcionInstitucion.getUzytavopcion_instituciontelefono());
                dato.setUzytavopcion_institucionpaguina_web(
                        dtoUzyTavOpcionInstitucion.getUzytavopcion_institucionpaguina_web());
                dato.setUzytavopcion_institucionmail(dtoUzyTavOpcionInstitucion.getUzytavopcion_institucionmail());
                dato.setUzytavopcion_institucionfax(dtoUzyTavOpcionInstitucion.getUzytavopcion_institucionfax());
                dato.setUzytavopcion_institucionrepresenta_legal(
                        dtoUzyTavOpcionInstitucion.getUzytavopcion_institucionrepresenta_legal());
                dato.setUzytavopcion_institucioncedula_rl(
                        dtoUzyTavOpcionInstitucion.getUzytavopcion_institucioncedula_rl());
                dato.setUzytavopcion_institucionorgano_ejecutor(
                        dtoUzyTavOpcionInstitucion.getUzytavopcion_institucionorgano_ejecutor());

                ModelUzyTavOpcionInstitucion updatedEntity = daoUzyTavOpcionInstitucion.save(dato);
                return mapper.entityToDto(updatedEntity);
            } else {
                throw new ResourceNotFoundException("No se encontró el registro con ID: " + id);
            }
        } catch (Exception e) {
            throw new ResourceNotFoundException("Error al editar el registro: " + e.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        ModelUzyTavOpcionInstitucion entity = daoUzyTavOpcionInstitucion.findById(id).orElse(null);
        if (entity != null) {
            daoUzyTavOpcionInstitucion.delete(entity);
            return true;
        }
        return false;
    }
}
