package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavParaEva;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavParaEva;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavParaEva;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavParaEvaMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavParaEva;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PathVariable;

import javax.transaction.Transactional;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Transactional
public class ServiceUzyTavParaEva implements IServiceUzyTavParaEva {

    private final DaoUzyTavParaEva daoUzyTavParaEva;

    private final UzyTavParaEvaMapper mapper;

    public ServiceUzyTavParaEva(DaoUzyTavParaEva daoUzyTavParaEva, UzyTavParaEvaMapper mapper) {
        this.daoUzyTavParaEva = daoUzyTavParaEva;
        this.mapper = mapper;
    }

    @Override
    public DtoUzyTavParaEva ListarPorID(@PathVariable Long id) {
        ModelUzyTavParaEva entity = daoUzyTavParaEva.findById(id)
                .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
        return mapper.entityToDto(entity);
    }

    @Override
    public List<DtoUzyTavParaEva> getAllDatosConParaEva() {
        List<ModelUzyTavParaEva> entities = daoUzyTavParaEva.getAllDatosConParaEva();
        return mapper.entitiesToDtos(entities);

    }

    @Override
    public List<DtoUzyTavParaEva> findByIdDatosConParaEvaTipo1(Long id) {
        List<ModelUzyTavParaEva> entities = daoUzyTavParaEva.findByIdDatosConParaEvaTipo1(id);
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public List<DtoUzyTavParaEva> findByIdDatosConParaEvaTipo2(Long id) {
        List<ModelUzyTavParaEva> entities = daoUzyTavParaEva.findByIdDatosConParaEvaTipo2(id);
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public List<DtoUzyTavParaEva> findByDatosUzytavcabparaevafId(Long id) {
        List<ModelUzyTavParaEva> entities = daoUzyTavParaEva.findByDatosUzytavcabparaevafId(id);
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public List<DtoUzyTavParaEva> ListarRegistros() {
        List<ModelUzyTavParaEva> entities = daoUzyTavParaEva.findAll();
        return entities.stream()
                .map(mapper::entityToDto)
                .collect(Collectors.toList());
    }

    @Override
    public DtoUzyTavParaEva guardar(DtoUzyTavParaEva dtoUzyTavParaEva) {
        ModelUzyTavParaEva entity = mapper.dtoToEntity(dtoUzyTavParaEva);
        ModelUzyTavParaEva nuevoEntity = daoUzyTavParaEva.save(entity);
        return mapper.entityToDto(nuevoEntity);
    }

    @Override
    public DtoUzyTavParaEva editar(@PathVariable Long id, DtoUzyTavParaEva dtoUzyTavParaEva) {
        try {
            ModelUzyTavParaEva entity = daoUzyTavParaEva.findById(id).get();
            if (entity != null) {

                entity.setUzytavparaeva_descrip(dtoUzyTavParaEva.getUzytavparaeva_descrip());
                entity.setUzytavparaeva_orden(dtoUzyTavParaEva.getUzytavparaeva_orden());
                entity.setUzytavparaeva_opcmenu(dtoUzyTavParaEva.getUzytavparaeva_opcmenu());
                entity.setUzytavcabparaevaf_id(dtoUzyTavParaEva.getUzytavcabparaevaf_id());
                entity.setUzytavparaeva_aspeva(dtoUzyTavParaEva.getUzytavparaeva_aspeva());
                entity.setUzytavparaeva_infoadic(dtoUzyTavParaEva.getUzytavparaeva_infoadic());
                entity.setUzytavparaeva_preginfoadic(dtoUzyTavParaEva.getUzytavparaeva_preginfoadic());
                ModelUzyTavParaEva updatedEntity = daoUzyTavParaEva.save(entity);
                return mapper.entityToDto(updatedEntity);
            } else {
                throw new ResourceNotFoundException("No se encontró el registro con ID: " + id);
            }
        } catch (Exception e) {
            throw new ResourceNotFoundException("Error al editar el registro: " + e.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        ModelUzyTavParaEva entity = daoUzyTavParaEva.findById(id).orElse(null);
        if (entity != null) {
            daoUzyTavParaEva.delete(entity);
            return true;
        }
        return false;
    }
}
