package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavPresup;


import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavPresup;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavPresup;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.InvalidArgumentException;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavPresupMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavPresup;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class ServiceUzyTavPresup implements IServiceUzyTavPresup {
    private static final String MESSAGE = "Usuario no encontrado con ID: ";

    private final DaoUzyTavPresup daoUzyTavPresup;

    private final UzyTavPresupMapper mapper;

    public ServiceUzyTavPresup(DaoUzyTavPresup daoUzyTavPresup, UzyTavPresupMapper mapper) {
        this.daoUzyTavPresup = daoUzyTavPresup;
        this.mapper = mapper;
    }

    @Override
    public DtoUzyTavPresup ListarPorID(Long id) {
        ModelUzyTavPresup entity = daoUzyTavPresup.findById(id)
                .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
        return mapper.entityToDto(entity);
    }

    @Override
    public List<DtoUzyTavPresup> ListarRegistros() {
        List<ModelUzyTavPresup> entities = daoUzyTavPresup.findAll();
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public List<DtoUzyTavPresup> obtenerTavPresupPorProyecID(Long proyecID) {
        List<ModelUzyTavPresup> entities = daoUzyTavPresup.obtenerTavPresupPorProyecID(proyecID);
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public List<DtoUzyTavPresup> obtenerTavPresupPorProyecIDandTipo1(Long proyecID) {
        List<ModelUzyTavPresup> entities = daoUzyTavPresup.obtenerTavPresupPorProyecIDandTipo1(proyecID);
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public List<DtoUzyTavPresup> obtenerTavPresupPorProyecIDandTipo2(Long proyecID) {
        List<ModelUzyTavPresup> entities = daoUzyTavPresup.obtenerTavPresupPorProyecIDandTipo2(proyecID);
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public List<DtoUzyTavPresup> obtenerTavPresupPorProyecIDandTipo3(Long proyecID) {
        List<ModelUzyTavPresup> entities = daoUzyTavPresup.obtenerTavPresupPorProyecIDandTipo3(proyecID);
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public DtoUzyTavPresup guardar(DtoUzyTavPresup dtoUzyTavPresup) {
        ModelUzyTavPresup entity = mapper.dtoToEntity(dtoUzyTavPresup);
        ModelUzyTavPresup nuevoEntity = daoUzyTavPresup.save(entity);
        return mapper.entityToDto(nuevoEntity);
    }

    @Override
    public DtoUzyTavPresup editar(Long id, DtoUzyTavPresup dtoUzyTavPresup) {
        if (id <= 0) {
            throw new InvalidArgumentException("El ID del usuario debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTavPresup entity = daoUzyTavPresup.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));

            if (dtoUzyTavPresup.getUzytavpresup_valor() != null) {
                entity.setUzytavpresup_valor(dtoUzyTavPresup.getUzytavpresup_valor());
            }
            if (dtoUzyTavPresup.getUzytavpresup_esptecnic() != null) {
                entity.setUzytavpresup_esptecnic(dtoUzyTavPresup.getUzytavpresup_esptecnic());
            }
            if (dtoUzyTavPresup.getUzytavpresup_tipogasto() != null) {
                entity.setUzytavpresup_tipogasto(dtoUzyTavPresup.getUzytavpresup_tipogasto());
            }
            if (dtoUzyTavPresup.getUzytavpresup_cantidad() != null) {
                entity.setUzytavpresup_cantidad(dtoUzyTavPresup.getUzytavpresup_cantidad());
            }
            if (dtoUzyTavPresup.getUzytavpresup_bien_servicio() != null) {
                entity.setUzytavpresup_bien_servicio(dtoUzyTavPresup.getUzytavpresup_bien_servicio());
            }
            if (dtoUzyTavPresup.getUzytavpresup_tipo() != null) {
                entity.setUzytavpresup_tipo(dtoUzyTavPresup.getUzytavpresup_tipo());
            }
            if (dtoUzyTavPresup.getUzytavpresup_total_ejec() != null) {
                entity.setUzytavpresup_total_ejec(dtoUzyTavPresup.getUzytavpresup_total_ejec());
            }
            if (dtoUzyTavPresup.getUzytavpresup_total_ejec() != null) {
                entity.setUzytavpresup_valoranual(dtoUzyTavPresup.getUzytavpresup_valoranual());
            }

            ModelUzyTavPresup updatedEntity = daoUzyTavPresup.save(entity);
            return mapper.entityToDto(updatedEntity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al editar el registro -> " + ex.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        ModelUzyTavPresup entity = daoUzyTavPresup.findById(id).orElse(null);
        if (entity != null) {
            daoUzyTavPresup.delete(entity);
            return true;
        }
        return false;
    }
}
