package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavProgObj;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavProgObj;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavProgObj;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavProgObjMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavProgObj;

import org.springframework.stereotype.Service;
import java.util.List;


@Service
public class ServiceUzyTavProgObj implements IServiceUzyTavProgObj {

    private final DaoUzyTavProgObj daoUzyTavProgObj;
    private final UzyTavProgObjMapper mapper;

    public ServiceUzyTavProgObj(DaoUzyTavProgObj daoUzyTavProgObj, UzyTavProgObjMapper mapper) {
        this.daoUzyTavProgObj = daoUzyTavProgObj;
        this.mapper = mapper;
    }

    @Override
    public DtoUzyTavProgObj ListarPorID(Long id) {
        ModelUzyTavProgObj entity = daoUzyTavProgObj.findById(id)
                .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
        return mapper.entityToDto(entity);
    }

    @Override
    public List<DtoUzyTavProgObj> ListarRegistros() {
        List<ModelUzyTavProgObj> entities = daoUzyTavProgObj.findAll();
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public List<DtoUzyTavProgObj> obtenerTavProgObjPorPrograma(Long programaId) {
        List<ModelUzyTavProgObj> entities = daoUzyTavProgObj.obtenerTavProgObjPorPrograma(programaId);
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public List<DtoUzyTavProgObj> obtenerTavProgObjPorProyecto(Long proyectoID) {
        List<ModelUzyTavProgObj> entities = daoUzyTavProgObj.obtenerTavProgObjPorProyecto(proyectoID);
        return mapper.entitiesToDtos(entities);
    }


    @Override
    public DtoUzyTavProgObj guardar(DtoUzyTavProgObj dtoUzyTavProgObj) {
        ModelUzyTavProgObj entity = mapper.dtoToEntity(dtoUzyTavProgObj);
        ModelUzyTavProgObj nuevoEntity = daoUzyTavProgObj.save(entity);
        return mapper.entityToDto(nuevoEntity);
    }

    @Override
    public DtoUzyTavProgObj editar(Long id, DtoUzyTavProgObj dtoUzyTavProgObj) {
        try {
            ModelUzyTavProgObj entity = daoUzyTavProgObj.findById(id).get();
            // Seteamos los nuevos datos del registro
            if (entity != null) {

                ModelUzyTavProgObj updatedEntity = daoUzyTavProgObj.save(entity);
                return mapper.entityToDto(updatedEntity);
            } else {
                throw new ResourceNotFoundException("No se encontró el registro con ID: " + id);
            }
        } catch (Exception e) {
            throw new ResourceNotFoundException("Error al editar el registro: " + e.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        ModelUzyTavProgObj entity = daoUzyTavProgObj.findById(id).orElse(null);
        if (entity != null) {
            daoUzyTavProgObj.delete(entity);
            return true;
        }
        return false;
    }
}
