package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavProlin;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavProlin;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoSubLineaLinea;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavProlin;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavProlinMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavProlin;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class ServiceUzyTavProlin implements IServiceUzyTavProlin{

    @Autowired
     private DaoUzyTavProlin daoUzyTavProlin;

    @Autowired
    private UzyTavProlinMapper mapper;

    @Override
    public DtoUzyTavProlin ListarPorID(Long id) {
        ModelUzyTavProlin entity = daoUzyTavProlin.findById(id)
                .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
        return mapper.entityToDto(entity);
    }

    @Override
    public List<DtoUzyTavProlin> ListarRegistros() {
        List<ModelUzyTavProlin> entities = daoUzyTavProlin.findAll();
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public DtoUzyTavProlin guardar(DtoUzyTavProlin dtoUzyTavProlin) {
        ModelUzyTavProlin entity = mapper.dtoToEntity(dtoUzyTavProlin);
        ModelUzyTavProlin nuevoEntity = daoUzyTavProlin.save(entity);
        return mapper.entityToDto(nuevoEntity);
    }

    @Override
    public DtoUzyTavProlin editar(Long id, DtoUzyTavProlin dtoUzyTavProlin) {
        try {
            ModelUzyTavProlin entity = daoUzyTavProlin.findById(id).get();
            if (entity != null) {
                //Seteamos los nuevos datos del registro

                ModelUzyTavProlin updatedEntity = daoUzyTavProlin.save(entity);
                return mapper.entityToDto(updatedEntity);
            }else {
                throw new ResourceNotFoundException("No se encontró el registro con ID: " + id);
            }
        } catch (Exception e) {
            throw new ResourceNotFoundException("Error al editar el registro: " + e.getMessage());
        }
    }

    @Override
    public List<DtoUzyTavProlin> obtenerRegistrosPorSubLinea(Long sublineaID) {
        List<ModelUzyTavProlin> entities = daoUzyTavProlin.obtenerRegistrosPorSubLinea(sublineaID);
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public List<DtoUzyTavProlin> obtenerTavProlinPorProyecID(Long proyecID) {
        List<ModelUzyTavProlin> entities = daoUzyTavProlin.obtenerTavProlinPorProyecID(proyecID);
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public List<DtoSubLineaLinea> obtenerSubLineaLinea(Long proyecID) {
        return daoUzyTavProlin.obtenerSubLineaLinea(proyecID);
    }

    @Override
    public boolean eliminar(Long id) {
        ModelUzyTavProlin entity = daoUzyTavProlin.findById(id).orElse(null);
        if (entity != null) {
            daoUzyTavProlin.delete(entity);
            return true;
        }
        return false;
    }
}
