package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavProyec;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTPerfil;
import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavCabComca;
import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavConparaEva;
import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavProyec;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoCabcomcaIdConparaevaID;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavProyec;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.InvalidArgumentException;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavProyecMapper;

import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTPerfil;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavCabComca;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavConparaEva;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavProyec;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PathVariable;

import java.util.List;
import java.util.Optional;


@Service
public class ServiceUzyTavProyec implements IServiceUzyTavProyec {

    private final DaoUzyTavProyec daoUzyTavProyec;
    private final UzyTavProyecMapper mapper;
    private final DaoUzyTavCabComca daoUzyTavCabComca;
    private final DaoUzyTavConparaEva daoUzyTavConparaEva;
    private final DaoUzyTPerfil daoUzyTPerfil;

    public static final String MESSAGE = "No se encontró el registro con ID: ";

    public ServiceUzyTavProyec(DaoUzyTavProyec daoUzyTavProyec, UzyTavProyecMapper mapper, DaoUzyTavCabComca daoUzyTavCabComca, DaoUzyTavConparaEva daoUzyTavConparaEva, DaoUzyTPerfil daoUzyTPerfil) {
        this.daoUzyTavProyec = daoUzyTavProyec;
        this.mapper = mapper;
        this.daoUzyTavCabComca = daoUzyTavCabComca;
        this.daoUzyTavConparaEva = daoUzyTavConparaEva;
        this.daoUzyTPerfil = daoUzyTPerfil;
    }

    @Override
    public DtoUzyTavProyec ListarPorID(Long id) {
        ModelUzyTavProyec entity = daoUzyTavProyec.findById(id)
                .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
        return mapper.entityToDto(entity);
    }

    @Override
    public List<DtoCabcomcaIdConparaevaID> findCabcomcaIdConparaevaIdByProyecId(Long idProyect) {
        return daoUzyTavProyec.findCabcomcaIdConparaevaIdByProyecId(idProyect);
    }


    @Override
    public List<DtoUzyTavProyec> ListarRegistros() {
        List<ModelUzyTavProyec> entities = daoUzyTavProyec.findAll();
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public List<DtoUzyTavProyec> findAllByUserIdAndRole(Long userId) {
        Optional<ModelUzyTPerfil> directorProyecto = daoUzyTPerfil.findByUzytperfilNombre("Director de Proyecto", userId);
        Optional<ModelUzyTPerfil> administradorSistema = daoUzyTPerfil.findByUzytperfilNombre("Administrador del Sistema", userId);

        List<ModelUzyTavProyec> entities;
        try {
            if (directorProyecto.isPresent()) {
                entities = daoUzyTavProyec.findAllByUserIdAndRole(userId);
            } else if (administradorSistema.isPresent()) {
                entities = daoUzyTavProyec.findAll();
            } else {
                throw new RuntimeException(" El usuario no tiene los roles requeridos");
            }
        } catch (Exception e) {
            throw new InvalidArgumentException("Error al obtener los roles del usuario-> " + e.getMessage());
        }

        return mapper.entitiesToDtos(entities);
    }


    @Override
    public DtoUzyTavProyec guardar(DtoUzyTavProyec dtoUzyTavProyec) {
        ModelUzyTavProyec entity = mapper.dtoToEntity(dtoUzyTavProyec);
        ModelUzyTavProyec nuevoEntity = daoUzyTavProyec.save(entity);
        return mapper.entityToDto(nuevoEntity);
    }

    @Override
    public DtoUzyTavProyec editar(@PathVariable Long id, DtoUzyTavProyec dtoUzyTavProyec) {
        try {
            ModelUzyTavProyec entity = daoUzyTavProyec.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));

            if (entity == null) {
                throw new ResourceNotFoundException(MESSAGE + id);
            }

            actualizarRelaciones(entity, dtoUzyTavProyec);
            actualizarAtributos(entity, dtoUzyTavProyec);

            ModelUzyTavProyec updatedEntity = daoUzyTavProyec.save(entity);
            return mapper.entityToDto(updatedEntity);
        } catch (Exception e) {
            throw new ResourceNotFoundException(MESSAGE + e.getMessage());
        }
    }

    private void actualizarRelaciones(ModelUzyTavProyec entity, DtoUzyTavProyec dto) {
        if (dto.getUzytavcabcomca_id() != null) {
            ModelUzyTavCabComca nuevaRelacionCabComca = daoUzyTavCabComca.findById(dto.getUzytavcabcomca_id())
                    .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro de cabcomca con ID: " + dto.getUzytavcabcomca_id()));
            entity.setUzytavcabcomca(nuevaRelacionCabComca);
        }
        if (dto.getUzytavconparaeva_id() != null) {
            ModelUzyTavConparaEva nuevaRelacionConparaEva = daoUzyTavConparaEva.findById(dto.getUzytavconparaeva_id())
                    .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro de conparaeva con ID: " + dto.getUzytavconparaeva_id()));
            entity.setUzytavconparaeva(nuevaRelacionConparaEva);
        }
    }

    private void actualizarAtributos(ModelUzyTavProyec entity, DtoUzyTavProyec dtoUzyTavProyec) {
        //Operador ternario: condición ? valorSiVerdadero : valorSiFalso
        if (dtoUzyTavProyec.getUzytavproyec_nombre() != null) {
            entity.setUzytavproyec_nombre(dtoUzyTavProyec.getUzytavproyec_nombre());
        }
        if (dtoUzyTavProyec.getUzytavproyec_cierre_imp() != null) {
            entity.setUzytavproyec_cierre_imp(dtoUzyTavProyec.getUzytavproyec_cierre_imp());
        }
        if (dtoUzyTavProyec.getUzytavproyec_aporuni() != null) {
            entity.setUzytavproyec_aporuni(dtoUzyTavProyec.getUzytavproyec_aporuni());
        }
        if (dtoUzyTavProyec.getUzytavproyec_aporuni_valor() != null) {
            entity.setUzytavproyec_aporuni_valor(dtoUzyTavProyec.getUzytavproyec_aporuni_valor());
        }
        if (dtoUzyTavProyec.getUzytavproyec_entiaupi() != null) {
            entity.setUzytavproyec_entiaupi(dtoUzyTavProyec.getUzytavproyec_entiaupi());
        }
        if (dtoUzyTavProyec.getUzytavproyec_contraparte() != null) {
            entity.setUzytavproyec_contraparte(dtoUzyTavProyec.getUzytavproyec_contraparte());
        }
        if (dtoUzyTavProyec.getUzytavproyec_fecha_presenta() != null) {
            entity.setUzytavproyec_fecha_presenta(dtoUzyTavProyec.getUzytavproyec_fecha_presenta());
        }
        if (dtoUzyTavProyec.getUzytavproyec_fecha_inicio() != null) {
            entity.setUzytavproyec_fecha_inicio(dtoUzyTavProyec.getUzytavproyec_fecha_inicio());
        }
        if (dtoUzyTavProyec.getUzytavproyec_fecha_finaliza() != null) {
            entity.setUzytavproyec_fecha_finaliza(dtoUzyTavProyec.getUzytavproyec_fecha_finaliza());
        }
        if (dtoUzyTavProyec.getUzytavproyec_bienes() != null) {
            entity.setUzytavproyec_bienes(dtoUzyTavProyec.getUzytavproyec_bienes());
        }
        if (dtoUzyTavProyec.getUzytavproyec_servicios() != null) {
            entity.setUzytavproyec_servicios(dtoUzyTavProyec.getUzytavproyec_servicios());
        }
        if (dtoUzyTavProyec.getUzytavproyec_bien_servicio() != null) {
            entity.setUzytavproyec_bien_servicio(dtoUzyTavProyec.getUzytavproyec_bien_servicio());
        }
        if (dtoUzyTavProyec.getUzytavproyec_situactual() != null) {
            entity.setUzytavproyec_situactual(dtoUzyTavProyec.getUzytavproyec_situactual());
        }
        if (dtoUzyTavProyec.getUzytavproyec_diagnostico() != null) {
            entity.setUzytavproyec_diagnostico(dtoUzyTavProyec.getUzytavproyec_diagnostico());
        }
        if (dtoUzyTavProyec.getUzytavproyec_lineabase() != null) {
            entity.setUzytavproyec_lineabase(dtoUzyTavProyec.getUzytavproyec_lineabase());
        }
        if (dtoUzyTavProyec.getUzytavproyec_directos_hombres() != null) {
            entity.setUzytavproyec_directos_hombres(dtoUzyTavProyec.getUzytavproyec_directos_hombres());
        }
        if (dtoUzyTavProyec.getUzytavproyec_directos_mujeres() != null) {
            entity.setUzytavproyec_directos_mujeres(dtoUzyTavProyec.getUzytavproyec_directos_mujeres());
        }
        if (dtoUzyTavProyec.getUzytavproyec_directos_discapac() != null) {
            entity.setUzytavproyec_directos_discapac(dtoUzyTavProyec.getUzytavproyec_directos_discapac());
        }
        if (dtoUzyTavProyec.getUzytavproyec_inidirectos_hombres() != null) {
            entity.setUzytavproyec_inidirectos_hombres(dtoUzyTavProyec.getUzytavproyec_inidirectos_hombres());
        }
        if (dtoUzyTavProyec.getUzytavproyec_inidirectos_mujeres() != null) {
            entity.setUzytavproyec_inidirectos_mujeres(dtoUzyTavProyec.getUzytavproyec_inidirectos_mujeres());
        }
        if (dtoUzyTavProyec.getUzytavproyec_factoresc() != null) {
            entity.setUzytavproyec_factoresc(dtoUzyTavProyec.getUzytavproyec_factoresc());
        }
        if (dtoUzyTavProyec.getUzytavproyec_restriccion() != null) {
            entity.setUzytavproyec_restricion(dtoUzyTavProyec.getUzytavproyec_restriccion());
        }
        if (dtoUzyTavProyec.getUzytavproyec_viabilidad() != null) {
            entity.setUzytavproyec_viabilidad(dtoUzyTavProyec.getUzytavproyec_viabilidad());
        }
        if (dtoUzyTavProyec.getUzytavproyec_perderia() != null) {
            entity.setUzytavproyec_perderia(dtoUzyTavProyec.getUzytavproyec_perderia());
        }
        if (dtoUzyTavProyec.getUzytavproyec_resultados() != null) {
            entity.setUzytavproyec_resultados(dtoUzyTavProyec.getUzytavproyec_resultados());
        }
        if (dtoUzyTavProyec.getUzytavproyec_impacto() != null) {
            entity.setUzytavproyec_impactoa(dtoUzyTavProyec.getUzytavproyec_impacto());
        }
        if (dtoUzyTavProyec.getUzytavproyec_sostenibilidad() != null) {
            entity.setUzytavproyec_sostenibilidad(dtoUzyTavProyec.getUzytavproyec_sostenibilidad());
        }
        if (dtoUzyTavProyec.getUzytavproyec_inidirectos_discapa() != null) {
            entity.setUzytavproyec_inidirectos_discapa(dtoUzyTavProyec.getUzytavproyec_inidirectos_discapa());
        }
        if (dtoUzyTavProyec.getUzytavproyec_tipo_proy() != null) {
            entity.setUzytavproyec_tipo_proy(dtoUzyTavProyec.getUzytavproyec_tipo_proy());
        }
        if (dtoUzyTavProyec.getStvsubj_code() != null) {
            entity.setStvsubj_code(dtoUzyTavProyec.getStvsubj_code());
        }
        if (dtoUzyTavProyec.getUzytavproyec_codigo() != null) {
            entity.setUzytavproyec_codigo(dtoUzyTavProyec.getUzytavproyec_codigo());
        }
        if (dtoUzyTavProyec.getUzytavproyec_aprob_con_dep_nom() != null) {
            entity.setUzytavproyec_aprob_con_dep_nom(dtoUzyTavProyec.getUzytavproyec_aprob_con_dep_nom());
        }
        if (dtoUzyTavProyec.getUzytavproyec_aprob_con_dep_url() != null) {
            entity.setUzytavproyec_aprob_con_dep_url(dtoUzyTavProyec.getUzytavproyec_aprob_con_dep_url());
        }
        if (dtoUzyTavProyec.getUzytavproyec_estado() != null) {
            entity.setUzytavproyec_estado(dtoUzyTavProyec.getUzytavproyec_estado());
        }
        if (dtoUzyTavProyec.getUzytavproyec_fecha_c() != null) {
            entity.setUzytavproyec_fecha_c(dtoUzyTavProyec.getUzytavproyec_fecha_c());
        }
        if (dtoUzyTavProyec.getUzytavproyec_url_pdf_acta() != null) {
            entity.setUzytavproyec_url_pdf_acta(dtoUzyTavProyec.getUzytavproyec_url_pdf_acta());
        }
        if (dtoUzyTavProyec.getUzytavproyec_observacio_c() != null) {
            entity.setUzytavproyec_observacio_c(dtoUzyTavProyec.getUzytavproyec_observacio_c());
        }
        if (dtoUzyTavProyec.getUzytavproyec_mensaje_correo() != null) {
            entity.setUzytavproyec_mensaje_correo(dtoUzyTavProyec.getUzytavproyec_mensaje_correo());
        }
        if (dtoUzyTavProyec.getUzytavproyec_asunto_correo() != null) {
            entity.setUzytavproyec_asunto_correo(dtoUzyTavProyec.getUzytavproyec_asunto_correo());
        }
        if (dtoUzyTavProyec.getUzytavproyec_direccion_correo() != null) {
            entity.setUzytavproyec_direccion_correo(dtoUzyTavProyec.getUzytavproyec_direccion_correo());
        }
        if (dtoUzyTavProyec.getUzytavproyec_directos_ejec() != null) {
            entity.setUzytavproyec_directos_ejec(dtoUzyTavProyec.getUzytavproyec_directos_ejec());
        }
        if (dtoUzyTavProyec.getUzytavproyec_situactual2() != null) {
            entity.setUzytavproyec_situactual2(dtoUzyTavProyec.getUzytavproyec_situactual2());
        }
        if (dtoUzyTavProyec.getUzytavproyec_nombre_pdf_memo() != null) {
            entity.setUzytavproyec_nombre_pdf_memo(dtoUzyTavProyec.getUzytavproyec_nombre_pdf_memo());
        }
        if (dtoUzyTavProyec.getUzytavproyec_url_pdf_memo() != null) {
            entity.setUzytavproyec_url_pdf_memo(dtoUzyTavProyec.getUzytavproyec_url_pdf_memo());
        }
        if (dtoUzyTavProyec.getUzytavconparaeva_adjunto_acta() != null) {
            entity.setUzytavconparaeva_adjunto_acta(dtoUzyTavProyec.getUzytavconparaeva_adjunto_acta());
        }
        if (dtoUzyTavProyec.getUzytavconparaeva_adjunto_memo() != null) {
            entity.setUzytavconparaeva_adjunto_memo(dtoUzyTavProyec.getUzytavconparaeva_adjunto_memo());
        }
        if (dtoUzyTavProyec.getUzytavproyec_cobertura() != null) {
            entity.setUzytavproyec_cobertura(dtoUzyTavProyec.getUzytavproyec_cobertura());
        }
        if (dtoUzyTavProyec.getUzytavproyec_directos_ninios() != null) {
            entity.setUzytavproyec_directos_ninios(dtoUzyTavProyec.getUzytavproyec_directos_ninios());
        }
        if (dtoUzyTavProyec.getUzytavproyec_indirectos_ninios() != null) {
            entity.setUzytavproyec_indirectos_ninios(dtoUzyTavProyec.getUzytavproyec_indirectos_ninios());
        }
        if (dtoUzyTavProyec.getUzytavproyec_cierre_prod() != null) {
            entity.setUzytavproyec_cierre_prod(dtoUzyTavProyec.getUzytavproyec_cierre_prod());
        }
        if (dtoUzyTavProyec.getUzytavproyec_cierre_bien() != null) {
            entity.setUzytavproyec_cierre_bien(dtoUzyTavProyec.getUzytavproyec_cierre_bien());
        }
        if (dtoUzyTavProyec.getUzytavproyec_cierre_resul() != null) {
            entity.setUzytavproyec_cierre_resul(dtoUzyTavProyec.getUzytavproyec_cierre_resul());
        }
        if (dtoUzyTavProyec.getUzytavproyec_cierre_aplic() != null) {
            entity.setUzytavproyec_cierre_aplic(dtoUzyTavProyec.getUzytavproyec_cierre_aplic());
        }
        if (dtoUzyTavProyec.getUzytavproyec_cierre_rest() != null) {
            entity.setUzytavproyec_cierre_rest(dtoUzyTavProyec.getUzytavproyec_cierre_rest());
        }
        if (dtoUzyTavProyec.getUzytavproyec_cierre_comen() != null) {
            entity.setUzytavproyec_cierre_comen(dtoUzyTavProyec.getUzytavproyec_cierre_comen());
        }
        if (dtoUzyTavProyec.getUzytavproyec_fecha_v() != null) {
            entity.setUzytavproyec_fecha_v(dtoUzyTavProyec.getUzytavproyec_fecha_v());
        }
        if (dtoUzyTavProyec.getUzytavproyec_observacio_v() != null) {
            entity.setUzytavproyec_observacio_v(dtoUzyTavProyec.getUzytavproyec_observacio_v());
        }
        if (dtoUzyTavProyec.getUzytavproyec_estado_activo() != null) {
            entity.setUzytavproyec_estado_activo(dtoUzyTavProyec.getUzytavproyec_estado_activo());
        }
        if (dtoUzyTavProyec.getUzytavproyec_fecha_cierre() != null) {
            entity.setUzytavproyec_fecha_cierre(dtoUzyTavProyec.getUzytavproyec_fecha_cierre());
        }
        if (dtoUzyTavProyec.getUzytavproyec_fecha_evaluacion() != null) {
            entity.setUzytavproyec_fecha_evaluacion(dtoUzyTavProyec.getUzytavproyec_fecha_evaluacion());
        }
        if (dtoUzyTavProyec.getUzytavproyec_fecha_mail_avance() != null) {
            entity.setUzytavproyec_fecha_mail_avance(dtoUzyTavProyec.getUzytavproyec_fecha_mail_avance());
        }
        if (dtoUzyTavProyec.getUzytavproyec_lineabase2() != null) {
            entity.setUzytavproyec_lineabase2(dtoUzyTavProyec.getUzytavproyec_lineabase2());
        }
        if (dtoUzyTavProyec.getUzytavproyec_indirectos() != null) {
            entity.setUzytavproyec_indirectos(dtoUzyTavProyec.getUzytavproyec_indirectos());
        }
        if (dtoUzyTavProyec.getStvcamp_code() != null) {
            entity.setStvcamp_code(dtoUzyTavProyec.getStvcamp_code());
        }
        if (dtoUzyTavProyec.getStvmajr_code() != null) {
            entity.setStvmajr_code(dtoUzyTavProyec.getStvmajr_code());
        }
        if (dtoUzyTavProyec.getUzytavproyec_com_part1_nom() != null) {
            entity.setUzytavproyec_com_part1_nom(dtoUzyTavProyec.getUzytavproyec_com_part1_nom());
        }
        if (dtoUzyTavProyec.getUzytavproyec_com_part1_url() != null) {
            entity.setUzytavproyec_com_part1_url(dtoUzyTavProyec.getUzytavproyec_com_part1_url());
        }
        if (dtoUzyTavProyec.getUzytavproyec_com_part2_nom() != null) {
            entity.setUzytavproyec_com_part2_nom(dtoUzyTavProyec.getUzytavproyec_com_part2_nom());
        }
        if (dtoUzyTavProyec.getUzytavproyec_com_part2_url() != null) {
            entity.setUzytavproyec_com_part2_url(dtoUzyTavProyec.getUzytavproyec_com_part2_url());
        }
        if (dtoUzyTavProyec.getUzytavproyec_fecha_finp() != null) {
            entity.setUzytavproyec_fecha_finp(dtoUzyTavProyec.getUzytavproyec_fecha_finp());
        }
        if (dtoUzyTavProyec.getUzytavproyec_aprob_con_dep_nro() != null) {
            entity.setUzytavproyec_aprob_con_dep_nro(dtoUzyTavProyec.getUzytavproyec_aprob_con_dep_nro());
        }
        if (dtoUzyTavProyec.getUzytavproyec_aprob_con_dep_fec() != null) {
            entity.setUzytavproyec_aprob_con_dep_fec(dtoUzyTavProyec.getUzytavproyec_aprob_con_dep_fec());
        }
        if (dtoUzyTavProyec.getUzytavproyec_aprob_viitt_nom() != null) {
            entity.setUzytavproyec_aprob_viitt_nom(dtoUzyTavProyec.getUzytavproyec_aprob_viitt_nom());
        }
        if (dtoUzyTavProyec.getUzytavproyec_aprob_viitt_url() != null) {
            entity.setUzytavproyec_aprob_viitt_url(dtoUzyTavProyec.getUzytavproyec_aprob_viitt_url());
        }
        if (dtoUzyTavProyec.getUzytavproyec_aprob_viitt_nro() != null) {
            entity.setUzytavproyec_aprob_viitt_nro(dtoUzyTavProyec.getUzytavproyec_aprob_viitt_nro());
        }
        if (dtoUzyTavProyec.getUzytavproyec_aprob_viitt_fec() != null) {
            entity.setUzytavproyec_aprob_viitt_fec(dtoUzyTavProyec.getUzytavproyec_aprob_viitt_fec());
        }
        if (dtoUzyTavProyec.getUzytavproyec_cons_acad_nroacta() != null) {
            entity.setUzytavproyec_cons_acad_nroacta(dtoUzyTavProyec.getUzytavproyec_cons_acad_nroacta());
        }
        if (dtoUzyTavProyec.getUzytavproyec_carta_com_url() != null) {
            entity.setUzytavproyec_carta_com_url(dtoUzyTavProyec.getUzytavproyec_carta_com_url());
        }
        if (dtoUzyTavProyec.getUzytavproyec_carta_com_nom() != null) {
            entity.setUzytavproyec_carta_com_nom(dtoUzyTavProyec.getUzytavproyec_carta_com_nom());
        }
        if (dtoUzyTavProyec.getUzytavproyec_fecha_fin_ejec() != null) {
            entity.setUzytavproyec_fecha_fin_ejec(dtoUzyTavProyec.getUzytavproyec_fecha_fin_ejec());
        }
        if (dtoUzyTavProyec.getUzytavproyec_fecmax_correc() != null) {
            entity.setUZYTAVPROYEC_FECMAX_CORREC(dtoUzyTavProyec.getUzytavproyec_fecmax_correc());
        }
        if (dtoUzyTavProyec.getUzytavproyec_secuencial() != null) {
            entity.setUzytavproyec_secuencial(dtoUzyTavProyec.getUzytavproyec_secuencial());
        }
        if (dtoUzyTavProyec.getUzytavproyec_efecto_esperado() != null) {
            entity.setUzytavproyec_efecto_esperado(dtoUzyTavProyec.getUzytavproyec_efecto_esperado());
        }
        if (dtoUzyTavProyec.getUzytavproyec_tipo() != null) {
            entity.setUzytavproyec_tipo(dtoUzyTavProyec.getUzytavproyec_tipo());
        }
        if (dtoUzyTavProyec.getUzytavproyec_descripcion_pobl() != null) {
            entity.setUzytavproyec_descripcion_pobl(dtoUzyTavProyec.getUzytavproyec_descripcion_pobl());
        }
        if (dtoUzyTavProyec.getUzytavproyec_com_part1_nom() != null) {
            entity.setUzytavproyec_com_part3_nom(dtoUzyTavProyec.getUzytavproyec_com_part1_nom());
        }
        if (dtoUzyTavProyec.getUzytavproyec_com_part3_url() != null) {
            entity.setUzytavproyec_com_part3_url(dtoUzyTavProyec.getUzytavproyec_com_part3_url());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        ModelUzyTavProyec entity = daoUzyTavProyec.findById(id).orElse(null);
        if (entity != null) {
            daoUzyTavProyec.delete(entity);
            return true;
        }
        return false;
    }
}
