package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavSeguim;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavSeguim;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavSeguim;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.InvalidArgumentException;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavSeguimMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavSeguim;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class ServiceUzyTavSeguim implements IServiceUzyTavSeguim{
    private static final String MESSAGE = "No se encontró el registro con ID: ";

    private final DaoUzyTavSeguim daoUzyTavSeguim;
    private final UzyTavSeguimMapper mapper;

    public ServiceUzyTavSeguim(DaoUzyTavSeguim daoUzyTavSeguim, UzyTavSeguimMapper mapper) {
        this.daoUzyTavSeguim = daoUzyTavSeguim;
        this.mapper = mapper;
    }

    @Override
    public DtoUzyTavSeguim ListarPorID(Long id) {
        if (id <= 0) {
            throw new IllegalArgumentException("El parámetro 'id' debe ser un valor positivo.");
        }
        try {
            ModelUzyTavSeguim entity = daoUzyTavSeguim.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
            return mapper.entityToDto(entity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) { // Manejo de cualquier error en el servidor
            throw new RuntimeException("Error al buscar el registro con ID: " + id);
        }
    }

    @Override
    public List<DtoUzyTavSeguim> ListarRegistros() {
        try {
            List<ModelUzyTavSeguim> entities = daoUzyTavSeguim.findAll();
            return mapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar los registros: " + ex.getMessage());
        }
    }
    @Override
    public DtoUzyTavSeguim guardar(DtoUzyTavSeguim dtoUzyTavSeguim) {
        try {
            ModelUzyTavSeguim entity = mapper.dtoToEntity(dtoUzyTavSeguim);
            ModelUzyTavSeguim nuevoEntity = daoUzyTavSeguim.save(entity);
            return mapper.entityToDto(nuevoEntity);
        }
        catch (Exception ex) {
            throw new RuntimeException(MESSAGE + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTavSeguim editar(Long id, DtoUzyTavSeguim dtoUzyTavSeguim) {
        if (id <= 0) {
            throw new InvalidArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTavSeguim entity = daoUzyTavSeguim.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
            if (dtoUzyTavSeguim.getUzytavseguim_descrip() != null) {
                entity.setUzytavseguim_descrip(dtoUzyTavSeguim.getUzytavseguim_descrip());
            }
            if (dtoUzyTavSeguim.getUzytavseguim_fecha_logro() != null) {
                entity.setUzytavseguim_fecha_logro(dtoUzyTavSeguim.getUzytavseguim_fecha_logro());
            }
            if (dtoUzyTavSeguim.getUzytavseguim_logro() != null) {
                entity.setUzytavseguim_logro(dtoUzyTavSeguim.getUzytavseguim_logro());
            }

            ModelUzyTavSeguim registroActualizado = daoUzyTavSeguim.save(entity);
            return mapper.entityToDto(registroActualizado);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al editar el usuario -> " + ex.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        if (id == null || id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTavSeguim entity = daoUzyTavSeguim.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException(MESSAGE + id));
            if (entity != null) {
                daoUzyTavSeguim.delete(entity);
                return true;
            }
            return false;
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al eliminar el registro: " + ex.getMessage());
        }
    }
}
