package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavTipres;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavTipres;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavTipres;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavTipresMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavTipres;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class ServiceUzyTavTipres implements IServiceUzyTavTipres{
    private final DaoUzyTavTipres daoUzyTavTipres;
    private final UzyTavTipresMapper mapper;

    public ServiceUzyTavTipres(DaoUzyTavTipres daoUzyTavTipres, UzyTavTipresMapper mapper) {
        this.daoUzyTavTipres = daoUzyTavTipres;
        this.mapper = mapper;
    }

    @Override
    public DtoUzyTavTipres ListarPorID(Long id) {
        if (id <= 0) {
            throw new IllegalArgumentException("El parámetro 'id' debe ser un valor positivo.");
        }
        try {
            ModelUzyTavTipres entity = daoUzyTavTipres.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
            return mapper.entityToDto(entity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) { // Manejo de cualquier error en el servidor
            throw new RuntimeException("Error al buscar el registro con ID: " + id);
        }
    }

    @Override
    public List<DtoUzyTavTipres> ListarRegistros() {
        try {
            List<ModelUzyTavTipres> entities = daoUzyTavTipres.findAll();
            return mapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException("Error al listar los registros: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTavTipres guardar(DtoUzyTavTipres dtoUzyTavTipres) {
        try {
            ModelUzyTavTipres entity = mapper.dtoToEntity(dtoUzyTavTipres);
            ModelUzyTavTipres nuevoEntity = daoUzyTavTipres.save(entity);
            return mapper.entityToDto(nuevoEntity);
        }
        catch (Exception ex) {
            throw new RuntimeException("Error al guardar el registro: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTavTipres editar(Long id, DtoUzyTavTipres dtoUzyTavTipres) {
        if (id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTavTipres entity = daoUzyTavTipres.findById(id).get();
            if (entity != null) {
                if(dtoUzyTavTipres.getUzytavtipres_descrip()!=null){
                    entity.setUzytavtipres_descrip(dtoUzyTavTipres.getUzytavtipres_descrip());
                }
                if(dtoUzyTavTipres.getUzytavtipres_orden() !=null){
                    entity.setUzytavtipres_orden(dtoUzyTavTipres.getUzytavtipres_orden());
                }


                ModelUzyTavTipres updatedEntity = daoUzyTavTipres.save(entity);
                return mapper.entityToDto(updatedEntity);
            }else {
                throw new ResourceNotFoundException("No se encontró el registro con ID: " + id);
            }
        } catch (Exception e) {
            throw new ResourceNotFoundException("Error al editar el registro: " + e.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        if (id == null || id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {

            ModelUzyTavTipres entity = daoUzyTavTipres.findById(id).orElseThrow(() -> new ResourceNotFoundException("Registro no encontrado con ID: " + id));
            if (entity != null) {
                daoUzyTavTipres.delete(entity);
                return true;
            }
            return false;
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al eliminar el registro: " + ex.getMessage());
        }
    }
}
