package ec.edu.espe.movilidad.MovilidadWS.Service.UzyTavZonaDetalle;


import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzyTavZonaDetalle;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzyTavZonaDetalle;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;
import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzyTavZonaDetalleMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTavZonaDetalle;
import org.springframework.stereotype.Service;

import java.util.List;


@Service
public class ServiceUzyTavZonaDetalle implements IServiceUzyTavZonaDetalle {

    private final DaoUzyTavZonaDetalle daoUzyTavZonaDetalle;
    private final UzyTavZonaDetalleMapper mapper;

    private static final String MESSAGE = "El parámetro 'id' debe ser un valor positivo.";

    private static final String MESSAGE2 = "Error al listar los registros: ";


    public ServiceUzyTavZonaDetalle(DaoUzyTavZonaDetalle daoUzyTavZonaDetalle, UzyTavZonaDetalleMapper mapper) {
        this.daoUzyTavZonaDetalle = daoUzyTavZonaDetalle;
        this.mapper = mapper;

    }

    @Override
    public DtoUzyTavZonaDetalle ListarPorID(Long id) {
        if (id <= 0) {
            throw new IllegalArgumentException(MESSAGE);
        }
        try {
            ModelUzyTavZonaDetalle entity = daoUzyTavZonaDetalle.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
            return mapper.entityToDto(entity);
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) { // Manejo de cualquier error en el servidor
            throw new RuntimeException("Error al buscar el registro con ID: " + id);
        }
    }

    @Override
    public List<DtoUzyTavZonaDetalle> ListarRegistros() {
        try {
            List<ModelUzyTavZonaDetalle> entities = daoUzyTavZonaDetalle.findAll();
            return mapper.entitiesToDtos(entities);
        } catch (Exception ex) {
            throw new RuntimeException(MESSAGE2 + ex.getMessage());
        }
    }

    @Override
    public List<DtoUzyTavZonaDetalle> ListarRegistrosRelacionadosConPrograma(Long programaId) {
        if (programaId <= 0) {
            throw new IllegalArgumentException(MESSAGE);
        }
        try {
            List<ModelUzyTavZonaDetalle> entities = daoUzyTavZonaDetalle.obtenerZonaDetallePorPrograma(programaId);
            return mapper.entitiesToDtos(entities);
        } catch (IllegalArgumentException ex) {
            throw ex;
        }
        catch (Exception ex) {
            throw new RuntimeException(MESSAGE2 + ex.getMessage());
        }
    }



    @Override
    public List<DtoUzyTavZonaDetalle> ListarRegistrosRelacionadosConProyec(Long proyecID) {
        if (proyecID <= 0) {
            throw new IllegalArgumentException(MESSAGE);
        }
        try {
            List<ModelUzyTavZonaDetalle> entities = daoUzyTavZonaDetalle.obtenerZonaDetallePorProyec(proyecID);
            return mapper.entitiesToDtos(entities);
        } catch (IllegalArgumentException ex) {
            throw ex;
        }
        catch (Exception ex) {
            throw new RuntimeException(MESSAGE2 + ex.getMessage());
        }
    }


    @Override
    public DtoUzyTavZonaDetalle guardar(DtoUzyTavZonaDetalle dtoUzyTavZonaDetalle) {
        try {
            ModelUzyTavZonaDetalle entity = mapper.dtoToEntity(dtoUzyTavZonaDetalle);
            ModelUzyTavZonaDetalle nuevoEntity = daoUzyTavZonaDetalle.save(entity);
            return mapper.entityToDto(nuevoEntity);
        }
        catch (Exception ex) {
            throw new RuntimeException("Error al guardar el registro: " + ex.getMessage());
        }
    }

    @Override
    public DtoUzyTavZonaDetalle editar(Long id, DtoUzyTavZonaDetalle dtoUzyTavZonaDetalle) {
        if (id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {
            ModelUzyTavZonaDetalle entity = daoUzyTavZonaDetalle.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
            entity.setUzytavzona_objetivos(dtoUzyTavZonaDetalle.getUzytavzona_objetivos());

            ModelUzyTavZonaDetalle updatedEntity = daoUzyTavZonaDetalle.save(entity);
            return mapper.entityToDto(updatedEntity);
        } catch (Exception e) {
            throw new ResourceNotFoundException("Error al editar el registro: " + e.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        if (id == null || id <= 0) {
            throw new IllegalArgumentException("El ID del registro debe ser válido y mayor que cero.");
        }
        try {

            ModelUzyTavZonaDetalle entity = daoUzyTavZonaDetalle.findById(id).orElseThrow(() -> new ResourceNotFoundException("Registro no encontrado con ID: " + id));
            if (entity != null) {
                daoUzyTavZonaDetalle.delete(entity);
                return true;
            }
            return false;
        } catch (ResourceNotFoundException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new RuntimeException("Error al eliminar el registro: " + ex.getMessage());
        }
    }
}
