package ec.edu.espe.movilidad.MovilidadWS.Service.UzytavObjetivoPrograma;

import ec.edu.espe.movilidad.MovilidadWS.Dao.DaoUzytavObjetivoPrograma;
import ec.edu.espe.movilidad.MovilidadWS.Dto.DtoUzytavObjetivoPrograma;
import ec.edu.espe.movilidad.MovilidadWS.Exceptions.ResourceNotFoundException;

import ec.edu.espe.movilidad.MovilidadWS.Mapper.Components_Class.UzytavObjetivoProgramaMapper;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzytavObjetivoPrograma;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.List;

@Service
public class ServiceUzytavObjetivoPrograma implements IServiceUzytavObjetivoPrograma {


    private final DaoUzytavObjetivoPrograma daoUzytavObjetivoPrograma;
    private final UzytavObjetivoProgramaMapper mapper;

    public ServiceUzytavObjetivoPrograma(DaoUzytavObjetivoPrograma daoUzytavObjetivoPrograma, UzytavObjetivoProgramaMapper mapper) {
        this.daoUzytavObjetivoPrograma = daoUzytavObjetivoPrograma;

        this.mapper = mapper;
    }

    @Override
    public DtoUzytavObjetivoPrograma ListarPorID(Long id) {
        ModelUzytavObjetivoPrograma entity = daoUzytavObjetivoPrograma.findById(id)
                .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
        return mapper.entityToDto(entity);
    }

    @Override
    public List<DtoUzytavObjetivoPrograma> ListarRegistros() {
        List<ModelUzytavObjetivoPrograma> entities = daoUzytavObjetivoPrograma.findAll();
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public List<DtoUzytavObjetivoPrograma> obtenerObjetivoprogramaPorPrograma(Long programaId) {
        List<ModelUzytavObjetivoPrograma> entities = daoUzytavObjetivoPrograma.obtenerObjetivoprogramaPorPrograma(programaId);
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public List<DtoUzytavObjetivoPrograma> obtenerObjetivoprogramaPorProyecto(Long proyectID) {
        List<ModelUzytavObjetivoPrograma> entities = daoUzytavObjetivoPrograma.obtenerObjetivoprogramaPorProyecto(proyectID);
        return mapper.entitiesToDtos(entities);
    }

    @Override
    public DtoUzytavObjetivoPrograma guardar(DtoUzytavObjetivoPrograma dtoUzytavObjetivoPrograma) {
        ModelUzytavObjetivoPrograma entity = mapper.dtoToEntity(dtoUzytavObjetivoPrograma);
        ModelUzytavObjetivoPrograma nuevoEntity = daoUzytavObjetivoPrograma.save(entity);
        return mapper.entityToDto(nuevoEntity);
    }

    @Override
    public DtoUzytavObjetivoPrograma editar(Long id, DtoUzytavObjetivoPrograma dtoUzytavObjetivoPrograma) {
        try {
            ModelUzytavObjetivoPrograma entity = daoUzytavObjetivoPrograma.findById(id).get();
            //Seteamos los nuevos datos del registro
            if (entity != null) {

                if (dtoUzytavObjetivoPrograma.getUzytavobjetivo_programaindicador() != null) {
                    entity.setUzytavobjetivo_programaindicador(dtoUzytavObjetivoPrograma.getUzytavobjetivo_programaindicador());
                }
                if (dtoUzytavObjetivoPrograma.getUzytavobjetivo_programamverifica() != null) {
                    entity.setUzytavobjetivo_programamverifica(dtoUzytavObjetivoPrograma.getUzytavobjetivo_programamverifica());
                }
                if (dtoUzytavObjetivoPrograma.getUzytavobjetivo_programasupuestos() != null) {
                    entity.setUzytavobjetivo_programasupuestos(dtoUzytavObjetivoPrograma.getUzytavobjetivo_programasupuestos());
                }
                if (dtoUzytavObjetivoPrograma.getUzytavobjetivo_programaobjetivo_proy() != null) {
                    entity.setUzytavobjetivo_programaobjetivo_proy(dtoUzytavObjetivoPrograma.getUzytavobjetivo_programaobjetivo_proy());
                }
                if (dtoUzytavObjetivoPrograma.getUzytavobjetivo_programatipo() != null) {
                    entity.setUzytavobjetivo_programatipo(dtoUzytavObjetivoPrograma.getUzytavobjetivo_programatipo());
                }
                if (dtoUzytavObjetivoPrograma.getUzytavobjetivo_programaorden() != null) {
                    entity.setUzytavobjetivo_programaorden(dtoUzytavObjetivoPrograma.getUzytavobjetivo_programaorden());
                }
                if (dtoUzytavObjetivoPrograma.getUzytavobjetivo_programafec_ini() != null) {
                    entity.setUzytavobjetivo_programafec_ini(dtoUzytavObjetivoPrograma.getUzytavobjetivo_programafec_ini());
                }
                if (dtoUzytavObjetivoPrograma.getUzytavobjetivo_programafec_fin() != null) {
                    entity.setUzytavobjetivo_programafec_fin(dtoUzytavObjetivoPrograma.getUzytavobjetivo_programafec_fin());
                }
                if (dtoUzytavObjetivoPrograma.getUzytavobjetivo_programaindicador() != null) {
                    entity.setUzytavobjetivo_programaindicador(dtoUzytavObjetivoPrograma.getUzytavobjetivo_programaindicador());
                }
                if (dtoUzytavObjetivoPrograma.getUzytavobjetivo_programaindicador_valor() != null) {
                    entity.setUzytavobjetivo_programaindicador_valor(dtoUzytavObjetivoPrograma.getUzytavobjetivo_programaindicador_valor());
                }
                if (dtoUzytavObjetivoPrograma.getUzytavobjetivo_programaindicador_valor_eje() != null) {
                    entity.setUzytavobjetivo_programaindicador_valor_eje(dtoUzytavObjetivoPrograma.getUzytavobjetivo_programaindicador_valor_eje());
                }

                ModelUzytavObjetivoPrograma updatedEntity = daoUzytavObjetivoPrograma.save(entity);
                return mapper.entityToDto(updatedEntity);
            } else {
                throw new ResourceNotFoundException("No se encontró el registro con ID: " + id);
            }
        } catch (Exception e) {
            throw new ResourceNotFoundException("Error al editar el registro: " + e.getMessage());
        }
    }

    @Override
    public boolean eliminar(Long id) {
        ModelUzytavObjetivoPrograma entity = daoUzytavObjetivoPrograma.findById(id).orElse(null);
        if (entity != null) {
            daoUzytavObjetivoPrograma.delete(entity);
            return true;
        }
        return false;
    }

    @Override
    public DtoUzytavObjetivoPrograma eliminarFechasPorId(Long id) {
        ModelUzytavObjetivoPrograma entity = daoUzytavObjetivoPrograma.findById(id)
                .orElseThrow(() -> new ResourceNotFoundException("No se encontró el registro con ID: " + id));
        entity.setUzytavobjetivo_programafec_ini(null);
        entity.setUzytavobjetivo_programafec_fin(null);

        ModelUzytavObjetivoPrograma updatedEntity = daoUzytavObjetivoPrograma.save(entity);
        return mapper.entityToDto(updatedEntity);
    }
    @Override
    @Transactional
    public boolean eliminarHijo(Long idHijo) {
        ModelUzytavObjetivoPrograma hijo = daoUzytavObjetivoPrograma.findById(idHijo).orElse(null);

        if (hijo != null) {
            ModelUzytavObjetivoPrograma padre = hijo.getUzytavobjetivo_programaPadre();
            if (padre != null) {
                padre.getModelUzytavObjetivoProgramaSet().remove(hijo);
                daoUzytavObjetivoPrograma.delete(hijo);
                return true;
            }
        }
        return false;
    }

}
