package ec.edu.espe.movilidad.MovilidadWS.Dao;

import static org.junit.jupiter.api.Assertions.*;

import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTPerfil;
import ec.edu.espe.movilidad.MovilidadWS.Model.ModelUzyTUsuario;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import java.util.List;
import java.util.Set;

@SpringBootTest
 class DaoUzyTUsuarioTests {

    @Autowired
    private DaoUzyTUsuario daoUzyTUsuario;


    @BeforeEach
    void setUp() {
    }
    @DisplayName("Test buscar un usuario")
    @Test
    void testfindByUsuario() {

        String usuarioBuscado = "JOEL";

        List<ModelUzyTUsuario> result = daoUzyTUsuario.findByUsuario(usuarioBuscado);

        // Verificar que el resultado no esté vacío
        assertFalse(result.isEmpty());

        // Verificar que el nombre o apellido del usuario coincida con el usuario buscado
        for (ModelUzyTUsuario usuario : result) {
            assertTrue(usuario.getUzytusuario_nombres().toLowerCase().contains(usuarioBuscado.toLowerCase())
                    || usuario.getUzytusuario_apellidos().toLowerCase().contains(usuarioBuscado.toLowerCase()));
        }
    }


    @DisplayName("Test buscar un usuario por el email institucional")
    @Test
    void testfindByEmail() {
        String usuarioEmailBuscado = "jamolina13@espe.edu.ec";

        ModelUzyTUsuario result = daoUzyTUsuario.findByEmail(usuarioEmailBuscado);

        // Verificar que el resultado no sea nulo
        assertNotNull(result);

        // Verificar que el email del usuario coincida con el email buscado
        assertEquals(usuarioEmailBuscado, result.getUzytusuario_email_institucional());
    }


    @DisplayName("Test buscar un los perfiles asignados al usuario")
    @Test
    void testfindPerfilesByUsuarioId() {
        Long usuarioID = 7L;

        Set<ModelUzyTPerfil> result = daoUzyTUsuario.findPerfilesByUsuarioId(usuarioID);

        // Verificar que el resultado no sea nulo y no este vacío
        assertNotNull(result);
        assertFalse(result.isEmpty());

        String nombrePerfilBuscado = "Director de Programa";
        boolean perfilEncontrado = result.stream()
                .anyMatch(perfil -> nombrePerfilBuscado.equalsIgnoreCase(perfil.getUzytperfil_nombre()));
        assertTrue(perfilEncontrado);
    }

}

